
import sys
import traceback
import os
from time import *
from string import *

class TestCase:
	"Base class for python based unit level and regression tests. \
To use this class, implement the runTest, initialize, and reset  \
methods.  Call the assert_ method to test for success."
	
	def __init__(self, name):
		self.testName = name
		self.failed = 1
		self.failureReason = "<<Not Tested>>"
		
	def runTest(self):
		"Run the test methods in this class"
		names = dir( self.__class__ )
		self.failed = 0
		self.failureReason = ""
		for name in names:
			# if this is a test method
			if name[:5] == "test_":
				
				self.initialize()
				try:
					func = getattr( self, name )
					self.writeMessage( func.__doc__ )
					# Execute the test
					func()
				finally:
					self.reset()
					
	def initialize(self):
		"Called before each test is run.  Should be overridden."
		pass
		
	def reset(self):
		"Called after each test is run.  Should be overridden."
		pass

	def failTest(self, reason):
		"Mark this test as having failed."
		self.failed = 1
		self.failureReason = reason
		
	def writeMessage(self, message):
		"Writes a message to the standard output device."
		self.__writeMessage(sys.stdout, message )
		
	def writeError(self, error):
		"Writes an error to the standard output device."
		self.__writeMessage( sys.stdout, "ERR " + error )
		
	def writeWarning(self, warning):
		"Writes an warning to the standard output device."
		self.__writeMessage( sys.stdout, "WRN " + warning )
		
	def __writeMessage(self, outFile, message):
		
		year, month, day, hour, minute, second, weekday, julian, daylight = \
			localtime(time())
		outFile.write( "%04d%02d%02d %02d:%02d:%02d\t" % (year, month, day, hour, minute, second) )
		outFile.write( "%s\t%s\n" % (self.testName, message) )

	def assert_(self, test):
		"Logs and generates an error if the test is false."
		if test == 0:

			stack = traceback.extract_stack()
			stackFrame = stack[len(stack)-2]
			file,line,method,call = stackFrame
			path,file = os.path.split(file)
			source = "%s %s %s %s" % (file, line, method, call)
			self.failTest( "Assertion failed: %s" % source )
			self.writeError( self.failureReason )

			raise AssertionError, source
		