#!/usr/bin/python

""" 
usage: testeng.py [--<testname>]

--<testname>: name of test file to include in the test.
"""

import string
import os
from testsuite import TestSuite

def getTests( scriptDir = os.curdir ):
	
	# Get the test files available to run
	testFiles = os.listdir( scriptDir )
	scripts = []
	for script in testFiles:
		script, ext = os.path.splitext(script)
		
		if string.lower(ext) == ".py" and string.lower(script)[:4] != "test":
			scripts.append( script)

	return scripts
		
if __name__ == "__main__":

	import sys
	import getopt
	
	scripts = []
	
	# Print usage message and exit
	def usage(*args):
		global scripts
		
		sys.stdout = sys.stderr
		for msg in args: print msg
		print __doc__
		for script in scripts:
			print "--" + script
			
		sys.exit(2)
	
	# title
	# Display the application title
	def title():
		print "TestEngine 1.0 for Python " + sys.platform

	# Main program: parse command line and start processing
	def main():
		global scripts

		# Save the current value of the standard output stream to write
		# messages we don't want redirected to the output file
		console = sys.stdout
		
		title()
	
		scripts = getTests()
		
		# Process the command line
		try:
			opts, args = getopt.getopt(sys.argv[1:], '', scripts)
						
		except getopt.error, msg:
			usage(msg)
	
		if args: usage("invalid options " + str(args))
		for o, a in opts:
			if o[:2] == '--': scripts = [o[2:]]

		# If there are any tests to run
		if len(scripts):

			ts = TestSuite()
			
			for script in scripts:
		
				print "Loading %s" % script
				exec( "import %s" % script )
				module = locals()[script]
				# if this module supports our testing interface
				if module.__dict__.has_key( "addTests" ):
					module.addTests( ts )

			ts.run()
		
	main()