
""" 
TestSuite.py -- Python class to run a collection of TestCase unit level 
tests.

"""

import sys, traceback

class TestSuite:
	"Runs a collection of TestCase based objects."
	
	def __init__(self):
		self.tests = []
		
	def add(self, test):
		"Adds a new test to this suite."
		self.tests.append( test )
		
	def remove(self, test):
		"Removes a previously added test from this suite."
		n = 0
		while n < len(self.tests):
			if self.tests[n].testName == test.testName:
				del self.tests[n]
				break
			n = n + 1
		
	def count(self):
		"Returns the number of tests in this suite."
		return len(self.tests)
		
	def runTest(self, test):
		"Runs the given unit test."
		
		test.failed = 0
		try:
		
			test.runTest()

		except AssertionError, source:
			pass
		except:
			# Get the exception and write it to the standard output
			type, value, tb = sys.exc_info()
			tb = None
			test.writeError( "Unexpected exception: %s\t%s" % (type, value) )
			test.failTest( "Unexpected exception: %s\t%s" % (type, value) )
			traceback.print_exc()

	def writeMessage(self, message):
		"Writes a message to the standard output device."
		self.__writeMessage(sys.stdout, message )
		
	def writeError(self, error):
		"Writes an error to the standard output device."
		self.__writeMessage( sys.stdout, "ERR " + error )
		
	def writeWarning(self, warning):
		"Writes an warning to the standard output device."
		self.__writeMessage( sys.stdout, "WRN " + warning )
		
	def __writeMessage(self, outfile, message):
		
		outfile.write( "%s\n" % (message) )

	def run(self):
		"Runs all the tests in the suite."
			
		currentTest = None
		
		for currentTest in self.tests:

			self.runTest( currentTest )

			if currentTest.failed:
				self.writeMessage( "%s\tFAILED" % (currentTest.testName) )
			else:
				self.writeMessage( "%s\tPASSED" % (currentTest.testName) )
