/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/
/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */


#include "ArithSup.h"
#include <iostream.h>

Arith::Arith(int minargs_, int maxargs_,
			Steppable *arg1,Steppable *arg2,Steppable *arg3,
			Steppable *arg4,Steppable *arg5,Steppable *arg6,
			Steppable *arg7,Steppable *arg8,Steppable *arg9)
			:minargs(minargs_),maxargs(maxargs_)
{
	assert(minargs >=0 && maxargs>=-1);		// -1 indicates ANY number of args
	assert(maxargs==-1 || maxargs>=minargs); 
	Append(arg1); Append(arg2); Append(arg3);
	Append(arg4); Append(arg5); Append(arg6);
	Append(arg7); Append(arg8); Append(arg9);
	done.Set(FUZZTRUE);
}

void Arith::Append(Steppable *arg_)
{
	if (arg_!=NULL) arg_list.Append(Getport(arg_));
}

float Arith::GetNumber(Steppable *arg_)
{
	if (arg_==NULL) return 0.0;
	Data *ret = arg_->Step();
	if (ret==NULL)  return 0.0;
	if (ret->type==DT_Number) return ((Number*)ret)->Get();
	if (ret->type==DT_Fuzz) return ((Fuzz*)ret)->Get();
	return 0.0;
}

Data *Arith::Step(void)
{
	int    ls   = arg_list.Size();
	float  res = 1.23456789; // initialize randomly
	result.Set(res);
	Steppable *arg1=NULL,*arg2=NULL;
	
	// wrong number of arguments in argument list ?
	assert_w(!(ls<minargs || (maxargs!=-1 && ls>maxargs))); // report that defensive programming was necessary
	
	arg_list.Reset();
	switch (ls) {
		case 0:
			result.Set(Eval1(0.1111111,0.2222222));
			break;
		case 1:
			arg1 = (Steppable*)arg_list.Iterate();
			result.Set(Eval1(GetNumber(arg1),0.3333333));
			break;
		default:
			arg1 = (Steppable*)arg_list.Iterate();
			arg2 = (Steppable*)arg_list.Iterate();
			res = Eval1(GetNumber(arg1),GetNumber(arg2));
			if (maxargs>2 || maxargs==-1)
				while (NULL!=(arg2=(Steppable*)arg_list.Iterate()))
					res = Eval1(res,GetNumber(arg2));
			result.Set(res);
			break;
	};
	
	return &result;
}

////////////////////////////////////////////////////////////
//
//  ComparArith
//

Data *ComparArith::Step(void)
{
	int    ls   = arg_list.Size();
	fresult.Set(0.666666666);
	Steppable *arg1=NULL,*arg2=NULL;
	
	arg_list.Reset();
	switch (ls) {
		case 1:
			arg1 = (Steppable*)arg_list.Iterate();
			fresult.Set(Eval1(GetNumber(arg1),0.3434343434343));
			break;
		case 2:
			arg1 = (Steppable*)arg_list.Iterate();
			arg2 = (Steppable*)arg_list.Iterate();
			fresult.Set(Eval1(GetNumber(arg1),GetNumber(arg2)));
			break;
		default:
			fresult.Set(FUZZFALSE);	// this case should never occur
			assert_w(arg1);			// this line should never be reached !!
			break;
	};
	
	return &fresult;
}
