/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _ArithSup_h_
#define _ArithSup_h_

/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include "PrgStruct.h"
#include "RobotProxy.h"
#include "math.h"

/////////////////////////////////////////////////////////////
//
//    Arithmetic SUPs
//

class Arith : public Action {
public:
	Arith(	int minargs_, int maxargs_,
			Steppable *arg1 = NULL,
			Steppable *arg2 = NULL,
			Steppable *arg3 = NULL,
			Steppable *arg4 = NULL,
			Steppable *arg5 = NULL,
			Steppable *arg6 = NULL,
			Steppable *arg7 = NULL,
			Steppable *arg8 = NULL,
			Steppable *arg9 = NULL);
	virtual	Data	*Step(void);
private:
	void 	Append(Steppable *arg_);
	Number	result;
	int		minargs,maxargs;// number of arguments wanted (-1 is any number)
protected:	
	float	GetNumber(Steppable *arg_);
	List	arg_list;		// arguments list
	virtual float	Eval1(float a, float b) =0;
};

class Sub : public Arith {
public:
	Sub(	Steppable *arg1,
			Steppable *arg2,
			Steppable *arg3 = NULL,
			Steppable *arg4 = NULL,
			Steppable *arg5 = NULL,
			Steppable *arg6 = NULL,
			Steppable *arg7 = NULL,
			Steppable *arg8 = NULL,
			Steppable *arg9 = NULL)
			:Arith(2,-1,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9){};
protected:
	virtual float	Eval1(float a, float b){return (a-b);};
};

class Add : public Arith {
public:
	Add(	Steppable *arg1,
			Steppable *arg2,
			Steppable *arg3 = NULL,
			Steppable *arg4 = NULL,
			Steppable *arg5 = NULL,
			Steppable *arg6 = NULL,
			Steppable *arg7 = NULL,
			Steppable *arg8 = NULL,
			Steppable *arg9 = NULL)
			:Arith(2,-1,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9){};
protected:
	virtual float	Eval1(float a, float b){return (a+b);};
};

class Mul : public Arith {
public:
	Mul(	Steppable *arg1,
			Steppable *arg2,
			Steppable *arg3 = NULL,
			Steppable *arg4 = NULL,
			Steppable *arg5 = NULL,
			Steppable *arg6 = NULL,
			Steppable *arg7 = NULL,
			Steppable *arg8 = NULL,
			Steppable *arg9 = NULL)
			:Arith(2,-1,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9){};
protected:
	virtual float	Eval1(float a, float b){return (a*b);};
};

class Div : public Arith {
public:
	Div(	Steppable *arg1,
			Steppable *arg2,
			Steppable *arg3 = NULL,
			Steppable *arg4 = NULL,
			Steppable *arg5 = NULL,
			Steppable *arg6 = NULL,
			Steppable *arg7 = NULL,
			Steppable *arg8 = NULL,
			Steppable *arg9 = NULL)
			:Arith(2,-1,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9){};
protected:
	virtual float	Eval1(float a, float b){return (a/b);};
};

class Abs : public Arith {
public:
	Abs(Steppable *arg1):Arith(1,1,arg1){};
protected:
	virtual float	Eval1(float a, float b){b=b;return (fabs(a));};	// b=b silences warnings of compiler
};

class Sin : public Arith {
public:
	Sin(Steppable *arg1):Arith(1,1,arg1){};
protected:
	virtual float	Eval1(float a, float b){b=b;return (sin(a));};
};

class Asin : public Arith {
public:
	Asin(Steppable *arg1):Arith(1,1,arg1){};
protected:
	virtual float	Eval1(float a, float b){b=b;return (asin(a));};
};

class Cos : public Arith {
public:
	Cos(Steppable *arg1):Arith(1,1,arg1){};
protected:
	virtual float	Eval1(float a, float b){b=b;return (cos(a));};
};

class Acos : public Arith {
public:
	Acos(Steppable *arg1):Arith(1,1,arg1){};
protected:
	virtual float	Eval1(float a, float b){b=b;return (acos(a));};
};

class Tan : public Arith {
public:
	Tan(Steppable *arg1):Arith(1,1,arg1){};
protected:
	virtual float	Eval1(float a, float b){b=b;return (tan(a));};
};

class Atan : public Arith {
public:
	Atan(Steppable *arg1):Arith(1,1,arg1){};
protected:
	virtual float	Eval1(float a, float b){b=b;return (atan(a));};
};

////////////////////////////////////////////////////////////////////
//
//    Boolean Logic Operations 
//

class ComparArith : public Arith {
public:
	ComparArith(int nargs, Steppable *arg1=NULL, Steppable *arg2=NULL)
		:Arith(nargs,nargs,arg1,arg2)
		{assert_w((nargs==1 || nargs==2) && (nargs==1 || arg2));};
	virtual	Data	*Step(void);
private:	
	Fuzz	fresult;
};

class Gt : public ComparArith {
public:
	Gt(Steppable *arg1, Steppable *arg2):ComparArith(2,arg1,arg2){};
protected:
	virtual float	Eval1(float a, float b){return ((a>b)?FUZZTRUE:FUZZFALSE);};
};

class Lt : public ComparArith {
public:
	Lt(Steppable *arg1, Steppable *arg2):ComparArith(2,arg1,arg2){};
protected:
	virtual float	Eval1(float a, float b){return ((a<b)?FUZZTRUE:FUZZFALSE);};
};

class Ge : public ComparArith {
public:
	Ge(Steppable *arg1, Steppable *arg2):ComparArith(2,arg1,arg2){};
protected:
	virtual float	Eval1(float a, float b){return ((a>=b)?FUZZTRUE:FUZZFALSE);};
};

class Le : public ComparArith {
public:
	Le(Steppable *arg1, Steppable *arg2):ComparArith(2,arg1,arg2){};
protected:
	virtual float	Eval1(float a, float b){return ((a<=b)?FUZZTRUE:FUZZFALSE);};
};

class Eq : public ComparArith {
public:
	Eq(Steppable *arg1, Steppable *arg2):ComparArith(2,arg1,arg2){};
protected:
	virtual float	Eval1(float a, float b){return ((a==b)?FUZZTRUE:FUZZFALSE);};
};

class AboutEq : public ComparArith {
public:
	AboutEq(Steppable *arg1, Steppable *arg2, float tolerance_=0.01)
		:tolerance(tolerance_),ComparArith(2,arg1,arg2){};
protected:
	virtual float	Eval1(float a, float b)
		{return ((fabs(a-b)<=tolerance)?FUZZTRUE:FUZZFALSE);};
	float tolerance;	// may differ by that much	
};

#endif	/* _ArithSup_h_ */
