/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/
/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */


#include <stdio.h>
#include <assert.h>
#include <windows/Qwindows.h>
#include <string.h>
#include <stdlib.h>
#include "CImage.h"
#include "LocalMap.h"

#define MAX_WINDOW_HEIGHT	20


//////////////////////////////////////////////////////////////////////
//  Constructors/Destructors management functions
//////////////////////////////////////////////////////////////////////

CImage::CImage(int width_, int height_)
{
	map_width 	= width_;
	map_height	= height_;
	map_size  	= (long)map_width * (long)map_height;
	int sum_size= map_width*(map_height+MAX_WINDOW_HEIGHT);
	map = new unsigned short [map_size];
	sum = new unsigned int [sum_size];
	sum2= new unsigned int [sum_size];
	assert(map!=0);
	unsigned short *ptr=map;
	for (int i=0;i<map_size;i++) {*ptr=0; ptr++;}
}

CImage::CImage(CImage & original)
{
	map_width 	= original.map_width;
	map_height	= original.map_height;
	map_size  	= original.map_size;
	int sum_size= map_width*(map_height+MAX_WINDOW_HEIGHT);
	map = new unsigned short [map_size];
	sum = new unsigned int [sum_size];
	sum2= new unsigned int [sum_size];
	assert(map!=0);
	unsigned short *ptr=map;
	for (int i=0;i<map_size;i++) {*ptr=0; ptr++;}
}

CImage::~CImage()
{
	delete map;
	delete sum;
	delete sum2;
}


//////////////////////////////////////////////////////////////////////
//  Storage management functions
//////////////////////////////////////////////////////////////////////

void CImage::Reallocate(int width_,int height_)
{
	if ((map_width==width_) && (map_height==height_)) return;
	
	// enter new size
	map_width = width_;
	map_height= height_;
	
	// reallocate buffer map if too small
	if (map_width*map_height>map_size) {
		delete map;
		delete sum;
		delete sum2;
		map_size  	= (long)map_width * (long)map_height;
		int sum_size= map_width*(map_height+MAX_WINDOW_HEIGHT);
		map = new unsigned short [map_size];
		sum = new unsigned int [sum_size];
		sum2= new unsigned int [sum_size];
	}	
}

word	CImage::Marshal(byte *buf)
/* format assumed: 5 bits, 0..31 */
/* format generated: 2 x 4 bits in each byte of buf, LSB of each pixel is lost */
{
	unsigned short *ptr, *endp;
	word len=0;
	word w = (word)map_width;
	word h = (word)map_height;
	word l = (word)(w*h);
	
	ADDWBUF(w);
	ADDWBUF(h);
	for (ptr=map, endp = map+l;ptr<endp;ptr+=2) {
		*buf++ = (byte)((((*ptr&0x001F)<<3)&0x00F0)|((*(ptr+1)>>1)&0x000F));
		len++;
	}
	return len;
}

word	CImage::Unmarshal(byte *buf)
/* format generated: 5 bits, colors 0..31 */
/* format consumed:  2 x 4 bits 0..15 in each byte of buf, LSB of each pixel is 0 */
{
	word len=0;
	unsigned short *ptr, *endp;
	word w,h,l;
	
	GETWBUF(w,buf); 
	GETWBUF(h,buf); 
	l = (word)(w*h);

	Reallocate(w,h);
	
	for (ptr=map, endp = map+l;ptr<endp;ptr+=2,buf++,len++) {
		*ptr     =  (unsigned short)((*buf & 0xF0)>>3);
		*(ptr+1) =  (unsigned short)((*buf & 0x0F)<<1);
	}
	return len;	
}

CImage & CImage::Erase(void)	// fill with zeroes
{
	int i,j;
	
	unsigned short *gbuf = map;

	for (i = 0; i < map_height; i++)  
		for (j = 0; j < map_width; j++) {
			*gbuf = 0;
			gbuf++;   
		}
		
	return *this;
}


//////////////////////////////////////////////////////////////////////
//  Display functions
//////////////////////////////////////////////////////////////////////

void CImage::Show(int x, int y)
{
	if (YES!=PixelMap(x,y,map_width,map_height,(short*)map))
		printf("CImage::Show : Error drawing pixels\n");
}

//////////////////////////////////////////////////////////////////////
//  Handling functions
//////////////////////////////////////////////////////////////////////

CImage & CImage::operator = (CImage & original)
{
	Reallocate(original.map_width,original.map_height);
	memcpy(map,original.map,map_size*sizeof(short)); // short ints are copied		
	return *this;
}

//////////////////////////////////////////////////////////////////////
//  Filter functions
//////////////////////////////////////////////////////////////////////

CImage &  CImage::MakeGray4(CImage &original)
/* rgb555 format assumed */
{
	// compute new size of image
	Reallocate(original.map_width,original.map_height);

	int i,j;
	
	unsigned short *cbuf = original.map;
	unsigned short *gbuf = map;

	for (i = 0; i < map_height; i++)  
		for (j = 0; j < map_width; j++) {
			*gbuf = (unsigned short)(((*cbuf>>6) & 15)+1);/* reduce to 16 colors == 4 bits */
			gbuf++;                     /* green channel is used only */
			cbuf++;						/* colors 1..16 for screen use */
		}
		
	return *this;
}

CImage &  CImage::MakeGray5(CImage &original, unsigned int forget)
/* rgb555 format assumed */
{
	// compute new size of image
	Reallocate(original.map_width-forget,original.map_height);

	int i,j;
	
	unsigned short *cbuf = original.map;
	unsigned short *gbuf = map;

	for (i = 0; i < map_height; i++)  {
		for (j = 0; j < map_width; j++) {
			*gbuf = (unsigned short)((*cbuf>>5) & 31); /* reduce to 32 colors == 5 bits */
			gbuf++;                    /* green channel is used only */
			cbuf++;					   /* colors 0..31 */
		}
		cbuf+=forget;
	}
		
	return *this;
}

CImage & CImage::MakeGray4fromOcc(LocalMap & occ)
/* 0..15 format assumed */
{
	// compute new size of image
	Reallocate(LMSIZE_CX,LMSIZE_CY);

	unsigned char  *cell = (unsigned char*) occ.ccell;	
	unsigned short *gbuf = map;

	for (int i = 0; i < map_height; i++)  
		for (int j = 0; j < map_width; j++) {
			*gbuf = (unsigned short)(16-((*cell) & 15));/* reduce to 8 colors == 4 bits & invert */
			gbuf++;                     
			cell++;			
		}
		
	return *this;
}

CImage &  CImage::MakeGray4from5(CImage &original)
/* 0..31 color format assumed */
{
	// compute new size of image
	Reallocate(original.map_width,original.map_height);

	int i,j;
	
	unsigned short *cbuf = original.map;
	unsigned short *gbuf = map;

	for (i = 0; i < map_height; i++)  
		for (j = 0; j < map_width; j++) {
			*gbuf = (unsigned short)(((*cbuf>>1) & 15)+1);/* reduce to 16 colors == 4 bits */
			gbuf++;                     
			cbuf++;					    /* colors 1..16 */
		}
		
	return *this;
}

CImage &  CImage::HalfSize(CImage &original)
/* no particular format assumed */
{
	// compute new size of image
	Reallocate(original.map_width/2,original.map_height/2);

	int i,j;

	// assign tmp pointers
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;

	// copy pixels from original
	for (i = 0; i < map_height; i++) { 
		ibuf = &original.map[original.map_width*i*2];
		for (j = 0; j < map_width; j++) {
			*obuf = *ibuf;
			obuf++;  
			ibuf+=2;    
		}
	}
		
	// return yourself
	return *this;
}

CImage &  CImage::QuarterSize(CImage &original)
/* no particular format assumed */
{
	// compute new size of image
	Reallocate(original.map_width/4,original.map_height/4);

	int i,j;

	// assign tmp pointers
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;

	// copy pixels from original
	for (i = 0; i < map_height; i++) { 
		ibuf = &original.map[original.map_width*i*4];
		for (j = 0; j < map_width; j++) {
			*obuf = *ibuf;
			obuf++;  
			ibuf+=4;    
		}
	}
		
	// return yourself
	return *this;
}

CImage &  CImage::Shrink(CImage &original, unsigned int factor)
/* no particular format assumed */
{
	// compute new size of image
	Reallocate(original.map_width/factor,original.map_height/factor);

	int i,j;

	// assign tmp pointers
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;

	// copy pixels from original
	for (i = 0; i < map_height; i++) { 
		ibuf = &original.map[original.map_width*i*factor];
		for (j = 0; j < map_width; j++) {
			*obuf = *ibuf;
			obuf++;  
			ibuf+=factor;    
		}
	}
		
	// return yourself
	return *this;
}

CImage &  CImage::Threshold(CImage &original, unsigned short thres)
/* gray level image format assumed */
{
	// compute new size of image
	Reallocate(original.map_width,original.map_height);

	int i,j;

	// temp pointers assigned
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;

	for (i = 0; i < map_height; i++)  
		for (j = 0; j < map_width; j++) {
			*obuf = (*ibuf>=thres)?15:1; 
			ibuf++;                     
			obuf++;
		}
		
	// return yourself
	return *this;
}

CImage & CImage::Sum2(CImage &original_1, CImage &original_2)
{
	// compute new size of image
	Reallocate(original_1.map_width,original_1.map_height);

	int i,j;

	// temp pointers assigned
	unsigned short *ibuf1 = original_1.map;
	unsigned short *ibuf2 = original_2.map;
	unsigned short *obuf = map;

	for (i = 0; i < map_height; i++)  
		for (j = 0; j < map_width; j++) {
			*obuf = (unsigned short)(((*ibuf1)+(*ibuf2))>>1);
			ibuf1++;                    
			ibuf2++;                    
			obuf++;
		}
		
	// return yourself
	return *this;
}

CImage & CImage::ADiff(CImage &original_1, CImage &original_2)
{
	// compute new size of image
	Reallocate(original_1.map_width,original_1.map_height);

	int i,j;

	// temp pointers assigned
	unsigned short *ibuf1 = original_1.map;
	unsigned short *ibuf2 = original_2.map;
	unsigned short *obuf = map;

	for (i = 0; i < map_height; i++)  
		for (j = 0; j < map_width; j++) {
			*obuf = (unsigned short)abs((int)*ibuf1-(int)*ibuf2);
			ibuf1++;                    
			ibuf2++;                    
			obuf++;
		}
		
	// return yourself
	return *this;
}

CImage & CImage::ADiffMix(CImage &original_1, CImage &original_2, unsigned short thres)
/* format: 5 bits gray */
{
	// compute new size of image
	Reallocate(original_1.map_width,original_1.map_height);

	int i,j;

	// temp pointers assigned
	unsigned short *ibuf1 = original_1.map;
	unsigned short *ibuf2 = original_2.map;
	unsigned short *obuf = map;

	for (i = 0; i < map_height; i++)  
		for (j = 0; j < map_width; j++) {
			*obuf = (unsigned short)((((unsigned short)abs((int)*ibuf1-(int)*ibuf2))>=thres)?
					31:  		// max value, if diff bigger than threshold
					(*ibuf1)/2); // else original value/2
			ibuf1++;                    
			ibuf2++;                    
			obuf++;
		}
		
	// return yourself
	return *this;
}

CImage & CImage::ADiffMixMax(CImage &original_1, CImage &original_2, unsigned short thres)
/* format 5 bits gray, 0..31 */
{
	// compute new size of image
	Reallocate(original_1.map_width,original_1.map_height);

	int i,j;

	// temp pointers assigned
	unsigned short *ibuf1 = original_1.map;
	unsigned short *ibuf2 = original_2.map;
	unsigned short *obuf = map;

	for (i = 0; i < map_height; i++)  
		for (j = 0; j < map_width; j++) {
			if (((unsigned short)abs((int)*ibuf1-(int)*ibuf2))>=thres) {
				*obuf = 31;		// max value, if diff bigger than threshold
			} else {			// else color--
				int i = (int)(*ibuf1)-1;
				*obuf = (unsigned short)((i<0)?0:i);
			}
			ibuf1++;                    
			ibuf2++;                    
			obuf++;
		}
		
	// return yourself
	return *this;
}

CImage & CImage::BoundingBox(CImage &original, 
				unsigned int &xmin, unsigned int &xmax,
				unsigned int &ymin, unsigned int &ymax,
				signed int &xctr, signed int &yctr,
				boolean	&exists,
				unsigned short thres)
{
	// compute new size of image
	Reallocate(original.map_width,original.map_height);
	
	int i,j;

	// temp pointers assigned
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;
	
	xmin	=	map_width-1;
	xmax	=	0;
	ymin	=	map_height-1;
	ymax	=	0;

	// find the bounding box
	for (i = 0; i < map_height; i++) {
		for (j = 0; j < map_width; j++) {
			if ((*ibuf)>=thres) {
				if (i < ymin)	ymin = i;
				if (i > ymax)	ymax = i;
				if (j < xmin)   xmin = j;
				if (j > xmax)	xmax = j;
				*obuf = 25;			// make clear, that it is above threshold
			} else
				*obuf = *ibuf;		// copy to output image
			ibuf++;
			obuf++;
		}
	}
	
	exists 	= (xmax>=xmin);
	xctr	= ((signed int)map_width  - ((signed int)(xmax+xmin)))/2;
	yctr	= ((signed int)map_height - ((signed int)(ymax+ymin)))/2;
	
	if (exists) {
		// draw the bounding box
		unsigned short *pf;
		unsigned short *pt;
		obuf = map;

		// upper, horizontal line
		pf = map + map_width*ymin + xmin;	// left starting point
		pt = map + map_width*ymin + xmax;	// right ending point
		while (pf<=pt) {
			*pf = 31;
			pf++;
		}

		// lower, horizontal line
		pf = map + map_width*ymax + xmin;	// left starting point
		pt = map + map_width*ymax + xmax;	// right ending point
		while (pf<=pt) {
			*pf = 31;
			pf++;
		}

		// left, vertical line
		pf = map + map_width*ymin + xmin;	// upper starting point
		pt = map + map_width*ymax + xmin;	// lower ending point
		while (pf<=pt) {
			*pf = 31;
			pf+=map_width;
		}

		// right, vertical line
		pf = map + map_width*ymin + xmax;	// upper starting point
		pt = map + map_width*ymax + xmax;	// lower ending point
		while (pf<=pt) {
			*pf = 31;
			pf+=map_width;
		}

	}
	
	return *this;
}


CImage & CImage::HorizontalAvg(CImage &original,unsigned short window) /* Filter out floor texture */
{
	// compute new size of image
	Reallocate(original.map_width,original.map_height);

	int i,j,k;

	// temp pointers assigned
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;

	for (i = 0; i < map_height; i++) {
		unsigned short *tmp_j=ibuf+i*map_width;
		unsigned short linep=(unsigned short)(i*map_width);
		for (j = map_width-1; j>=window; j--)  {
			unsigned short sum=0;
			for (k=j; k>=j-window;k--) {
				sum += *(tmp_j+k);
			}
			*(obuf+j+linep) = (unsigned short)(sum / window);
		}
	}
	
//  This is the code for vertical low pass
//	for (j = 0; j < map_width; j++) {
//		unsigned short reference,sum=0;
//		unsigned short *tmp_i=ibuf+j;
//		for (k=map_height-1; k>=map_height-1-FLOORWINDOW;k--) {
//			sum += *(tmp_i+k*map_width);
//		}
//		*(obuf+j+(map_height-1)*map_width) = reference = (sum >> FLOORSHIFT);
//
//		for (i = map_height-2; i>=FLOORWINDOW; i--)  {
//			sum=0;
//			tmp_i=ibuf+j;
//			for (k=i; k>=i-FLOORWINDOW;k--) {
//				sum += *(tmp_i+k*map_width);
//			}
//			*(obuf+j+i*map_width) = (sum >> FLOORSHIFT);
//		}
//	}
	
	return *this;
}

#define HISTSIZE  8
#define HISTDIVF  2

CImage & CImage::WindowHist(CImage &original, unsigned short h, unsigned short v)
{
	// compute new size of image
	Reallocate(original.map_width,original.map_height);
	
	unsigned int i,j,k,l,sum;
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;
	
	unsigned short hist[HISTSIZE]; // 32 gray levels at 5 bits
	unsigned short rhist[HISTSIZE]; 
	
	for (j = 0; j < map_width-h; j++) {
		// compute reference histogramm
		for (k=0;k<HISTSIZE;k++) rhist[k]=0;
		for (k=map_height-1; k>=map_height-v;k--) {
			unsigned int linep = k*map_width;
			assert(((*(ibuf+l+linep))>>HISTDIVF)<HISTSIZE);
			for (l=j;l<j+h;l++) ++rhist[(*(ibuf+l+linep))>>HISTDIVF];
		} 
		// compute all other histograms and compare
		for (i = map_height-1; i>=v; i--)  {
			for (k=0;k<HISTSIZE;k++) hist[k]=0;
			for (k=i; k>=i+1-v;k--) {
				unsigned int linep = k*map_width;
				assert(((*(ibuf+l+linep))>>HISTDIVF)<HISTSIZE);
				for (l=j;l<j+h;l++) ++hist[(*(ibuf+l+linep))>>HISTDIVF];
			}
			sum=0;
			for (l=0;l<HISTSIZE;l++) sum+=abs((int)hist[l]-(int)rhist[l]);
			//cout <<(int)sum << " ";
			*(obuf+j+i*map_width)= (unsigned short)(sum/2);
		}
	}

	return *this;	
}

// Time:

CImage & CImage::WindowAvg(CImage &original, unsigned short h, unsigned short v)
{
	assert(MAX_WINDOW_HEIGHT>v);
	unsigned int org_width = original.map_width;
	unsigned int org_height= original.map_height;
	
	// compute new size of image
	Reallocate(	org_width-h+1,org_height-v+1);
	
	register unsigned int asum,tsumh,tsum1;
	unsigned int *sump,*sumpB;
	unsigned int i,j,k,l,N=h*v;
	unsigned short *linep_i,*linep_o;
	unsigned short *ibuf = original.map;
	unsigned short *ibuf1 = original.map-1;
	unsigned short *ibufh1 = original.map+h-1;
	unsigned short *linep_i1, *linep_ih1;
	unsigned short *obuf = map;
	
	
	/////////////////////////////////////////////////////
	// compute horizontal sums of all lines 
	linep_i		= ibuf;
	linep_i1	= ibuf1;
	linep_ih1	= ibufh1;
	sump		= sum;
	for (i=0;i<org_height;i++) {
		// compute sums of first pixel in current line
		asum=0;
		for (l=0;l<h;l++) {
			tsum1 = (*(linep_i+l));
			asum	+= tsum1;
		}
		(*(sump))	= asum;

		// incrementally compute sums of other pixels of lines
		for (j=1;j<map_width;j++) {
			tsumh = (*(linep_ih1+j));
			tsum1 = (*(linep_i1+j));
			asum	+= tsumh;
			asum -= tsum1;		// add and subtract new/old
			(*(sump+j))	= asum;
		}
		
		// update array index helpers
		linep_i 	+= org_width;
		linep_i1	+= org_width;
		linep_ih1	+= org_width;
		sump		+= map_width;
	}
	
	/////////////////////////////////////////////////////
	// compute all vertical sums from the horizontal sums
	for (j=0;j<map_width;j++) {
		sump	=	sum+j;
		asum	=	0;
		// do it for the first line
		for (k=0;k<v;k++) {
			asum	+=	(*(sump));
			sump	+= map_width;
		}
		// do it for all other lines
		linep_o	=	obuf+j;
		sumpB	=	sum+j;
		for (i=0;i<map_height;i++) {
			(*(linep_o))	= (unsigned short)(asum/N);	// compute variance
			asum	+= (*(sump));
			asum	-= (*(sumpB));
			
			linep_o	+= map_width;
			sump	+= map_width;
			sumpB	+= map_width;
		}
	}

	return *this;	
}

// Time: 24-27 msec (+/- 8 msec per FLOP in output loop)

CImage & CImage::WindowVariance(CImage &original, unsigned short h, unsigned short v, unsigned int varscale)
{
	assert(MAX_WINDOW_HEIGHT>v);
	unsigned int org_width = original.map_width;
	unsigned int org_height= original.map_height;
	
	// compute new size of image
	Reallocate(	org_width-h+1,org_height-v+1);
	
	
	register unsigned int asum,asum2,tsumh,tsum1;
	unsigned int *sump,*sum2p,*sumpB,*sum2pB;
	unsigned int i,j,k,l,N=h*v;
	unsigned short *linep_i,*linep_o;
	unsigned short *ibuf = original.map;
	unsigned short *ibuf1 = original.map-1;
	unsigned short *ibufh1 = original.map+h-1;
	unsigned short *linep_i1, *linep_ih1;
	unsigned short *obuf = map;	
	
	/////////////////////////////////////////////////////
	// compute horizontal sums of all lines 
	linep_i		= ibuf;
	linep_i1	= ibuf1;
	linep_ih1	= ibufh1;
	sump		= sum;
	sum2p		= sum2;
	for (i=0;i<org_height;i++) {
		// compute sums of first pixel in current line
		asum=asum2=0;
		for (l=0;l<h;l++) {
			tsum1 = (*(linep_i+l));
			asum	+= tsum1;
			asum2+= tsum1*tsum1;
		}
		(*(sump))	= asum;
		(*(sum2p))	= asum2;

		// incrementally compute sums of other pixels of lines
		for (j=1;j<map_width;j++) {
			tsumh = (*(linep_ih1+j));
			tsum1 = (*(linep_i1+j));
			asum += tsumh;
			asum -= tsum1;		// add and subtract new/old
			asum2+= tsumh*tsumh; 
			asum2-= tsum1*tsum1;
			(*(sump+j))	= asum;
			(*(sum2p+j))= asum2;
		}
		
		// update array index helpers
		linep_i 	+= org_width;
		linep_i1	+= org_width;
		linep_ih1	+= org_width;
		sump		+= map_width;
		sum2p		+= map_width;
	}
	
	/////////////////////////////////////////////////////
	// compute all vertical sums from the horizontal sums
	for (j=0;j<map_width;j++) {
		sump	=	sum+j;
		sum2p	=	sum2+j;
		asum	=	0;
		asum2	=	0;
		// do it for the first line
		for (k=0;k<v;k++) {
			asum	+=	(*(sump));
			asum2	+=	(*(sum2p));
			sump	+= map_width;
			sum2p	+= map_width;
		}
		// do it for all other lines
		linep_o	=	obuf+j;
		sumpB	=	sum+j;
		sum2pB	=	sum2+j;
		for (i=0;i<map_height;i++) {
			(*(linep_o))	= (unsigned short)((asum2-asum*asum/N)/varscale);	// compute variance
			asum	+= (*(sump));
			asum2	+= (*(sum2p));
			asum	-= (*(sumpB));
			asum2	-= (*(sum2pB));
			
			linep_o	+= map_width;
			sump	+= map_width;
			sum2p	+= map_width;
			sumpB	+= map_width;
			sum2pB	+= map_width;
		}
	}

	return *this;	
}

// This function takes 30-32 ms to compute for a 100x75 image with a 5x5 window

CImage & CImage::WindowVarianceAvg(CImage &original, unsigned short h, unsigned short v, CImage &avgout, unsigned int varscale)
{
	assert(MAX_WINDOW_HEIGHT>v);
	unsigned int org_width = original.map_width;
	unsigned int org_height= original.map_height;
	
	// compute new size of image
	Reallocate(	org_width-h+1,org_height-v+1);
	avgout.Reallocate(	org_width-h+1,org_height-v+1);
	
	register unsigned int asum,asum2,tsumh,tsum1;
	unsigned int *sump,*sum2p,*sumpB,*sum2pB;
	unsigned int i,j,k,l,N=h*v;
	unsigned short *linep_i,*linep_o,*linep_o_avg;
	unsigned short *ibuf = original.map;
	unsigned short *ibuf1 = original.map-1;
	unsigned short *ibufh1 = original.map+h-1;
	unsigned short *linep_i1, *linep_ih1;
	unsigned short *obuf = map;
	unsigned short *obuf_avg = avgout.map;
	
	
	/////////////////////////////////////////////////////
	// compute horizontal sums of all lines 
	linep_i		= ibuf;
	linep_i1	= ibuf1;
	linep_ih1	= ibufh1;
	sump		= sum;
	sum2p		= sum2;
	for (i=0;i<org_height;i++) {
		// compute sums of first pixel in current line
		asum=asum2=0;
		for (l=0;l<h;l++) {
			tsum1 = (*(linep_i+l));
			asum	+= tsum1;
			asum2+= tsum1*tsum1;
		}
		(*(sump))	= asum;
		(*(sum2p))	= asum2;

		// incrementally compute sums of other pixels of lines
		for (j=1;j<map_width;j++) {
			tsumh = (*(linep_ih1+j));
			tsum1 = (*(linep_i1+j));
			asum	+= tsumh;
			asum -= tsum1;		// add and subtract new/old
			asum2+= tsumh*tsumh; 
			asum2-= tsum1*tsum1;
			(*(sump+j))	= asum;
			(*(sum2p+j))= asum2;
		}
		
		// update array index helpers
		linep_i 	+= org_width;
		linep_i1	+= org_width;
		linep_ih1	+= org_width;
		sump		+= map_width;
		sum2p		+= map_width;
	}
	
	/////////////////////////////////////////////////////
	// compute all vertical sums from the horizontal sums
	for (j=0;j<map_width;j++) {
		sump	=	sum+j;
		sum2p	=	sum2+j;
		asum	=	0;
		asum2	=	0;
		// do it for the first line
		for (k=0;k<v;k++) {
			asum	+=	(*(sump));
			asum2	+=	(*(sum2p));
			sump	+= map_width;
			sum2p	+= map_width;
		}
		// do it for all other lines
		linep_o	    = obuf+j;
		linep_o_avg = obuf_avg+j;
		sumpB	=	sum+j;
		sum2pB	=	sum2+j;
		for (i=0;i<map_height;i++) {
			unsigned int avg = asum/N;
			(*(linep_o_avg))= (unsigned short)avg;
			(*(linep_o))	= (unsigned short)((asum2-asum*asum/N)/varscale);	// compute variance
			asum	+= (unsigned int)(*(sump));
			asum2	+= (*(sum2p));
			asum	-= (*(sumpB));
			asum2	-= (*(sum2pB));
			
			linep_o	+= map_width;
			linep_o_avg += map_width;
			sump	+= map_width;
			sum2p	+= map_width;
			sumpB	+= map_width;
			sum2pB	+= map_width;
		}
	}

	return *this;	
}

// Run time: 49-51 msec  for a 100x75 image
//
//#define VARSCALE 10
//
//CImage & CImage::WindowVariance(CImage &original, unsigned short h, unsigned short v)
//{
//	unsigned int org_width = original.map_width;
//	unsigned int org_height= original.map_height;
//	
//	// compute new size of image
//	Reallocate(	org_width-h+1,org_height-v+1);
//	
//	unsigned int i,j,k,l,sum,sum2,var,N=h*v;
//	unsigned short *tmp_i;
//	unsigned short *ibuf = original.map;
//	unsigned short *obuf = map;
//		
//	for (j=0; j<map_width; j++) {
//		// compute initial variance
//		sum=sum2=0;
//		for (k=0; k<v;k++) {
//			unsigned int linep = k*org_width;
//			for (l=j;l<j+h;l++) {
//				sum+=(*(ibuf+l+linep)); 
//				sum2+=(*(ibuf+l+linep))*(*(ibuf+l+linep));
//			}
//		}
//		var=sum2-sum*sum/N;
//		*(obuf+j)= var/VARSCALE;
//		
//		// compute increments
//		for (i=1; i<map_height;i++) {
//			// add new line
//			unsigned int linep = (i+v-1)*org_width;
//			for (l=j;l<j+h;l++) {
//				sum+=(*(ibuf+l+linep)); 
//				sum2+=(*(ibuf+l+linep))*(*(ibuf+l+linep));
//			}
//			// subtract last line 
//			linep = (i-1)*org_width;
//			for (l=j;l<j+h;l++) {
//				sum-=(*(ibuf+l+linep)); 
//				sum2-=(*(ibuf+l+linep))*(*(ibuf+l+linep));
//			}
//
//			var=sum2-sum*sum/N;
//			*(obuf+j+i*map_width)= var/VARSCALE;
//		}
//	}
//
//	return *this;	
//}

CImage & CImage::WindowConv(CImage &original, CKernel &kernel)
{
	// compute new size of image
	Reallocate(	original.map_width,
				original.map_height);
	
	unsigned int i,j,k,l,v,h;
	unsigned int x,y;
	int sum;
	short *kbuf = kernel.kernel;
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;
	
	h=kernel.map_width;
	v=kernel.map_height;
		
	for (j=0; j<map_width-h; j++) {
		for (i=0; i<map_height-v; i++)  {
			sum=0;
			for (k=i,y=0; k<i+v;k++,y++) {
				unsigned int linep = k*map_width;
				unsigned int kernp = y*h;
				for (l=j,x=0;l<j+h;l++,x++) 
					sum+=(*(ibuf+l+linep))*(*(kbuf+x+kernp)); 
			}
			
//			cout << sum << " ";
			*(obuf+j+i*map_width)= (unsigned short)(abs(sum)/20);
		}
	}

	return *this;	
}

CImage & CImage::ObstDist(CImage &original, unsigned short threshold)
{
	// compute new size of image
	Reallocate(	original.map_width,
				original.map_height);
	
	unsigned int i,j;
	unsigned short *p_i, *p_o;
	unsigned int *sum_p;
	unsigned short *ibuf = original.map;
	unsigned short *obuf = map;
	
	// do it for first line: initialize, defaults to line 0
	sum_p = sum;
	p_o   = obuf;
	for (j=0; j<map_width;j++) {
		*sum_p = 0;
		*p_o =  1;
		p_o++;
		sum_p++;
	}	
	
	// set to first elem of first line
	p_i = ibuf+map_width; 
	sum_p = sum;
	
	// do it for all other lines
	for (i=1;i<map_height;i++) {
		for (j=0; j<map_width;j++) {
			if ((*p_i)>=threshold) *sum_p = i;
			*p_o =  1;
			sum_p++;
			p_i++;
			p_o++;
		}
		sum_p = sum;
	}
	
	// draw line of recognized free space
	p_o = obuf;
	sum_p = sum;
	min_pixel_dist = 0;
	for (j=0;j<map_width;j++) {
		(*(p_o+(map_width*(*sum_p)))) = 20;
		if ((*sum_p)>min_pixel_dist) min_pixel_dist=(*sum_p);
		p_o++;
		sum_p++;
	}
	min_pixel_dist = map_height-min_pixel_dist-1; // now reverse orientation
	
	return *this;	
}

/////////////////////////////////////////////////
// stuff on navigation

// run this after running ObstDist !
unsigned int CImage::MinPixelDist(void)
{
	return min_pixel_dist;
}

// run this after running ObstDist !
unsigned int CImage::MaxPixelDist(unsigned int width)
{
	unsigned int *sum_p = sum;
	unsigned short *o_p;
	unsigned int j,l;
	unsigned int best_position=0, best_lowest_pixel=map_height;
	max_pixel_dist = 0;

	for (j=0;j<=map_width-width;j++) {
		unsigned int lowest_pixel=0;
		unsigned int to=j+width;
		sum_p = sum+j;
		// find lowest position in current window
		for (l=j;l<to;l++) {
			if (lowest_pixel<(*sum_p)) lowest_pixel=(*sum_p);
			sum_p++;
		}
		// update best (highest) vertical position
		if (lowest_pixel<best_lowest_pixel) {
			best_lowest_pixel = lowest_pixel;
			best_position = j;
		}
	}
	max_pixel_dist = map_height-best_lowest_pixel-1; // now reverse orientation
	
	// now display the stuff
	o_p=(map+map_width*best_lowest_pixel+best_position);
	unsigned int to = best_position+width;
	for (j=best_position;j<to;j++) {
		(*(o_p)) = 30;
		o_p++;		
	}
	
	return max_pixel_dist;
}

// run this after running ObstDist !
void CImage::MaxPixelDist2(unsigned int width, 
						   unsigned int &best_lowest_pixel,
						   unsigned int &best_position)
{
	unsigned int *sum_p = sum;
	unsigned int j,l;
	max_pixel_dist = 0;

	best_position=0;
	best_lowest_pixel=map_height;

	for (j=0;j<=map_width-width;j++) {
		unsigned int lowest_pixel=0;
		unsigned int to=j+width;
		sum_p = sum+j;
		// find lowest position in current window
		for (l=j;l<to;l++) {
			if (lowest_pixel<(*sum_p)) lowest_pixel=(*sum_p);
			sum_p++;
		}
		// update best (highest) vertical position
		if (lowest_pixel<best_lowest_pixel) {
			best_lowest_pixel = lowest_pixel;
			best_position = j;
		}
	}	
}

// run this after running ObstDist !
int CImage::OptLocalDirection(unsigned int width)
{
	unsigned int *sum_p = sum;
	unsigned short *o_p;
	unsigned int u, h_want, h_is,h_was,current_pos=0,previous_pos,j;
	unsigned int wdiff = map_width-width;
	boolean  terminated=TRUE;
	unsigned int pos_sum=0,pos_cnt=0;
	h_want = map_height-1;
	
	for (u=width;u<=map_width;u++) {
		previous_pos = current_pos;
		h_was = h_is;
		// h_want = (map_height*(u-width))/(wdiff);
		MaxPixelDist2(u,h_is,current_pos);
		if (h_is>=h_want) {terminated=FALSE;break;}
		
		pos_sum += (current_pos+current_pos+u);
		pos_cnt++;
		
		// now display the stuff
		o_p=(map+map_width*h_is+current_pos);
		unsigned int to = current_pos+u;
		for (j=current_pos;j<to;j++) {
			(*(o_p)) = 30;
			o_p++;		
		}

	}
	
	if (pos_cnt==0) return 0;
	
	cout << "[" << ((int)(pos_sum/pos_cnt)-map_width)/2 << "]";
	return ((int)(pos_sum/pos_cnt)-map_width)/2;
	
	
	// now display the stuff
//	o_p=(map+map_width*h_was+previous_pos);
//	unsigned int to = previous_pos+u-1;
//	for (j=previous_pos;j<to;j++) {
//		(*(o_p)) = 30;
//		o_p++;		
//	}
//	
//	max_pixel_dist = map_height-u-1;
//	return max_pixel_dist;
	
}


// run this after running ObstDist !
FreeSpace CImage::FreePath(unsigned int width_top, unsigned int width_bottom)
{
	// zeroes indicate default is desired
	if (width_bottom==0) width_bottom=map_width;
	if (width_top==0) width_top=map_width/3;
	
	// check for valid values
	if (width_top>map_width)	width_top	= map_width;
	if (width_bottom>map_width)	width_bottom= map_width;
	
	assert(width_top<width_bottom);

	unsigned int *sum_p = sum;
	unsigned short *o_p;
	unsigned int from, to;
	unsigned int  i;	
	unsigned int lmax,mmax,rmax;
	lmax=mmax=rmax=map_height-1;
	
	unsigned int wb = (map_width-width_bottom)/2;
	unsigned int wt = (map_width-width_top)/2;
	unsigned int dw = (wt-wb);
	
	for (i=0;i<map_width;i++) {	
		unsigned int l   = (map_height-(*sum_p));
		unsigned int col = dw*l/map_height+wb;
		if (i<col)                    {if (lmax>l) lmax=l;}
		else if (i>=(map_width-col))  {if (rmax>l) rmax=l;}
		else                          {if (mmax>l) mmax=l;}
		sum_p++;
	}
	
	// draw line for left sector
	from = 0;
	to   = dw*lmax/map_height+wb;
	o_p=map+map_width*(map_height-lmax-1); 
	for (i=from;i<to;i++) {
		*o_p = 30;
		o_p++;
	}
	
	// draw line for right sector
	from = map_width-(dw*rmax/map_height+wb);
	to   = map_width;
	o_p=map+map_width*(map_height-rmax-1)+from;
	for (i=from;i<to;i++) {
		*o_p = 30;
		o_p++;
	}
	
	// draw line for right sector
	from = dw*mmax/map_height+wb;
	to   = (map_width-from);
	o_p=map+map_width*(map_height-mmax-1)+from;
	for (i=from;i<to;i++) {
		*o_p = 30;
		o_p++;
	}
		
	freesp.l = lmax;	// units are pixels counted from bottom up
	freesp.m = mmax;
	freesp.r = rmax;
	
	return freesp;
}

#define HORSWILLTHRES  4
#define HORSWILLEXA    2

void	CImage::VanishHorswill(CImage & original,int &axis, int &var)
{
	// compute new size of image
	Reallocate(	original.map_width-1,
				original.map_height-2);
		
	signed int i,j;
	int tox,toy;
	signed int dx,dy;
	unsigned short *p,*pn;
	int x_int, mean=0, pts=0,sum=0,sum2=0;
	unsigned short *obuf = map;
	
	var=0;  // initialize variance
	
	toy=original.map_height-1; // upper processing boundaries
	tox=original.map_width-1;
	
	p	= original.map+original.map_width;  // init pointers
	pn	= p+original.map_width;
	
	for (i=1;i<toy;i++) {			// for each line
		for (j=0; j<tox;j++) {		// for each pixel in line
			dx = *p - *(p+1);		// calculate gradient
			dy = *p - *pn;
			if (abs(dy)<HORSWILLTHRES) 				// if gradient too small
				dy = *(p-original.map_width) - *p;	// look above
			if ((abs(dx)>HORSWILLTHRES) && 			// if gradients big enough
				(abs(dy)>HORSWILLTHRES)) {
				// found an edge candidate -> calculate intersection
				x_int = ((signed int)HORSWILLEXA*i*dx)/dy+j;
				if (abs(x_int)<1000) {	// if not out of bounds
					*obuf = 31;			// set image point
					sum +=  x_int;		// update sum (for mean)
					sum2+=  x_int*x_int;// update sum of squares (for var)
					pts++;				// go to next point
				} else {
					*obuf = 15;			// if out of bounds -> gray dots
				}
			} else {
				*obuf = 0;//abs(dx)+abs(dy); // for non-edges
			}
			p++;	// increment pointers
			pn++;
			obuf++;
		}
		p++;		// increment pointers for last column of pixels
		pn++;
	}
	
	if (	(pts>8) &&				// if the right amount of edge pixels
			(pts<260)) {
		mean = sum/pts; 			// calc mean
		var  = sum2/pts-mean*mean;	// calc variance
	} else {
		mean=original.map_width/2;	// set mean to middle if wrong amount
		var=3000;					// set really BIG variance
	}
	
	// draw a line for the mean
	if ((mean>=0) && (mean<map_width)) {	// if mean in picture
		obuf=map+mean;						// set pointer
		for (i=0;i<map_height;i++) {		// for all pixels in column
			*obuf = 31;						// set to max
			obuf+=map_width;				// increment pointer
		}
	}

	// move axis coordinate system to middle		
	axis = mean-original.map_width/2;	// offset to middle
}
