/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _CImage_h_
#define _CImage_h_

/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include "defines.h"
#include "CKernel.h"
class LocalMap;

//---- CImage -----------------------------------------------------------

class FreeSpace {
public:
	unsigned int l,m,r;
};


class CImage  {
friend class FrameGrabber;
public:

	CImage(int width_=80, int height_=60);
	CImage(CImage & original);
	~CImage();
	
	
	// Operators on captured images (rgb555 format)
	CImage & MakeGray4(CImage &original); /* for screen use, without the 0 */
	CImage & MakeGray5(CImage &original, unsigned int forget); /* 5 bits for calc */

	// Operators on screen compatible images (colors 1..16, 4 bits)
	void     Show(int x, int y);
	
	// Operators for occupancy grids with values 0..15
	CImage & MakeGray4fromOcc(LocalMap & occ);	// 4 bits for screen display
	
	// Operators for 5 bit calc images (colors 0..31, 5 bits)
	CImage & MakeGray4from5(CImage &original); /* convert to 4 bits */
	CImage & Sum2(CImage &original_1, CImage &original_2);
	CImage & ADiff(CImage &original_1, CImage &original_2);
	CImage & ADiffMix(CImage &original_1, CImage &original_2, unsigned short thres);
	CImage & ADiffMixMax(CImage &original_1, CImage &original_2, unsigned short thres);
	CImage & BoundingBox(CImage &original, 
				unsigned int &xmin, unsigned int &xmax,
				unsigned int &ymin, unsigned int &ymax,
				signed int &xctr, signed int &yctr,
				boolean	&exists,
				unsigned short thres);
	word	 Marshal(byte *buf);	// only uses 4 of 5 bits, colors 0..15
	word	 Unmarshal(byte *buf);	// only uses 4 of 5 bits, colors 0..15
	CImage & Erase(void);	// fill with zeroes
	
	// Operators for pop-ups (for 5 bit calc images (colors 0..31, 5 bits))
	// 
	// Run-times: Image size: 100x75, window size 5x5, and 3x3, on 486/66
	//            last column is for a 3x3 window on a 50x30 image
	//
	//  Avg:       12 - 13     11 - 14    2 -  3
	//  StdDevAvg: 30 - 32     29 - 31    5 -  6
	//  StdDev:    24 - 26     21 - 24    4 -  5
	//  Diff:       3 -  4      3 -  4    0 -  1 (no windows)
	//  Threshold:  2 -  3      2 -  3    0 -  1 (no windows)
	//  
	CImage & HorizontalAvg(CImage &original,unsigned short window); /* Filter out floor texture */
	CImage & WindowAvg(CImage &original, unsigned short h, unsigned short v);
	CImage & WindowHist(CImage &original, unsigned short h, unsigned short v);
	CImage & WindowVariance(CImage &original, unsigned short h, unsigned short v, unsigned int varscale);
	CImage & WindowVarianceAvg(CImage &original, unsigned short h, unsigned short v, CImage &avgout, unsigned int varscale);
	CImage & ObstDist(CImage &original, unsigned short threshold);
	CImage & WindowConv(CImage &original, CKernel &kernel);
	void     MaxPixelDist2(unsigned int width, 
						   unsigned int &best_lowest_pixel,
						   unsigned int &best_position);
	int 	OptLocalDirection(unsigned int width);
	FreeSpace FreePath(unsigned int width_top, unsigned int width_bottom);
	void	VanishHorswill(CImage & original,int &axis, int &var);

	// Stuff about navigation
	unsigned int MinPixelDist(void);
	unsigned int MaxPixelDist(unsigned int width);

	// Operators for all image formats (5 and 4 bits, and rgb555)
	unsigned int	GetImageHeight(void){return map_height;}
	unsigned int	GetImageWidth(void) {return map_width;}
	CImage & operator = (CImage & original);
	CImage & HalfSize(CImage &original); 
	CImage & QuarterSize(CImage &original); 
	CImage & Shrink(CImage &original, unsigned int factor); 
	CImage & Threshold(CImage &original, unsigned short thres); // not for rgb555 format
	
private:	
	void Reallocate(int width_,int height_);

	unsigned short *map;  		// pixel array
	int				map_width;	// width of pixel array
	int				map_height;	// height of pixel array
	long			map_size;	// number of pixels of map

	unsigned int	*sum;	// stores sums for variance
	unsigned int	*sum2;	// sums of squares	
	
	unsigned int	min_pixel_dist;
	unsigned int	max_pixel_dist;
	
	FreeSpace		freesp;
};

#endif	/* _CImage_h_ */
