/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/
/* Rene Schaad                                                    */
/* Computer Science Institute, University of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include <iostream.h>

#include "Errors.h"

//---- Errors ---------------------------------------------------------


// -- declare all error messages
const char *error_msgs[] = {
"Err 00: Cannot open file for input: ",
"Err 01: Command in command file not recognised (line skipped): ",
"Err 02: Command line args not valid: ",
"Err 03: Command execution failed, line skipped: ",
"Err 04: Command in command-string-list, but not in switch statement: ",
"Err 05: Expected integer or real command line argument: ",
"Err 06: Expected integer command line argument: ",
"Err 07: Could not find average state var in source module.",
"Err 08: Randomizer WARNING: lower bound > upper bound! ",
"Err 09: Named rep does not exist, could not allocate Sensor or Effector mapping. ",
"Err 10: This method not to be used directly (virtual method): ",
"Err 11: This rep number is not assigned (line skipped): ",
"Err 12: Expected integer command line argument in connectivity file: ",
"Err 13: Expected real command line argument in connectivity file",
"Err 14: Wrong number of args in connectivity file in command (line skipped): ",
"Err 15: Referenced cell does not exist: ",
"Err 16: Not enough space to allocate new rep: ",
"Err 17: This rep identifier is already taken: ",
"Err 18: Could not cut dendritic tree, because cell-number is not assigned: ",
"Err 19: This Sensor identifier is already taken: ",
"Err 20: The requested type of Sensor does not exist: ",
"Err 21: The Sensory/Effector field is greater than the connected rep (Sensor/Effector not created): ",
"Err 22: Cannot open file for writing: ",
"Err 23: This Sensor identifier is already taken: ",
"Err 24: The requested type of Sensor does not exist: ",
"Err 25: No NSystem assigned to this Agent, yet: ",
"Err 26: This Agent already has a NSystem: ",
"Err 27: This Physical Model identifier already exists: ",
"Err 28: Don't know this type of sensor: ",
"Err 29: Don't know this type of effector: ",
"Err 30: Don't know this type of physical model: ",
"Err 31: Don't know this sensor id: ",
"Err 32: Don't know this effecor id: ",
"Err 33: This rep id does not exist: ",
"Err 34: String argument too long: ",
"Err 35: This name does not exist (line skipped): ",
"Err 36: This name already exists (line skipped): ",
"Err 37: Can't connect topologically, because dimensions do not match: ",
"Err 38: Did not recognize argument keyword: ",
"Err 39: No such initializer value key: ",
"Err 40: Do not know this keyword: ",
"Err 41: Do not have this state variable: ",
"Err 42: Could not find desired entry in DockingRequest: ",
"Err 43: Abstract method was called: ",
"Err 44: Delay must be at least 2: ",
"Err 45: Could not find this dendrite-id: ",
"Err 46: Cannot allocate dendrite because id already exists: ",
"Err 47: This method should not be called: ",
"Err 48: Cannot connect into average state variable",
"Err 49: Cannot connect to unexisting unit",
"Err 50: Can only handle motor repertoires 0..3: ",
"Err 51: Sizes of source and target repertoires are not appropriate: "
};

const int max_err = 51;

// -- error function
void out_error(int errn, const int nr)
{
	if (errn>max_err) cerr << "1: This error number does not exist: " << errn << "\n";
   else cerr << error_msgs[errn] << nr << "\n";
}

void out_error(int errn, const char *argp)
{
   if (errn>max_err) cerr << "2: This error number does not exist: " << errn << "\n";
   else cerr << error_msgs[errn] << argp << "\n";
}

void out_error(int errn, const char *argp, const int nr)
{
	if (errn>max_err) cerr << "3: This error number does not exist: " << errn << "\n";
   else cerr << error_msgs[errn] << " (" << argp << "): " << nr << "\n";
}
