/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/
/*  grabber.cc: initialize the PMS-board and
             grabbing captured frames from the PMS-board
  a part of PMS-grabber package
  
  Class FrameGrabber created by Rene Schaad, Univ. of Zurich
  based on code by Wolfgang Koehler, (wolf@first.gmd.de), Dec. 1994
*/

//   Versions: 
//    5.07.1996: rS: V.0.1 Adapted from D.Lambrinos' grabber.c to QNX
//   15.08.1996: rS: V.0.2 Changed to object oriented structure (FrameGrabber)
    
    
//  *********  INCLUDES *************     
#include <conio.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h> 
#include <unistd.h> /* iopl,ioperm*/
#include <string.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <windows/Qwindows.h>
#include "mvv_parms.h" /* definition of port, irq, address*/
#include "FrameGrabber.h"

// define hardware access functions for compatability with Linux version
#define outb(VALUE,PORT)	outp((PORT),(VALUE))
#define outw(VALUE,PORT)    outpw((PORT),(VALUE))
#define inb(PORT)			inp(PORT)
#define OUT_MVV(val) 		outb(val, mvpi)
#define IN_MVV 				(inb(mvpd))
int iopl(int i) {i=i;return 0;}   				// dummy, for compatability
int ioperm(int i,int j, int k) {i=j=k;return 0;}	// dummy, for compatability

// misc. defines and constants

#define  MVVMEMORYWIDTH    	0x40    /*512*/
#define  UNDEFINED  		0x0000
#define  MOTOROLA  			0x0001    /*Early debug board*/
#define  PHILIPS2  			0x0002
#define  PHILIPS1  			0x0003
#define  VMAX 				280
#define  MVVBUFSZ 			0x1000
#define  PAGE_SIZE 			0x1000

// data types

#ifndef FALSE
int FALSE =  0;
int TRUE  = 1;
#endif


//////////  MEMBER FUNCTIONS  /////////////////////////

FrameGrabber::FrameGrabber(int biWidth_, int biHeight_)
{
	mvpi = ProMoviePort;	// initialize port numbers
	mvpd = ProMoviePort+1;
	mvps = ProMoviePort+2;
	mvv_base = ProMovieAddr;
	
	iI2CCount = 0;

	wMVVDecoder = 2; /*1=Motorola, 2=Philips 2-chip,  3=Philips 3-chip*/
	wVideoStandard = 2; /*0=Auto, 1=NTSC, 2=PAL, 3=SECAM*/
	iMVVBoardID = 0;
	iMVVID = 0;

	biWidth = biWidth_;
	biHeight = biHeight_;

    mvv_init();
    all_defaults();
}

FrameGrabber::~FrameGrabber(void) {}


//////////////////////////////////////////////////////////////////////////
//////       Memory access functions

void *FrameGrabber::pmem_map(unsigned base, unsigned buf_size) {
  void *buf;
//  unsigned char *bbuf;
//  unsigned off;

  int mem_fd = shm_open("Physical", O_RDWR,0777);
  if (mem_fd < 0) {
    printf("cannot open /dev/shmem (error %d)\n",errno);
    printf("program must run with root permissions !\n");
    exit(-1);
  }

//  The following stuff was the original LINUX code:
//  if ((buf = malloc(buf_size + (PAGE_SIZE-1))) == NULL) {
//    printf("cannot malloc\n");
//    exit(-1);
//  }
//  off = (unsigned long) buf % PAGE_SIZE;
//  bbuf=(unsigned char*)buf;			/* introduced for QNX compatability */
//  if (off) bbuf += PAGE_SIZE - off;
//  buf =(void*)buf;					/* introduced for QNX compatability */
  buf = mmap( (void*) 0, buf_size, PROT_READ | PROT_WRITE,
		  MAP_SHARED, mem_fd, base);
  if (buf == (void*) -1) printf("mem mapping error (%d)\n",(int)errno);
  /*  test_mem(mem_fd); */
  close (mem_fd);
  return buf;
}


//////////////////////////////////////////////////////////////////////////
//////       Hardware access functions

/* write value to mvv-register [index]*/
void FrameGrabber::mvv_write(char index, char value) {
  outw(index | (value << 8),mvpi);
}

unsigned char FrameGrabber::mvv_read(char index) {
  outb(index, mvpi);
  return((unsigned char)inb(mvpd));
}

int FrameGrabber::mvv_init() {
	/*     int idec,decst = i2cstat(0x43);  */
	int idec,decst;
	mvv_buf = pmem_map(mvv_base, MVVBUFSZ);
	if (mvv_buf == (void*) -1) {
		printf(" no mvv_buf !\n"); exit(1);
	}
	/* Init .. */
	{  /* WakeUpMVV...*/ 
		{   /* SetPort ...*/
			iopl(3); /* the only level to get through here*/
			outb(0xb8, 0x9a01);
			outb(mvpi >> 4, 0x9a01);            /* assign port 0x250 (>>4)*/
			iopl(0); /* reset to normal permissions*/
			printf("io-port is set to %x\n",mvpi);
			ioperm(mvpi,3,1);               /* 3 ports permission level 1*/
		}
		printf("MVV ID = %d ",mvv_read(3)); 
		decst = i2cstat(0x43); 
		if (decst != -1) idec = 2; else
		if (i2cstat(0xb9) != -1) idec = 3; else
		if (i2cstat(0x8b) != -1) idec = 1; else idec = 0;
		printf("Decoder = %d \n",idec);
		if (idec == 0) exit(1);
	}
  
	/* SetAddr ... */
	mvv_write(0x04,mvv_base >> 12);    /* set PC memory address to 0xyy000*/
	/* SetIrq ...
	int temp = inb(ProMoviePort);
	outw(0x882E,ProMoviePort);
	outb(temp,ProMoviePort);
	outb((inb(ProMoviePort + 2) & 0xF0) | ProMovieIRQ, ProMoviePort + 2);
	IdentifyBoard */

	I2CWrite(0x8A,0x00,0x4C);
	I2CWrite(0x8A,0x01,0x30);
	I2CWrite(0x8A,0x02,0x00);
	I2CWrite(0x8A,0x03,0xE8);
	I2CWrite(0x8A,0x04,0xB6);
	I2CWrite(0x8A,0x05,0xE2);  /*0xF2*/
	I2CWrite(0x8A,0x06,0x00);
	I2CWrite(0x8A,0x07,0x00);
	I2CAndOr(0x8A,0x08,0x07,0x00);  /*QUAM*/
	I2CAndOr(0x8A,0x09,0x07,0x00);  /*SECAM*/
	I2CWrite(0x8A,0x0A,0x00);    /*0x40*/
	I2CWrite(0x8A,0x0B,0x00);    /*0x50*/
	I2CWrite(0x8A,0x0C,0x00);
	I2CWrite(0x8A,0x0D,0x00);
	I2CWrite(0x8A,0x0E,0x78);
	I2CWrite(0x8A,0x0F,0x98 | 0x00);
	I2CWrite(0x8A,0x10,0x00);
	I2CWrite(0x8A,0x11,0x00);
	I2CWrite(0x8A,0x12,0x00);
	I2CWrite(0x8A,0x13,0x00);
	I2CWrite(0x8A,0x14,0x34);
	I2CWrite(0x8A,0x15,0x0A);
	I2CWrite(0x8A,0x16,0xF4);
	I2CWrite(0x8A,0x17,0xCE);
	I2CWrite(0x8A,0x18,0xE4);  /*0xF4*/

	I2CWrite(0xB8,0x00,0x12 | 0x00);
	I2CWrite(0xB8,0x04,0x00);
	I2CWrite(0xB8,0x07,0x00);
	I2CWrite(0xB8,0x08,0x00);

	I2CWrite(0xB8,0x09,0xFF);
	I2CWrite(0xB8,0x0A,0x00);
	I2CWrite(0xB8,0x0B,0x10);
	I2CWrite(0xB8,0x10,0x03);

	mvv_write(0x01,0x00);    /*Interrupt Mask: Enable Field/Compression interrupts*/
	/*    mvv_write(0x05,0x20);*/    /*Refresh Ctrl*/
	mvv_write(0x05,0xA0);    /*Refresh Ctrl*/
	mvv_write(0x08,0x25);    /*PC Access: Enabled, RGB555*/
	mvv_write(0x09,0x00);    /*Mem Page Ctrl: 0000h*/
	mvv_write(0x0A,0x20 | MVVMEMORYWIDTH);  /*Mem Page Slct: 1024 WORDs, Auto incr*/
	mvv_write(0x10,0x02);    /*Capture Ctrl: continuous capture enable, RGB565*/
	mvv_write(0x1E,0x0C);    /*Cap Horz Total:magic PLL number*/
	mvv_write(0x1F,0x03);    /*Cap Horz Total:magic PLL number*/
	mvv_write(0x26,0x06);    /*????*/
	mvv_write(0x2B,0x00);    /*????*/
	mvv_write(0x2C,0x20);    /*Clamp Offset*/
	mvv_write(0x2D,0x00);    /*Clamp Length*/
	/*    mvv_write(0x2E,0x31);*/    /*Composite input*/
	mvv_write(0x2F,0x70);    /*Re-sync enable (H,V,F)*/
	/*    mvv_write(0x30,0x22);*/    /*Comp */
	mvv_write(0x32,0x00);    /*Comp Mem Page Ctrl: 0000h*/
	mvv_write(0x33,MVVMEMORYWIDTH);  /*Comp Mem Page Slct: 1024 WORDs*/
	mvv_write(0x34,0x00);    /*Comp DMA Count (L)*/
	mvv_write(0x35,0x00);    /*Comp DMA Count (H)*/

	mvv_write(0x3A,0x80);    /*Compressor Luminance Control*/
	mvv_write(0x3B,0x10);   /*Compressor Chrominance Control*/


	mvv_write(0x20,0x00);
	mvv_write(0x21,0x00);

	mvv_write(0x30,0x22);    /*Comp*/ 

	return (1);
}

int FrameGrabber::i2cstat(unsigned char slave) { /* .. 405ff*/
  unsigned short cx = 0;
  int i;
  OUT_MVV(0x28); 
  while ((IN_MVV & 1) == 0) if (--cx == 0) break;
  while ((IN_MVV & 1) != 0) if (--cx == 0) break;
  outb(slave, mvpd);
  cx = 0;
  while ((IN_MVV & 1) == 0) if (--cx == 0) break;
  while ((IN_MVV & 1) != 0) if (--cx == 0) break;
/*  printf("test slave %x\n",slave);*/
  for (i=0; i < 12; i++) {
    char st = (char)IN_MVV;
    if ((st & 2) != 0) return -1;
    if ((st & 1) == 0) break;
  }
  OUT_MVV(0x29);
  return (IN_MVV); 
}

int FrameGrabber::I2CWrite(WORD bSlaveAddr, WORD bSubAddr,WORD bData)
{
    int iSkipWrite = FALSE;
    unsigned short cx = 0;
    char ah = (char)IN_MVV;

    int  i;

    for(i=0;i<iI2CCount;i++) {
      if((i2cInfo[i].bSlave==bSlaveAddr) &&(i2cInfo[i].bSubAddr==bSubAddr)) {
        iSkipWrite = (i2cInfo[i].bData==bData);
        i2cInfo[i].bData = (BYTE) bData;
        i = iI2CCount+1;
      }
    }
    if((i==iI2CCount) && (iI2CCount<64)) {
      i2cInfo[iI2CCount].bSlave   = (BYTE) bSlaveAddr;
      i2cInfo[iI2CCount].bSubAddr = (BYTE) bSubAddr;
      i2cInfo[iI2CCount].bData    = (BYTE) bData;
      iI2CCount++;
    }

    if(iSkipWrite) return(0);

    mvv_write (0x29, bSubAddr );
    mvv_write (0x2A, bData );
    mvv_write (0x28, bSlaveAddr );

    OUT_MVV(0x28); 
    while ((IN_MVV & 1) == 0) if (--cx == 0) break;
    while ((IN_MVV & 1) != 0) if (--cx == 0) break;
    
    if (ah & 2) return(-1); else return(ah);

}

int FrameGrabber::I2CRead(WORD bSlaveAddr, WORD bSubAddr)
{
  int i;
  for(i=0;i<iI2CCount;i++)
    if((i2cInfo[i].bSlave==bSlaveAddr) &&(i2cInfo[i].bSubAddr==bSubAddr))
      return(i2cInfo[i].bData);
  return(0);
}


void FrameGrabber::I2CAndOr(WORD bSlaveAddr, WORD bSubAddr,WORD bAND,WORD bOR)
{
    BYTE  bTmp;
    bTmp = (byte)I2CRead(bSlaveAddr,bSubAddr);
    bTmp = (byte)((bTmp & bAND) | bOR);
    I2CWrite(bSlaveAddr,bSubAddr,bTmp);
}

/*  ***********************************************************************/
/*   Testing the iic- read/write mechanism*/

void FrameGrabber::print_iinfo() {
  int i;
  printf("I2C regs \n i slave addr val\n");
  for (i=0; i< iI2CCount; i++) {
    printf("%2d   %2x   %2x   %2x\n",i,
	   i2cInfo[i].bSlave,i2cInfo[i].bSubAddr,i2cInfo[i].bData);
  }
}

void FrameGrabber::test_i2c() {
  unsigned char st;
  int i;
  do { 
    st = mvv_read(0x28);
    printf(" st = %x",st);
    for (i=0x29; i < 0x2c; i++) printf(" %2x",mvv_read(i));
  } while ((st & 7) != 0);
  printf("\n");
}

char FrameGrabber::i2c_read(char slave, char addr) {
  printf("reading %x %x\n",(unsigned char) slave,(unsigned char) addr);
  mvv_write (0x29, addr);
  test_i2c();  	
  mvv_write (0x28, slave | 1); /* set read bit*/
  /* while (mvv_read(0x28) != (slave | 1));*/
  test_i2c(); 		
  return (mvv_read (0x2A));
}


//////////////////////////////////////////////////////////////////////////
//////       Hardware access functions

void FrameGrabber::SetMVVVideoSource(WORD wData) { 
  mvv_write(0x2E,(wData) ? 0x31 : 0x30 );
}

void FrameGrabber::SetMVVHue(WORD wData) {
    switch(wMVVDecoder) {
  case MOTOROLA :
      I2CWrite(0x8A,0x00,wData);
      break;
  case PHILIPS2 :
      I2CWrite(0x8A,0x07,wData);
      break;
  case PHILIPS1 :
      I2CWrite(0x42,0x07,wData);
      break;
    }
}

void FrameGrabber::SetMVVColor(WORD wData)
{
    switch(wMVVDecoder) {
  case MOTOROLA :
      I2CWrite(0x8A,0x00,wData);
      break;
  case PHILIPS1 :
      I2CWrite(0x42,0x12,wData);
      break;
    }
}

void FrameGrabber::SetMVVContrast(WORD wData)
{
    switch(wMVVDecoder) {
  case MOTOROLA :
      I2CWrite(0x8A,0x00,wData);
      break;
  case PHILIPS1 :
      I2CWrite(0x42,0x13,wData);
      break;
    }
}

void FrameGrabber::SetMVVBrightness(WORD wData)
{
    switch(wMVVDecoder) {
  case MOTOROLA :
      I2CWrite(0x8A,0x00,wData);  /*Red*/
      I2CWrite(0x8A,0x00,wData);  /*Green*/
      I2CWrite(0x8A,0x00,wData);  /*Blue*/
      break;
  case PHILIPS1 :
      I2CWrite(0x42,0x19,wData);
      break;
    }
}

void FrameGrabber::SetMVVHStart(WORD wData)
{
  switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CWrite(0x8A,0x05,wData);  /*PAL/SECAM (+-118)*/
      I2CWrite(0x8A,0x18,wData);  /*NTSC (+- 97)*/
      break;
  case PHILIPS1 :
      I2CWrite(0x42,0x05,wData);
      I2CWrite(0x42,0x18,wData);
      break;
    }
}

void FrameGrabber::SetMVVVStart(WORD wData)
{
    mvv_write(0x16,wData);
    mvv_write(0x17,(wData >> 8) & 0x01);
}

void FrameGrabber::SetMVVVideoStd(WORD wData)
{
    wVideoStandard = wData;

    switch(wMVVDecoder) {
  case PHILIPS2 :
      switch(wData) {
      case 0 :    /*Auto*/
            I2CAndOr(0x8A,0x0D,0xFE,0x00);
            I2CAndOr(0x8A,0x0F,0x3F,0x80);
        break;
      case 1 :    /*NTSC*/
            I2CAndOr(0x8A,0x0D,0xFE,0x00);
            I2CAndOr(0x8A,0x0F,0x3F,0x40);
        break;
      case 2 :    /*PAL*/
            I2CAndOr(0x8A,0x0D,0xFE,0x00);
            I2CAndOr(0x8A,0x0F,0x3F,0x00);
        break;
      case 3 :    /*SECAM*/
            I2CAndOr(0x8A,0x0D,0xFE,0x01);
            I2CAndOr(0x8A,0x0F,0x3F,0x00);
        break;
      } break;
  case PHILIPS1 :
      switch(wData) {
      case 0 :    /*Auto*/
            I2CAndOr(0x42,0x0D,0xFE,0x00);
            I2CAndOr(0x42,0x0F,0x3F,0x80);
        break;
      case 1 :    /*NTSC*/
            I2CAndOr(0x42,0x0D,0xFE,0x00);
            I2CAndOr(0x42,0x0F,0x3F,0x40);
        break;
      case 2 :    /*PAL*/
            I2CAndOr(0x42,0x0D,0xFE,0x00);
            I2CAndOr(0x42,0x0F,0x3F,0x00);
        break;
      case 3 :    /*SECAM*/
            I2CAndOr(0x42,0x0D,0xFE,0x01);
            I2CAndOr(0x42,0x0F,0x3F,0x00);
        break;
      }
  default :
      break;
    }
}

void FrameGrabber::SetMVVVCRInput(WORD wData)
{
    switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CAndOr(0x8A,0x0D,0x7F,(wData & 0x01) <<7);
      break;
  case PHILIPS1 :
      I2CAndOr(0x42,0x0D,0x7F,(wData & 0x01) <<7);
      break;
  default :
      break;
    }
}

void FrameGrabber::SetMVVBandpass(WORD wData)
{
    switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CAndOr(0x8A,0x06,0xCF,((wData & 0x03) << 4));
      break;
  case PHILIPS1 :
      I2CAndOr(0x42,0x06,0xCF,((wData & 0x03) << 4));
      break;
    }
}

void FrameGrabber::SetMVVAntiSnow(WORD wData)
{
    switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CAndOr(0x8A,0x06,0xF3,((wData & 0x03) << 2));
      break;
  case PHILIPS1 :
      I2CAndOr(0x42,0x06,0xF3,((wData & 0x03) << 2));
      break;
    }
}

void FrameGrabber::SetMVVSharpness(WORD wData)
{
    switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CAndOr(0x8A,0x06,0xFC,wData & 0x03);
      break;
  case PHILIPS1 :
      I2CAndOr(0x42,0x06,0xFC,wData & 0x03);
      break;
    }
}

void FrameGrabber::SetMVVChromaAGC(WORD wData)
{
    switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CAndOr(0x8A,0x0C,0x9F,((wData & 0x03) << 5));
      break;
  case PHILIPS1 :
      I2CAndOr(0x42,0x0C,0x9F,((wData & 0x03) << 5));
      break;
    }
}

void FrameGrabber::SetMVVVertNoise(WORD wData)
{
    switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CAndOr(0x8A,0x10,0xFC,wData & 0x03);
      break;
  case PHILIPS1 :
      I2CAndOr(0x42,0x10,0xFC,wData & 0x03);
      break;
    }
}

void FrameGrabber::SetMVVSECAMCross(WORD wData) {
  switch(wMVVDecoder) {
  case PHILIPS2 :
    I2CAndOr(0x8A,0x0F,0xDF,((wData & 0x01) << 5));
    break;
  case PHILIPS1 :
    I2CAndOr(0x42,0x0F,0xDF,((wData & 0x01) << 5));
    break;
  }
}

void FrameGrabber::SetMVVForceColor(WORD wData) {
  switch(wMVVDecoder) {
  case PHILIPS2 :
    I2CAndOr(0x8A,0x0C,0x7F,((wData & 0x01) << 7));
    break;
  case PHILIPS1 :
    I2CAndOr(0x42,0x0C,0x7F,((wData & 0x01) << 7));
      break;
  }
}

void FrameGrabber::SetMVVAntiGamma(WORD wData) {
  switch(wMVVDecoder) {
  case PHILIPS2 :
    I2CAndOr(0xB8,0x00,0x7F,((wData & 0x01) << 7));
    break;
  case PHILIPS1 :
    I2CAndOr(0x42,0x20,0x7F,((wData & 0x01) << 7));
    break;
  }
}

void FrameGrabber::SetMVVPrefilter(WORD wData) {
  switch(wMVVDecoder) {
  case PHILIPS2 :
    I2CAndOr(0x8A,0x06,0xBF,((wData & 0x01) << 6));
    break;
  case PHILIPS1 :
    I2CAndOr(0x42,0x06,0xBF,((wData & 0x01) << 6));
    break;
  }
}

void FrameGrabber::SetMVVHFilter(WORD wData) {
  switch(wMVVDecoder) {
  case PHILIPS2 :
    I2CAndOr(0xB8,0x04,0x1F,((wData & 0x07) << 5));
    break;
  case PHILIPS1 :
    I2CAndOr(0x42,0x24,0x1F,((wData & 0x07) << 5));
      break;
  }
}

void FrameGrabber::SetMVVVFilter(WORD wData) {
  switch(wMVVDecoder) {
  case PHILIPS2 :
    I2CAndOr(0xB8,0x08,0x9F,((wData & 0x03) << 5));
    break;
  case PHILIPS1 :
    I2CAndOr(0x42,0x28,0x9F,((wData & 0x03) << 5));
    break;
  }
}

void FrameGrabber::SetMVVColorKill(WORD wData) {
  switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CAndOr(0x8A,0x08,0x07,((wData & 0x1F) << 3));  /*QUAM*/
      I2CAndOr(0x8A,0x09,0x07,((wData & 0x1F) << 3));  /*SECAM*/
    break;
  case PHILIPS1 :
      I2CAndOr(0x42,0x08,0x07,((wData & 0x1F) << 3));  /*QUAM*/
      I2CAndOr(0x42,0x09,0x07,((wData & 0x1F) << 3));  /*SECAM*/
    break;
  }
}

void FrameGrabber::SetMVVSwSense(WORD wData) {
    switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CWrite(0x8A,0x0A,wData);  /*PAL*/
      I2CWrite(0x8A,0x0B,wData);  /*SECAM*/
      break;
  case PHILIPS1 :
      I2CWrite(0x42,0x0A,wData);  /*PAL*/
      I2CWrite(0x42,0x0B,wData);  /*SECAM*/
      break;
    }
}

void FrameGrabber::SetMVVChromaGain(WORD wData) {
    switch(wMVVDecoder) {
  case PHILIPS2 :
      I2CWrite(0x8A,0x11,wData);  /*QUAM*/
      break;
  case PHILIPS1 :
      I2CWrite(0x42,0x11,wData);  /*QUAM*/
      break;
    }
}

void FrameGrabber::SetMVVSpacialCompL(WORD wData)
{
    mvv_write(0x3B,wData);
}

void FrameGrabber::SetMVVSpacialCompH(WORD wData) {
    mvv_write(0x3A,wData);
}

void FrameGrabber::SetMVVFrameRate(WORD wData) {
    if((wVideoStandard==1)) {
      mvv_write(0x14,0x80 | (30/wData));
    } else mvv_write(0x14,0x80 | (25/wData));
    mvv_write(0x15,0x01);   /*Field Deci Num*/
}
/*  ***********************************************************************
        MVV Specific Handling
    ************************************************************************/

/* needed for direct setting*/
void FrameGrabber::MVVSetVert(unsigned char wDeciDen, unsigned char wDeciNum) {
    /* printf("vert %d %d\n", wDeciDen, wDeciNum);*/
    mvv_write(0x1C,wDeciDen); /*Vert Decimation Denominator*/
    mvv_write(0x1D,wDeciNum); /*Vert Decimation Numerator*/
}

void FrameGrabber::MVVSetVertDeci(WORD wDeciNum,WORD wDeciDen) {
  if((wDeciNum%5)==0) {
    wDeciDen /= 5; wDeciNum /= 5;
  }
  while(((wDeciNum%3)==0)&&((wDeciDen%3)==0)) {
    wDeciDen /= 3; wDeciNum /= 3;
  }
  while((((wDeciNum&0x0001)|(wDeciDen&0x0001))==0x0000)) {
    wDeciDen >>= 1;
    wDeciNum >>= 1;
  }
  while((wDeciDen>32)) {
    wDeciDen >>= 1;
    wDeciNum = (WORD)((wDeciNum+1)>>1);
  }
  if(wDeciDen==32) wDeciDen--;
  MVVSetVert(wDeciDen, wDeciNum);
}

void FrameGrabber::MVVSetHorzDeci(WORD wDeciNum, WORD wDeciDen) {
  if(wDeciNum<=512) {
    if((wDeciNum%5)==0) {
      wDeciDen /= 5;
      wDeciNum /= 5;
    }
  } else {
    wDeciNum = 512;
    wDeciDen = 640;     /*++Ideally we would do 768*/
  }

  while ( ((wDeciNum&0x0001) | (wDeciDen&0x0001)) == 0x0000) {
    wDeciDen >>= 1;
    wDeciNum >>= 1;
  }
  while((wDeciDen>32)) {
    wDeciDen >>= 1;
    wDeciNum = (WORD)((wDeciNum+1)>>1);
  }
  if(wDeciDen==32) wDeciDen--;
  /* printf("horz %d %d\n", wDeciDen, wDeciNum);*/
  mvv_write(0x24,0x80 | wDeciDen);  /* Horz Decimation Denominator*/
  mvv_write(0x25,wDeciNum);   /*Horz Decimation Numerator*/

}

/* new function : H <= 287 tested, above : not stable*/ 
/* different for SECAM / PAL ? (*** Secam not tested !!***)*/

void FrameGrabber::SetMVVResolution(WORD wWidth, WORD wHeight) {
  biWidth=wWidth;
  biHeight=wHeight;
  
  fg_height = (wHeight > VMAX) ? VMAX : wHeight;

  mvv_write(0x18, fg_height);    /* Cap Vert Len (L)*/
  mvv_write(0x19, fg_height>>8); /* Cap Vert Len (H)*/
    
  if (wVideoStandard == 1) {
      mvv_write(0x1A,0xFC);   /* Cap Vert Comp (L)*/
      mvv_write(0x1B,0x00);   /* Cap Vert Comp (H)*/
    if(wHeight > fg_height) MVVSetVertDeci(240,240);
    else MVVSetVertDeci(fg_height,240);
  } else {
      mvv_write(0x1A,0x1A);   /* Cap Vert Comp (L)*/
      mvv_write(0x1B,0x01);   /* Cap Vert Comp (H)_*/

    if (fg_height > 256) MVVSetVertDeci(270,270);
    else MVVSetVertDeci(fg_height,270);
  }
  /*printf("Vst %d ",wVideoStandard); -> 0*/
  mvv_write(0x12,0x00);     /*Cap Mem Page Ctrl: 0000h*/
  mvv_write(0x13,MVVMEMORYWIDTH);  /*Cap Mem Page Slct: 512 words*/

  mvv_write(0x42,0x00);     /*Disp Mem Page Ctrl: 0000h*/
  mvv_write(0x43,0x00);     /*Disp Mem Page Ctrl: 0000h*/
  mvv_write(0x44,MVVMEMORYWIDTH);  /*Disp Mem Page Slct: 512 words*/

  mvv_write(0x22, wWidth+8);   /*6/3: Hack around left edge*/
  mvv_write(0x23, (wWidth+8) >> 8);

  if(wVideoStandard == 1) MVVSetHorzDeci(wWidth, 640); 
  else MVVSetHorzDeci(wWidth, 768);
 

  mvv_write(0x30,mvv_read(0x30) & 0xFE);
  mvv_write(0x08,mvv_read(0x08) | 0x01);
  mvv_write(0x01,mvv_read(0x01) & 0xFD);  /*6/1:Hack to get compressed capture working*/
  mvv_write(0x32,0x00);
  mvv_write(0x33,MVVMEMORYWIDTH);
  
}


//////////////////////////////////////////////////////////////////////////
//////       Control functions


void FrameGrabber::all_defaults() {
	SetMVVFrameRate(10);	// Range: 1..30
	SetMVVVideoSource(1);	// Range: 0..1 ("Composite","S-video")
	SetMVVHStart(-35);		// Range: -128..127
	SetMVVVStart(20);		// Range: 0..255  /*NTSC vs PAL/SECAM?*/
	SetMVVHue(0);			// Range: 0..255
	SetMVVVideoStd(0);		// Range: 0..3 ("Auto-detect","NTSC","PAL","Secam")
	SetMVVVCRInput(1);		// Range: 0..1 ()
	SetMVVBandpass(3);		// Range: 0..3 ("Lowest Detail","Low Detail",
							//              "High Detail","Highest Detail")
	SetMVVAntiSnow(0);		// Range: 0..3 ("Disabled","1 LSB of 8-bit",
							//              "2 LSB of 8-bit","3 LSB of 8-bit")
	SetMVVSharpness(0);		// Range: 0..3 ("Weighting Factor 0","Weighting Factor 0.25",
							//              "Weighting Factor 0.5","Weighting Factor 1")
	SetMVVChromaAGC(0);		// Range: 0..3 ("Slow","Medium","Fast","Fixed")
	SetMVVVertNoise(0);		// Range: 0..3 ("Normal","Searching","Free-running","Bypass")
	SetMVVSECAMCross(0);	// Range: 0..1 ()
	SetMVVForceColor(0);	// Range: 0..1 ()
	SetMVVAntiGamma(1);		// Range: 0..1 ()
	SetMVVPrefilter(0);		// Range: 0..1 ()
	SetMVVHFilter(4);		// Range: 0..7 ("2-tap filter","3-tap filter","5-tap filter",
							//              "9-tap filter","Bypass","Bypass with delay",
							//				"8-tap filter","4-tap filter")
	SetMVVVFilter(0);		// Range: 0..3 ("Bypass","Delay","2-tap filter","3-tap filter")
	SetMVVColorKill(31);	// Range: 0..31 ()
	SetMVVSwSense(64);		// Range: 0..255 ()
	SetMVVChromaGain(89);	// Range: 0..255 ()
	SetMVVSpacialCompL(16);	// Range: 0..254 ()
	SetMVVSpacialCompH(128);// Range: 1..255 ()
    SetMVVResolution(biWidth, biHeight); // set resolution
    
}

void FrameGrabber::capture_old(CImage &image) {
  image.Reallocate(biWidth,biHeight);
  void *buf = image.map;
  int y,
		ww = biWidth, 	/* the width of the image */
		wh = biHeight, 	/* the heigth of the image */
		dinc, zz, dw, loops;
		
  short *dst = (short *) buf;  		/* cast output buffer */
  short *src = (short *) mvv_buf;	/* cast grabber buffer */
  
  dw = 2*ww;	   	/* dw is double the width */
  
  /* the following hack is to ensure proper timing */
  /* big images are grabbed in two passes */
  if (wh > 256) {	/* if height is greater than 256 (8 bits) */
	dinc = 2*ww; 		/* let increment be double the width */
	zz = wh/2; 			/* let # of iterations be half the height */
	loops = 2;  		/* make two passes */
  } else {			/* if height is SMALLER than 256 */
	dinc = ww; 			/* let increment be the width */
	zz = wh; 			/* let # of iterations be the height */
	loops = 1; 			/* make one pass */
  }

field_cap:  /* // capture one field */
  
  mvv_write(0x08,0x25); 	/* capture rgb555, init DRAM, PC enable */

  for (y = 0; y < zz; y++) {/* for each line or every other line */
    memcpy(dst,src, 16); 		/* discard 8 word */ 
    memcpy(dst,src, dw);		/* copy */
    dst += dinc; 				/* increment destination pointer */
    *src = 0;  					/* synchronisiert neue Zeile */
  }
  
  if (loops-- > 0) {  /* this might be a bug !! see in capture() */
    mvv_write(0x14, mvv_read(0x14) | 0xC0 ); /* capture odd fields (b7,b6)=(11) */
    dst = (short*) buf + ww; 				/* address odd lines */
    goto field_cap;
  }
  
  /* the next time capture even fields (b7,b6)=(10) */
  if (wh > 256) mvv_write(0x14, (mvv_read(0x14) | 0x80) & ~0x40); 
} 

void  FrameGrabber::capture(CImage &image, int rgb555) 
{
  image.Reallocate(biWidth,biHeight);
  void *buf = image.map;
  unsigned char r8;
  int y,				/* loop counter for data getting */
	  ww = biWidth,  	/* the width of the image */
	  wh = biHeight,    /* the heigth of the image */
	  dinc, 			/* increment for copying lines */
	  zz, 				/* number of lines to copy per pass */
	  dw = 2*ww, 		/* # of bytes per line */
	  loops;			/* # of passes to grab one image */
	  
  short *dst = (short *) buf;	  /* cast output buffer */
  short *src = (short *) mvv_buf; /* cast grabber buffer */
  
  /* the following hack is to ensure proper timing */
  /* big images are grabbed in two passes */
  /*  (~ interlaced vs non-interlaced) */
  if (wh > 256) {	/* if height is greater than 256 */
	dinc = 2*ww; 		/* let increment be double the width (every other line) */
	zz = wh/2; 			/* let # of iterations be half the height */
	loops = 2;  		/* make two passes */
  } else {			/* if height is SMALLER than 256 */
	dinc = ww; 			/* let increment be the width */
	zz = wh; 			/* let # of iterations be the height */
	loops = 1; 			/* make one pass */
  }
  
  /* set rgb mode */
  if (rgb555) r8  = 0x5 | 0x20;  /* rgb=555 mode */
  else        r8  = 0x5;         /* else use untranslated rgb = 565*/
  
field_cap:  /* capture one field*/
  
  mvv_write(0x08,r8); /* capture rgb555/565, init DRAM, PC enable*/

  /* note reading grabber memory changes this memory, too */
  /* here comes the data grabbing routine: */
  for (y = 0; y < zz; y++) {/* for each or every other line */
	memcpy(dst,src, 16); 		/* discard 8 words */
    memcpy(dst,src, dw); 		/* copy one line of pixels */
    dst += dinc; 				/* increment by one or two lines */
    *src = 0;  					/* sync new line of pixels*/
  }
  
  if (--loops > 0) {		/* if interlaced, get rest of lines */
	mvv_write(0x14, mvv_read(0x14) | 0xC0 );/* capture odd fields (b7,b6)=(11)   */
    dst = (short*) buf + ww; 				/* address odd lines*/
    goto field_cap;							/* do it again */
  }
  
  /* the next time capture even fields (b7,b6)=(10)*/
  if (wh > 256) mvv_write(0x14, (mvv_read(0x14) | 0x80) & ~0x40); 
} 

/* the image is in y direction stretched for the ratio ystretch/fg_heigth
 fg_height is the actually cpatured line number (global)
 (in pixels = 16bit), either rgb =  555 or 565*/
//void FrameGrabber::fast_capture(unsigned short *dst, int width, int ystretch, int rgb555) {
//  int y, dw = 2*width;
//  short *src = (short *) mvv_buf;
//
//  unsigned char r8 = 0x05; /* value for reg8*/ 
//  char tmp[dw+16]; /* using a temp buffer is faster than direct */
//  int cnt = 0;
//  if (rgb555) r8 |= 0x20; /* else use untranslated rgb = 565*/
//  mvv_write(0x08,r8); /* capture rgb555/565, init DRAM, PC enable*/
//
//  /* printf("%d %d %d %d %d %x %x\n",width,height,voff,nom,den,mvv_buf); */
//  
//  for (y = 0; y < fg_height; y++ ) {
//      *src = 0;  /* synchronisiert neue Zeile*/
//      memcpy(tmp, src, dw+16); /* discard 8 word*/ 
//    cnt -= ystretch;
//    while (cnt < 0) { 
//      cnt += fg_height;
//      memcpy(dst, tmp+16, dw);  dst += width;    
//    }
//  }
//}



 




