/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _FrameGrabber_h_
#define _FrameGrabber_h_

#include "CImage.h"

typedef unsigned char  BYTE;
typedef unsigned short  WORD;

typedef struct {
    BYTE  bSlave;
    BYTE  bSubAddr;
    BYTE  bData;
    BYTE  bHits;
} I2CInfoType;

class FrameGrabber  {
public:
	int biWidth;
	int biHeight;


	FrameGrabber(int biWidth_=80, int biHeight_=60);
	~FrameGrabber();
	
	// Control and access functions
	void all_defaults();
	void capture(CImage &image, int rgb555);
	void capture_old(CImage &image);
	
	// Configure frame grammer board
	void SetMVVVideoSource(WORD wData);
	void SetMVVHue(WORD wData);
	void SetMVVColor(WORD wData);
	void SetMVVContrast(WORD wData);
	void SetMVVBrightness(WORD wData);
	void SetMVVHStart(WORD wData);
	void SetMVVVStart(WORD wData);
	void SetMVVVideoStd(WORD wData);
	void SetMVVVCRInput(WORD wData);
	void SetMVVBandpass(WORD wData);
	void SetMVVAntiSnow(WORD wData);
	void SetMVVSharpness(WORD wData);
	void SetMVVChromaAGC(WORD wData);
	void SetMVVVertNoise(WORD wData);
	void SetMVVSECAMCross(WORD wData);
	void SetMVVForceColor(WORD wData); 
	void SetMVVAntiGamma(WORD wData);
	void SetMVVPrefilter(WORD wData);
	void SetMVVHFilter(WORD wData);
	void SetMVVVFilter(WORD wData);
	void SetMVVColorKill(WORD wData);
	void SetMVVSwSense(WORD wData);
	void SetMVVChromaGain(WORD wData);
	void SetMVVSpacialCompL(WORD wData);
	void SetMVVSpacialCompH(WORD wData);
	void SetMVVFrameRate(WORD wData);
	void MVVSetVert(unsigned char wDeciDen, unsigned char wDeciNum);
	void MVVSetVertDeci(WORD wDeciNum,WORD wDeciDen);
	void MVVSetHorzDeci(WORD wDeciNum, WORD wDeciDen);
	void SetMVVResolution(WORD, WORD);

private:	
	// Hardware access functions
	int  			mvv_init();
	void 			mvv_write(char index, char value);
	unsigned char 	mvv_read(char index);
	int 			I2CWrite(WORD bSlaveAddr, WORD bSubAddr,WORD bData);
	int 			I2CRead(WORD bSlaveAddr, WORD bSubAddr);
	void 			I2CAndOr(WORD bSlaveAddr, WORD bSubAddr,WORD bAND,WORD bOR);
	void 			print_iinfo();
	void 			test_i2c();
	char 			i2c_read(char slave, char addr);
	int 			i2cstat(unsigned char slave); /* .. 405ff*/
	
	// Memory access functions
	void *pmem_map(unsigned base, unsigned buf_size);

	// Instance variables
	int mvpi; /* the mvv port : index/ interrupt status*/
	int mvpd; /* data register*/
	int mvps ; /* irq/dma select*/
	int mvv_base; /* def = 0xc8000;  0xd0000;*/ 
	int iI2CCount;
	I2CInfoType i2cInfo[64];

	WORD  wMVVDecoder; /*1=Motorola, 2=Philips 2-chip,  3=Philips 3-chip*/
	WORD  wVideoStandard; /*0=Auto, 1=NTSC, 2=PAL, 3=SECAM*/
	int   iMVVBoardID;
	int   iMVVID;
	int fg_height; /* the actual captured line number */
	void *mvv_buf; /* frame buffer pointer */

};


#endif

