/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*//* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include "Lists.h"
#include <assert.h>
#include <stdio.h>


/////////////////////////////////////////////////////////////////
// keeps list ordered according to priority "prio"

PriorityQueue::PriorityQueue()
{
}

PriorityQueue::~PriorityQueue()
{
	DeleteAll();
}

boolean	PriorityQueue::Enqueue(ISobject *newelem, float prio)
{
	// find place to insert
	PQElem *pqptr=NULL;
	Reset();
	while (NULL!=(pqptr = (PQElem*)List::Iterate())) {
		if (pqptr->priority >= prio) {
			return Insert(new PQElem(newelem,prio));
		}
	}
	return Append(new PQElem(newelem,prio));
}

ISobject *PriorityQueue::Dequeue(float &prio)
{
	PQElem *op = (PQElem*)Decap();
	if (op==NULL) return NULL;		// leaves prio unaltered !
	ISobject *obj = op->obj;
	prio = op->priority;
	delete op;
	return obj;
}

ISobject *PriorityQueue::Find(ISobject *cand)
{
	// verbatim copy of List::Find, but using PQ's Iterate (which is not virtual)
	ISobject *co;
	Reset();
	while (NULL!=(co = Iterate())) 
		if (co == cand) return cand;
	return NULL;
}

ISobject *PriorityQueue::Remove(ISobject *cand)
{
	ISobject	*retval;
	if (NULL!=(retval=Find(cand))) 
		delete (PQElem*)RemoveCurrent();
	return retval;
}

ISobject *PriorityQueue::FindFirst(void)
{
	PQElem *f = ((PQElem*)List::FindFirst());
	if (f) return f->obj;
	else return NULL;
}

ISobject *PriorityQueue::Iterate(void)
{
	PQElem *f = ((PQElem*)List::Iterate());
	if (f) return f->obj;
	else return NULL;
}

void	PriorityQueue::Print(void)
{
	// print all priorities
	Reset();
	PQElem *pptr;
	printf("\n---------Priority Queue with %d elements ---------------------\n",Size());
	while (NULL!=(pptr = (PQElem*)List::Iterate())) 
		printf("Prio %f at addr %u\n",pptr->priority,(unsigned)(pptr->obj));
	printf("\n-----------------------------------------------------------------\n");
}

/////////////////////////////////////////////////////////////////

List::List()
{
	next_iter=first=previous=current=last=NULL;
	size=0;
}

List::~List()
{
	Clear();
}

List &List::operator = (List &org)
{
	ISobject *to;
	Clear();
	org.Reset();
	while (NULL!=(to=org.Iterate())) Append(to);
	return *this;
}

boolean	List::Append(ISobject *newelem)
{
	if (newelem==NULL) return FALSE;  // do not accept NULL elemetns
	
	if (first==NULL) {	
		next_iter=first=current=last = new AListElem;
		previous=NULL;
	} else {
		last->next = new AListElem;
		last       = last->next;
		if (next_iter==NULL) next_iter=last;
	}
	last->next = NULL;
	last->obj  = newelem;
	size++;
	return TRUE;
}

boolean	List::Prepend(ISobject *newelem)
{
	if (newelem==NULL) return FALSE;  // do not accept NULL elemetns
	
	if (first==NULL) {	
		next_iter=first=current=last = new AListElem;
		previous = NULL;
		first->obj  = newelem;
		first->next = NULL;
	} else {
		AListElem *ntmp = new AListElem;
		ntmp->next  = first;
		ntmp->obj   = newelem;
		if (current==first) previous=ntmp;
		first       = ntmp;
	}
	size++;
	return TRUE;
}

boolean	List::Insert(ISobject *newelem)
{
	if (newelem==NULL) return FALSE;  // do not accept NULL elemetns
	
	// check for special cases
	if (current==first || current==NULL)	{
		return Prepend(newelem);
	}
	
	// make sure pointers are consistent
	assert(previous);
	assert(previous->next==current);
	
	// insert the new element before the current element
	AListElem *nptr = new AListElem;
	nptr->obj 		= newelem;
	nptr->next		= current;
	previous->next	= nptr;
	previous		= nptr;
	
	size++;
	return TRUE;
}

ISobject *List::Decap(void) 
{
	ISobject *retobject;

	if (first==NULL) return NULL; // zero elements
	
	if (current  ==first) current=first->next;
	if (next_iter==first) next_iter=first->next;
	if (previous ==first) {
		previous=NULL;
		assert(first->next==current);
	}
	
	if (first==last) {			// only one element
		retobject = first->obj;
		delete first;
		next_iter=first=last=previous=current=NULL;
	} else {					// many elements
		AListElem *newfirst = first->next;
		retobject = first->obj;
		delete first;
		first=newfirst;
	}
	assert(size>0);
	size--;
	return retobject;
}

void	List::Reset(void) 
{
	previous=NULL;
	next_iter=current=first;
}

ISobject	*List::Iterate(void)
{
	if (next_iter==NULL) return NULL;
	else {
		if (current!=next_iter) {
			previous = current;
			current  = next_iter;
		}
		next_iter = next_iter->next;
		return current->obj;
	}
}

boolean	List::IsLast(void)
{
	return (current==last);
}

ISobject *List::Remove(ISobject *cand)
{
	Find(cand);
	return RemoveCurrent();
}

ISobject *List::RemoveCurrent(void)
{
	ISobject	*retval;
	if (current) {
		retval = current->obj;
		if (current==last) 		last=previous;
		if (current==first)		first=current->next;
		if (current==next_iter)	next_iter=current->next;
		if (previous)			previous->next=current->next; 
		AListElem *next = current->next;
		delete current;
		current = next;
		size--;
		Reset();
		return retval;
	} else {
		return NULL;		// could not be found
	}
}

ISobject *List::Replace(ISobject *cand, ISobject *newob)
{
	if (! newob) return NULL;	// can't replace with NULL
	ISobject	*retval;
	Find(cand);
	if (current) {
		retval = current->obj;
		current->obj = newob;
		return retval;
	} else {
		return NULL;		// could not be found
	}
}

ISobject *List::Find(ISobject *cand)
{
	ISobject *co;
	Reset();
	while (NULL!=(co = Iterate())) 
		if (co == cand) return cand;
	return NULL;
}

ISobject *List::FindFirst(void)
{
	Reset();
	if (!current) return NULL;
	return current->obj;
}

ISobject *List::FindLast(void)
{
	// the loop is only done so that "previous" is set correctly !
	Reset();
	while (Iterate());					// iterate to end of list
	assert(current==last);				// current must now be the last
	return FindCurrent();
}

ISobject *List::FindCurrent(void)
{
	if (!current) return NULL;
	return current->obj;
}

ISobject *List::Get(unsigned int pos)
{
	unsigned int cnt=0;
	
	Reset();
	while (Iterate()) {
		if (cnt==pos) return current->obj;
		cnt++;
	}
	return NULL;
}

void List::Clear(void)
{
	current=first;
	while (current!=NULL) {
		AListElem *next = current->next;
		delete current;
		current = next;
	}

	next_iter=first=current=last=previous=NULL;
	size=0;
}

void List::DeleteAll(void)
{
	current=first;
	while (current!=NULL) {
		AListElem *next = current->next;
		delete current->obj;	// delete the content
		delete current;			// delete the container
		current = next;
	}

	next_iter=first=current=last=previous=NULL;
	size=0;
}


void List::Reverse(void)
{
	// save old state for later
	AListElem *new_current = current;
	AListElem *new_previous;
	if (current==NULL)	new_previous = NULL;
	else				new_previous = current->next;
	
	previous=NULL;
	current=first;
	
	while (current!=NULL) {
		AListElem *next = current->next;
		current->next = previous;
		previous = current;
		current = next;
	}

	last=first;
	first=previous;
	current		= new_current;
	previous	= new_previous;
}








