/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _Lists_h_
#define _Lists_h_

/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include "defines.h"

//---- List Elements ---------------------------------------------------

class ISobject {
public:
	// abstract class for all to inherit from (for lists)
	ISobject(){};
	virtual ~ISobject(){};
};

class AListElem : public ISobject{
friend class List;
friend class Stack;
protected:
	ISobject	*obj;
	AListElem	*next;
};

class PQElem : public AListElem {
friend class PriorityQueue;
	PQElem(ISobject *o, float p):priority(p){obj=o;};
private:
	float		priority; // priority of the object
};

class List : public ISobject {
public:
	List();
	~List();
	
	List			&operator = (List &org);	// copy all contents
	boolean			Append(ISobject *newelem);	// append to end of list
	boolean			Prepend(ISobject *newelem);	// append to beginning of list
	boolean			Insert(ISobject *newelem);  // insert object at current place
	ISobject		*Decap(void);				// remove and return first element
	ISobject		*Remove(ISobject *cand);	// remove and return an element
	ISobject		*RemoveCurrent(void);		// remove and return an element
	ISobject		*Replace(ISobject *cand, ISobject *newob);	// like remove, but add new object in place
	ISobject		*Find(ISobject *cand);	// find and return an element
	ISobject		*FindFirst(void);		// get first element of list
	ISobject		*FindLast(void);		// get last element of list
	ISobject		*FindCurrent(void);		// get current element of list
	ISobject		*Get(unsigned int pos);	// find and return an element, using its position number >=0
	unsigned int	Size(void) {return size;};
	void			Reset(void);
	ISobject		*Iterate(void); 
	boolean			IsLast(void);	// is this the last element of the list ?
	void			Clear(void);	 // destroy list, don't delete objects in it
	void			DeleteAll(void); // delete all objects of list
	void			Reverse(void);   // reverse the order of the list
protected:
	AListElem		*first,*current,*previous,*last,*next_iter;
	unsigned int	size;
};

class Stack : public List {			// an alias for list
public:
	boolean			Push(ISobject *newelem) {return Prepend(newelem);};	// alias for prepend
	ISobject		*Pop(void) 				{return Decap();};			// alias for decap
};

class Queue : public List {			// an alias for list
public:
	boolean			Enqueue(ISobject *newelem)	{return Append(newelem);};	// alias for append
	ISobject		*Dequeue(void)				{return Decap();};			// alias for Pop
};

class PriorityQueue : public List {			// a list with priorities
public:
	PriorityQueue();
	~PriorityQueue();
	boolean			Enqueue(ISobject *newelem, float prio);
	ISobject		*Dequeue(float &prio);	// return first obj, and its priority
	ISobject		*FindFirst(void);
	ISobject		*Iterate(void);
	ISobject		*Find(ISobject *cand);	// find and return an element
	ISobject		*Remove(ISobject *cand);	// remove and return an element
	void			Print();
	
	// ATTENTION: CAVEAT: all other functions returning a ISobject, do not
	// actually return the object, but the PQElement that points to it!
	// Likewise, DeleteAll is equivalent to Clear of list, so don't use
	// these functions.
	// Actually, PriorityQueue should not really inherit from List, but
	// I was too lazy to do it otherwise.
};

#endif	/* _Lists_h_ */





