/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _LocalMap_h_
#define _LocalMap_h_

/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include "CImage.h"
#include "RobotProxy.h"
#include "PrgStruct.h"

#define CELLSIZE_X		0.15
#define CELLSIZE_Y  	0.15
#define LMSIZE_CX		40
#define LMSIZE_CY		40
#define LMSIZE_X		LMSIZE_CX*CELLSIZE_X
#define LMSIZE_Y		LMSIZE_CY*CELLSIZE_Y
#define OFFSET_CX		LMSIZE_CX/2
#define OFFSET_CY		LMSIZE_CY/2

#define MIN_CERT		0
#define MAX_CERT		15
#define INI_CERT		0
#define	LM_DEC			1
#define LM_INC			3
#define NEIGHBOR_SCALE 	2
#define CLUSTER_THRES  	30
#define IND_THRES	  	7

#define SONAR_RAD   	0.21
#define MAX_SONAR_R 	4.0
#define MIN_SONAR_R 	0.41

//---- LocalMap -----------------------------------------------------------

class LocalMap : public Action {
friend class CImage;
public:
	LocalMap(RobotProxy *robot_p);
	~LocalMap();
	
	void	Init(RobotProxy *robot_p);
	virtual Data *Step(void);	  // update the cell array (position, sensor input)
	void	Move(void);
	void	Fuse(void);
	void	Cluster(void);
	void	FreeSpace(int size);
	void	ConfLine(int sx, int sy, int ex, int ey);
	void	ConfPoint(int px, int py);
	void	ConfPointMIN(int px, int py);
	void	ConfPointMAX(int px, int py);
	void	ConfEndPoint(int px, int py);
	
private:
	float	cpos_x, cpos_y, cpos_a;		// position of center of map
	float	apos_x,apos_y,apos_a;		// position of center of agent
	float	dx,dy,da;					// delta agent-cell array
	unsigned char	cell[LMSIZE_CX][LMSIZE_CY];	// array of certainties
	unsigned char	ccell[LMSIZE_CX][LMSIZE_CY];	// array of squared certs
	unsigned char	sqcell[LMSIZE_CX][LMSIZE_CY];	// array of squared certs
	unsigned char	fcell[LMSIZE_CX][LMSIZE_CY];	// array of squared certs
	
	RobotProxy	*robot;	// pointer to the robot's data for position and sensors
};

#endif	/* _LocalMap_h_ */
