/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*//* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */


#include "Monsup.h"
#include <stdio.h>


////////////////////////////////////////////////////////////////////////
// if argument-marker is registered

RegisteredQ::RegisteredQ(Steppable *mark_)
	:mark(NULL)
{
	mark = Getport(mark_);
}

Data *RegisteredQ::Step(void)
{
	if (!mark) return &fuzz_false;
	Data *rmark = mark->Step();
	if (rmark==NULL) return &fuzz_false;
	if ((rmark->type>DT_MARKER_MIN) && (rmark->type<DT_MARKER_MAX)) {
		if (((Marker*)rmark)->IsRegistered()) return &fuzz_true;
		else return &fuzz_false;
	} else {
		return &fuzz_false;
	}
}


////////////////////////////////////////////////////////////////////////

Data *Register::Step(void)
{
	if (!registrar || !checker) {
		done.SetTrue();
		return NULL;
	}
	
	Data *retval;
	
	// if not registered, try to register
	if (! registered)	{
		retval = registrar->Step();
		done.SetFalse();
		if (retval) {
			if (retval->type <=DT_MARKER_MAX && retval->type>=DT_MARKER_MIN) {
				most_recent = (Marker*)retval;
				if (most_recent->IsRegistered()) {
					registered = TRUE;
					checker->Reset();
					done.SetTrue();
				}
				return most_recent;
			}
		} 
		return NULL;
	} else {
	// if already registered, check whether it should stay registered
		retval = checker->Step();
		done.SetTrue();
		if (retval) {
			if (retval->type <=DT_MARKER_MAX && retval->type>=DT_MARKER_MIN) {
				most_recent = (Marker*)retval;
				// is it a marker ?
				if (! (((Marker*)retval)->IsRegistered())) {
					registered = FALSE;
					registrar->Reset();
					done.SetFalse();
				}
				return most_recent;
			} else if (retval->type==DT_Fuzz) {
				// or is it a Fuzz ? In this case true indicates END OF REGISTRATION !!
				if (((Fuzz*)retval)->True()) {
					registered=FALSE;
					registrar->Reset();
					done.SetFalse();
				}
				return most_recent;	// is used, not written on this cycle
			}
		} 
		registered = FALSE;
		registrar->Reset();
		return NULL;
	}
}

void Register::Reset(void)
{
	if (registrar) registrar->Reset();
	if (checker) checker->Reset();
	registered	=	FALSE;
	most_recent =	NULL;
	done.SetFalse();
}

void Register::Abort(void)
{
	if (registrar) registrar->Abort();
	if (checker) checker->Abort();
	registered	=	FALSE;
	most_recent =	NULL;
	done.SetTrue();
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////



Data	*ShowVisual::Step(void)
{
	done.Set(FUZZTRUE);
	// do some checking
	if (! vis_marker) return NULL;
	Data *vm = vis_marker->Step();
	if (!vm) return NULL;
	if (vm->type<=DT_VISMIN || vm->type>=DT_VISMAX) return NULL;
	VisualMarker *rvm = (VisualMarker*)vm;
	
	// get and display the image
	CImage *img = rvm->GetImage();
	if (img) outbuf.MakeGray4from5(*img).Show(cx,cy);
	return rvm;
}

Data	*SaccadeTo::Step(void)
{
	const float panfact = -1.2/180.0*3.1415926; // 0.020943951
	const float tiltfact= -1.2/180.0*3.1415926; // 0.020943951

	done.Set(FUZZTRUE);
	if (!robot) return NULL;
	VisualPoint *rvm;
	GETVALRET(VisualPoint,rvm,vis_point);	// returns from function upon error
	
	// get the data and execude the saccade
	CImage *img;
	boolean registered;
	float pan,tilt;
	rvm->Get(img,pan,tilt,registered);
	if (registered) robot->MoveCameraPos(pan*panfact,tilt*tiltfact);
	return rvm;
}

Data	*MonitorCam::Step(void)
{
	const int thres=8;	// threshold used for determining moving stuff

	done.Set(FUZZTRUE);
	
	if (!robot) return NULL;
	
	prev_image = curr_image;	// store actu
	curr_image = robot->GetCameraImage();
	
	// compute difference image
	diff_image.ADiff(curr_image,prev_image);
	
	// find region of motion
	unsigned int xmin,xmax,ymin,ymax;
	signed int xctr,yctr;
	boolean exists;
	bbox_image.BoundingBox(diff_image,xmin,xmax,ymin,ymax,xctr,yctr,exists,thres);
	
	// set and return the marker to the object
	target.Set(&bbox_image,-xctr,yctr,exists);
	return &target;
}

