/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*//* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */


#include "Packet.h"
#include "defines.h"
#include <string.h>
#include <stdio.h>
#include <iostream.h>

#define POUT_PACK_FACTOR   1000.0 
#define SONAR_PACK_FACTOR  6000.0
#define JOY_PACK_FACTOR   20000.0
#define POS_PACK_FACTOR     100.0
#define ATT_PACK_FACTOR    1000.0 
#define DRT_PACK_FACTOR      10.0 
#define DRA_PACK_FACTOR      10.0
#define BATT_PACK_FACTOR    255.0
#define TS_PACK_FACTOR    10000.0
#define AS_PACK_FACTOR     1000.0
#define CP_PACK_FACTOR     1000.0
#define CA_PACK_FACTOR    10000.0
#define DOCK_PACK_FACTOR    255.0

#define MAX_CMDS		    300

#define REPLACE_CRNL		'@'
#define MSGDELIMITER		';'


/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
////////////////////////  IPC WRAPPERS      /////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

#ifdef __QNX__

#include <sys/sendmx.h>
#include <sys/kernel.h>

word MsgLength(byte *buf)
{
        return (word)((*buf)+((*(buf+1))<<8));
}

int SendLen(pid_t pid, void *smsg, void *rmsg, 
			word snbytes, word rnbytes, word &arnbytes)
{
	byte len[2];
	struct _mxfer_entry smx[2];
	struct _mxfer_entry rmx[2];

	len[0]  = (byte)(((word)snbytes) & 0xFF);
	len[1]  = (byte)((((word)snbytes) & 0XFF00) >> 8);
	_setmx(&smx[0],len,(word)2);
	_setmx(&smx[1],smsg,snbytes);

	_setmx(&rmx[0],len,(word)2);
	_setmx(&rmx[1],rmsg,rnbytes);

	arnbytes=0;
	if (-1==Sendmx(pid,2,2,smx,rmx)) return(-1);
	arnbytes=MsgLength(len);
	return 0;
}			
			
int ReceiveLen(pid_t pid, void *msg,word nbytes, word &anbytes)
{
	byte len[2];
	struct _mxfer_entry mx[2];

	_setmx(&mx[0],len,(word)2);     // add packet length
	_setmx(&mx[1],msg,nbytes);

	pid_t rpid=Receivemx(pid,2,mx);
	if (rpid>0) anbytes=MsgLength(len);
	else        anbytes=0;
	return rpid;
}

int CReceiveLen(pid_t pid, void *msg,word nbytes, word &anbytes)
{
	byte len[2];
	struct _mxfer_entry mx[2];

	_setmx(&mx[0],len,(word)2);     // add packet length
	_setmx(&mx[1],msg,nbytes);

	pid_t rpid=Creceivemx(pid,2,mx);
	if (rpid>0) anbytes=MsgLength(len);
	else        anbytes=0;
	return rpid;
}

int ReplyLen(pid_t pid, void *msg, word nbytes)
{
	byte len[2];
	struct _mxfer_entry mx[2];

	len[0]  = (byte)(((word)nbytes) & 0xFF);  // add packet length
	len[1]  = (byte)((((word)nbytes) & 0XFF00) >> 8);

	_setmx(&mx[0],len,(word)2);
	_setmx(&mx[1],msg,nbytes);

	return Replymx(pid,2,mx);
}
	
#endif


/////////////////////////////////////////////////////////////////////////

 
//---- InstructionReplyPacket ---------------------------------------------------------

InstructionReplyPacket::InstructionReplyPacket():nchars(0)
{
	next = text = new char [MAX_PROTOCOL_LEN+10];
	Clear();
} 

InstructionReplyPacket::~InstructionReplyPacket()
{
	delete [] text;
}

void InstructionReplyPacket::Clear(void)
{
	nchars=0;
	next=text;
	text[0]=0;
}

void InstructionReplyPacket::Append(char *newtxt)
{ 
	word nlen = (word)strlen(newtxt);
	if (nlen+nchars>=MAX_PROTOCOL_LEN) {
		cout << "WARNING: reply packet does not fit buffer, reply ignored\n";
		return;
	}
	strcpy(next,newtxt);
	nchars+=nlen;
	next  +=nlen;
}

word InstructionReplyPacket::Marshal(byte *buf)
{
	if (buf==NULL) return 0;
	word len=0;
	ADDBBUF(data_protocol);	// add magic number
	ADDWBUF(nchars);
	if (nchars>0) strncpy((char*)buf,text,nchars+1);
	return (word)(nchars+len);
}

word InstructionReplyPacket::Unmarshal(byte *buf)
{
	if (buf==NULL) return 0;
	word len=0;
	byte id;

	text[0]=0;
	GETBBUF(id,buf);
	if (id!=data_protocol) return 0; // do not read if magic number not correct
	GETWBUF(nchars,buf);
	if (nchars<=0) {return len;};
	strncpy(text,(char*)buf,nchars+1);
	text[nchars]=0;	// add a null character
	return (word)(nchars+len);
}
//---- StatPacket ---------------------------------------------------------

word StatPacket::MarshalASCII(byte *buf, float comm_rate, float line_qual, char *ttext, int maxlen)
{
	if (buf==NULL) return 0;
	char *thebuf = (char*)buf;
	int len=0;
	
	sprintf(thebuf,
		"%d %d %4.2f %4.2f %4.2f %4.2f %4.2f %4.2f %4.2f %4.2f %4.2f %4.2f %4.2f %4.2f %3.1f %3.1f %4.2f %3.1f %d %d %d %d %d %d ",
//		"%d %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %d %d %d %d %d %d ",
		behavior_running,
		mode,
		tspeed,
		aspeed,
		sonar[0], sonar[1], sonar[2], sonar[3], sonar[4], sonar[5], sonar[6], sonar[7],
		pos_x,
		pos_y,
		attitude,
		comm_rate,
		line_qual,
		compass,
		light[0], light[1], light[2], light[3], light[4], light[5]);
	
	if (strlen(ttext)>=1) {
		len = strlen(thebuf);					// length of data string
		int remaining = maxlen - len - 2;		// how much space is left?
		if (remaining>0) {
			thebuf[len] = MSGDELIMITER;				// insert delimiter
			len++;
			strncpy(&thebuf[len],ttext,remaining);	// copy the stuff to the buffer
			thebuf[maxlen-1]=0;						// make sure the thing is terminated

			// replace all new lines / carriage returns with another character
			char *ptr = thebuf;						// prepare pointers for search
			char *loc;								// location of found new line char
			while (NULL!=(loc=strchr(ptr,(int)10))) { // search for new line character
				*loc = (char)REPLACE_CRNL;			// replace it with something else
				ptr = loc;							// restart search
			}
			ptr = thebuf;
			while (NULL!=(loc=strchr(ptr,(int)11))) { // search for new line character
				*loc = (char)REPLACE_CRNL;			// replace it with something else
				ptr = loc;							// restart search
			}
			ptr = thebuf;
			while (NULL!=(loc=strchr(ptr,(int)13))) { // search for new line character
				*loc = (char)REPLACE_CRNL;			// replace it with something else
				ptr = loc;							// restart search
			}
		}
	}
	
	len = strlen(thebuf);					// determine new length
	thebuf[len]='\n';						// append a new line character
	len++;									// update length
	thebuf[len]=0;							// null-terminate the string
		
	return len;
}

word StatPacket::Marshal(byte *buf)
{
	if (buf==NULL) return 0;
	word len=0;
	int i;
	
	ADDBBUF(data_status);	// add magic number
	for (i=0;i<NO_SONARS;i++) 	ADDWBUF((word)(sonar[i]*SONAR_PACK_FACTOR));
	ADDIBUF((signed short int)(pos_x*POS_PACK_FACTOR));	
	ADDIBUF((signed short int)(pos_y*POS_PACK_FACTOR));	
	ADDIBUF((signed short int)(attitude*ATT_PACK_FACTOR));	
	ADDWBUF((word)(drt*DRT_PACK_FACTOR));
	ADDWBUF((word)(dra*DRA_PACK_FACTOR));
	ADDBBUF((byte)(battery*BATT_PACK_FACTOR));
	ADDIBUF((signed short int)(tspeed*TS_PACK_FACTOR));
	ADDIBUF((signed short int)(aspeed*AS_PACK_FACTOR));
	for (i=0;i<NO_LIGHTS;i++) 	ADDWBUF(light[i]);
	ADDWBUF(light50);
	ADDWBUF(bump_stat);
	ADDBBUF(radar);
	ADDBBUF(pir);
	ADDBBUF(behavior_running);
	ADDIBUF((signed short int)(compass*CP_PACK_FACTOR));
	ADDIBUF((signed short int)(camera_pan*CA_PACK_FACTOR));
	ADDIBUF((signed short int)(camera_tilt*CA_PACK_FACTOR));
	ADDBBUF((byte)(soundno));
	ADDBBUF((byte)(mode));
	ADDBBUF((byte)(dock_pos*DOCK_PACK_FACTOR));
	return len;
}

word StatPacket::Unmarshal(byte *buf)
{
	word len=0;
	byte id,dummy;
	int i;
	
	GETBBUF(id,buf); 
	if (id!=data_status) return 0; // do not read if magic number is not correct
	
	for (i=0;i<NO_SONARS;i++) {
		GETWBUF(sonar[i],buf);
		sonar[i]/=SONAR_PACK_FACTOR;
	}
	GETIBUF(pos_x,buf);		pos_x    /= POS_PACK_FACTOR;
	GETIBUF(pos_y,buf); 	pos_y    /= POS_PACK_FACTOR;
	GETIBUF(attitude,buf);	attitude /= ATT_PACK_FACTOR;
	GETWBUF(drt,buf);		drt		 /= DRT_PACK_FACTOR;
	GETWBUF(dra,buf);		dra		 /= DRA_PACK_FACTOR;
	GETBBUF(battery,buf);	battery	 /= BATT_PACK_FACTOR;
	GETIBUF(tspeed,buf);	tspeed	 /= TS_PACK_FACTOR;
	GETIBUF(aspeed,buf);	aspeed	 /= AS_PACK_FACTOR;
	for (i=0;i<NO_LIGHTS;i++) 		GETWBUF(light[i],buf);
	GETWBUF(light50,buf);
	GETWBUF(bump_stat,buf);
	GETBBUF(radar,buf);
	GETBBUF(pir,buf);
	GETBBUF(behavior_running,buf);
	GETIBUF(compass,buf);		compass	 /= CP_PACK_FACTOR;
	GETIBUF(camera_pan,buf);	camera_pan	 /= CA_PACK_FACTOR;
	GETIBUF(camera_tilt,buf);	camera_tilt	 /= CA_PACK_FACTOR;
	GETBBUF(dummy,buf); soundno = (sound_t)dummy;
	GETBBUF(mode,buf);
	GETBBUF(dock_pos,buf);		dock_pos /= DOCK_PACK_FACTOR;
	return len;
}

//---- AckPacket ---------------------------------------------------------

word AckPacket::Marshal(byte *buf)
{
	if (buf==NULL) return 0;
	word len=0;
	ADDBBUF(data_ack);	// add magic number
	return len;
}

word AckPacket::Unmarshal(byte *buf)
{
	word len=0;
	byte id;
	GETBBUF(id,buf);
	if (id!=data_ack) return 0; // do not read if magic number not correct
	return len;
}


//---- CmdPacket ---------------------------------------------------------

CmdPacket::CmdPacket()
{
	cmd			= cmd_error;		// virtual class only !
	reply_code	= rly_send_nothing;
}

word CmdPacket::Marshal(byte *buf)
{
	if (buf==NULL) return 0;
	word len=0;
	
	ADDBBUF(cmd);
	ADDBBUF(reply_code);
	return len;
}

word CmdPacket::Unmarshal(byte *buf)
{
	if (buf==NULL) return 0;
	word len=0;

	GETBBUF(cmd,buf);
	GETBBUF(reply_code,buf);
	return len;
}

word UnmarshalAny(byte *buf, CmdPacket *&newp)
{
	word len=0;
	CmdPacket	*newob = NULL;
	switch (*buf) {
		case	cmd_noop: 			newob=new NoopPacket; break;
		case	cmd_set_pid:		newob=new PidPacket; break;
		case	cmd_set_sound:		newob=new SoundPacket; break;
		case	cmd_set_speech:		newob=new SpeechPacket; break;
		case	cmd_set_velocity:	newob=new SpeedPacket; break;
		case	cmd_set_drpos:		newob=new DRPosPacket; break;
		case	cmd_set_drdist:		newob=new DRDistPacket; break;
		case	cmd_set_campos:		newob=new CamPacket; break;
		case	cmd_set_mode:		newob=new ModePacket; break;
		case	cmd_set_battery:	newob=new BatteryPacket; break;
		case	cmd_set_dockpos:	newob=new DockPacket; break;
		case	cmd_composite:		newob=new CompositePacket; break;
		case	cmd_start_stop:		newob=new StartStopPacket; break;
		case	cmd_list_term:		newob=NULL;	
									break;
		default:
			cerr << "Invalid CmdPacket code (Packet.cc: UnmarshalAny) "
				 << (int)*buf << "\n";	
			newp=NULL;
			return 0;
	}
	if (newob!=NULL) {
		len = newob->Unmarshal(buf);
	}

	newp=newob;		
	return len;
}

#define MARSHAL_PARENT(code)	cmd = (code); word len=CmdPacket::Marshal(buf); \
								 if (len==0) return 0; buf += len;
// unmarshal header, fails if magic number is not correct (ie. !=code)
#define UNMARSHAL_PARENT(code)	word len=CmdPacket::Unmarshal(buf); buf+=len; \
								if (cmd!=code) return 0;

//---- InstructionPacket ---------------------------------------------------------

InstructionPacket::InstructionPacket()
{
	cmd	= cmd_instr;
}

word	InstructionPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_instr);
	ADDWBUF(textlen);
	if (textlen>0) {
		char *bufptr = (char*)buf;
		strncpy(bufptr,text,textlen+1);
	}
	return (word)(textlen+len);
}

word	InstructionPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_instr);
	GETWBUF(textlen,buf);
	if (textlen<=0) return 2;
	char *bufptr = (char*)buf;
	strncpy(text,bufptr,textlen+1);
	text[textlen]=0;	// add a null character
	return (word)(textlen+len);
}
	
word 	InstructionPacket::Store(const char *txt)
{
	if ((textlen=(word)strlen(txt))>SPK_TXTLEN) {
		cout << "ERROR: text to be sent as instruction is too long\n";
		return 0;
	}
	strcpy(text,txt);
	return textlen;
}

void 	InstructionPacket::Clear(void)
{
	textlen=0;
}
	
//---- SpeechPacket ---------------------------------------------------------

SpeechPacket::SpeechPacket()
{
	cmd	= cmd_set_speech;
}

word	SpeechPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_speech);
	ADDWBUF(textlen);
	if (textlen>0) {
		char *bufptr = (char*)buf;
		strncpy(bufptr,text,textlen+1);
	}
	return (word)(textlen+len);
}

word	SpeechPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_speech);
	GETWBUF(textlen,buf);
	if (textlen<=0) return 2;
	char *bufptr = (char*)buf;
	strncpy(text,bufptr,textlen+1);
	text[textlen]=0;	// add a null character
	return (word)(textlen+len);
}
	
word 	SpeechPacket::Store(const char *txt)
{
	if ((textlen=(word)strlen(txt))>SPK_TXTLEN) {
		cout << "ERROR: text to be spoken is too long\n";
		return 0;
	}
	strcpy(text,txt);
	return textlen;
}

void 	SpeechPacket::Clear(void)
{
	textlen=0;
}
	
//---- PidPacket ---------------------------------------------------------

PidPacket::PidPacket()
{
	cmd	= cmd_set_pid;
}

word PidPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_pid);
	ADDBBUF(cmd);
	ADDWBUF((word)(pout*POUT_PACK_FACTOR));
	ADDBBUF(p);
	ADDBBUF(i);
	ADDBBUF(d);
	ADDBBUF(cnt);
	return len;
}

word PidPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_pid);
	GETBBUF(cmd,buf);
	GETWBUF(pout,buf);  pout /= POUT_PACK_FACTOR;
	GETBBUF(p,buf);
	GETBBUF(i,buf);
	GETBBUF(d,buf);
	GETBBUF(cnt,buf);
	return len;
}

//---- SoundPacket ---------------------------------------------------------

SoundPacket::SoundPacket()
{
	cmd	= cmd_set_sound;
}

word SoundPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_sound);
	ADDBBUF(sound_no);
	return len;
}

word SoundPacket::Unmarshal(byte *buf)
{
	byte dummy;
	UNMARSHAL_PARENT(cmd_set_sound);
	GETBBUF(dummy,buf); sound_no = (sound_t)dummy;
	return len;
}

//---- SpeedPacket ---------------------------------------------------------

SpeedPacket::SpeedPacket()
{
	cmd	= cmd_set_velocity;
}

word SpeedPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_velocity);
	ADDIBUF((signed short int)(t*TS_PACK_FACTOR));
	ADDIBUF((signed short int)(a*AS_PACK_FACTOR));
	ADDBBUF(joymode);
	return len;
}

word SpeedPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_velocity);
	GETIBUF(t,buf);	t /= TS_PACK_FACTOR;
	GETIBUF(a,buf);	a /= AS_PACK_FACTOR;
	GETBBUF(joymode,buf);
	return len;
}

//---- DRPosPacket ---------------------------------------------------------

DRPosPacket::DRPosPacket()
{
	cmd	= cmd_set_drpos;
}

word DRPosPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_drpos);
	ADDIBUF((signed short int)(x*POS_PACK_FACTOR));	
	ADDIBUF((signed short int)(y*POS_PACK_FACTOR));	
	ADDIBUF((signed short int)(a*ATT_PACK_FACTOR));	
	return len;
}

word DRPosPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_drpos);
	GETIBUF(x,buf);	x /= POS_PACK_FACTOR;
	GETIBUF(y,buf); y /= POS_PACK_FACTOR;
	GETIBUF(a,buf);	a /= ATT_PACK_FACTOR;
	return len;
}

//---- ModePacket ---------------------------------------------------------

ModePacket::ModePacket()
{
	cmd	= cmd_set_mode;
}

word ModePacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_mode);
	ADDBBUF(mode);
	return len;
}

word ModePacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_mode);
	GETBBUF(mode,buf);
	return len;
}

//---- CamPacket ---------------------------------------------------------

CamPacket::CamPacket()
{
	cmd	= cmd_set_campos;
}

word CamPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_campos);
	ADDIBUF((signed short int)(pan*CA_PACK_FACTOR));
	ADDIBUF((signed short int)(tilt*CA_PACK_FACTOR));
	return len;
}

word CamPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_campos);
	GETIBUF(pan,buf);	pan		 /= CA_PACK_FACTOR;
	GETIBUF(tilt,buf);	tilt	 /= CA_PACK_FACTOR;
	return len;
}

//---- DockPacket ---------------------------------------------------------

DockPacket::DockPacket()
{
	cmd	= cmd_set_dockpos;
}

word DockPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_dockpos);
	ADDBBUF((byte)(amount*DOCK_PACK_FACTOR));
	return len;
}

word DockPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_dockpos);
	GETBBUF(amount,buf);	amount /= DOCK_PACK_FACTOR;
	return len;
}

//---- BatteryPacket ---------------------------------------------------------

BatteryPacket::BatteryPacket()
{
	cmd	= cmd_set_battery;
}

word BatteryPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_battery);
	ADDBBUF((byte)(level*BATT_PACK_FACTOR));
	return len;
}

word BatteryPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_battery);
	GETBBUF(level,buf);	level /= BATT_PACK_FACTOR;
	return len;
}

//---- DRDistPacket ---------------------------------------------------------

DRDistPacket::DRDistPacket()
{
	cmd	= cmd_set_drdist;
}

word DRDistPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_set_drdist);
	ADDWBUF((word)(drt*DRT_PACK_FACTOR));
	ADDWBUF((word)(dra*DRA_PACK_FACTOR));
	return len;
}

word DRDistPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_set_drdist);
	GETWBUF(drt,buf);		drt		 /= DRT_PACK_FACTOR;
	GETWBUF(dra,buf);		dra		 /= DRA_PACK_FACTOR;
	return len;
}

//---- StartStopPacket ---------------------------------------------------------

StartStopPacket::StartStopPacket()
{
	cmd	= cmd_start_stop;
}

word StartStopPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_start_stop);
	ADDBBUF(item);
	ADDBBUF(start);
	return len;
}

word StartStopPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_start_stop);
	GETBBUF(item,buf);
	GETBBUF(start,buf);
	return len;
}



//---- NoopPacket ---------------------------------------------------------

NoopPacket::NoopPacket()
{
	cmd	= cmd_noop;
}

word NoopPacket::Marshal(byte *buf)
{
	MARSHAL_PARENT(cmd_noop);
	return len;
}

word NoopPacket::Unmarshal(byte *buf)
{
	UNMARSHAL_PARENT(cmd_noop);
	return len;
}

//---- CompositePacket ---------------------------------------------------------

CompositePacket::CompositePacket(void)
{
	cmd	= cmd_composite;
	Clear();
}

word CompositePacket::Marshal(byte *buf)
{

	MARSHAL_PARENT(cmd_composite);
	CmdPacket *ptr = First();
	while (ptr != NULL) {
		word inc = ptr->Marshal(buf);
		buf+=inc; len+=inc;
		ptr = Next();
	}
	*buf = cmd_list_term;	// add a terminating flag
	len++;
	return len;
}

word CompositePacket::Unmarshal(byte *buf)
{
	// deleting the objects in the list is the responsibility
	// of the user of this class
	// this method CREATES new objects !!
	Clear();
	UNMARSHAL_PARENT(cmd_composite);
	CmdPacket	*newob = NULL; 
	do {
		word inc = UnmarshalAny(buf,newob);
		if ((inc>0)&&(newob->cmd!=cmd_composite)) {
			len+=inc; buf+=inc;
			if (AddUnique(newob)==FALSE) 
				delete newob; // this is ok, since newob was allocated in UnmarshalAny
		} else {
			delete newob;
			newob=NULL;
		}
	} while (newob!=NULL);
		
	return len;
}

void CompositePacket::Clear(void)
{
	first = last = current = NULL;
	no_items = 0;
}

void CompositePacket::Delete(void)
{
	CmdPacket *curr = First();
	while (curr) {
		CmdPacket *to_delete = curr;
		curr = Next();
		delete to_delete;
	}
	Clear();
}

boolean CompositePacket::AddUnique(CmdPacket *new_pack)
{
	if (new_pack==NULL) return FALSE;
		
	CmdPacket *ptr		= first;
	CmdPacket *previous = ptr;
	
	while (NULL!=ptr) {
		if (new_pack==ptr) return FALSE; // this packet was already added
		else {
			previous=ptr;
			ptr=ptr->next;
		}
	}
	
	// if no equal one was found, add it to the list
	if (first==NULL) {
		first=last=current=new_pack;
	} else {
		last->next = new_pack;
		last = new_pack;
	}
	no_items++;
	new_pack->next=NULL;
	
	return TRUE;
}

CmdPacket	*CompositePacket::First(void) 
{
	current=first;
	return first;
}

CmdPacket	*CompositePacket::Next(void)  
{
	if (current==NULL) return NULL;
	current=current->next;
	return current;
}

inline word CompositePacket::NoItems(void)
{
	return no_items;
}

