/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _PrgStruct_h_
#define _PrgStruct_h_

/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include "defines.h"
#include "utils.h"
#include "Lists.h"
#include "TimeArith.h"
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

class Fuzz;
class Marker;
class Data;
class Steppable;
class Dock;
class Parameter;
extern Fuzz fuzz_true;
extern Fuzz fuzz_false;
extern Fuzz fuzz_unknown;
extern Fuzz fuzz_evidence;
extern Fuzz fuzz_in_progress;
extern Fuzz chain_exit;

// The following value interpretations hold for Fuzz variables:
//
// [-0.5...0]   FALSE
// [1.0...}     TRUE
// {0.0...1.0}  Evidence exists or progress (proportional to value)
// -1.0         UNKNOWN
// -2.0			Currently trying to determine value, don't act otherwise

#define FUZZDETERMINING	-2.0
#define FUZZUNKNOWN		-1.0
#define FUZZFALSE		 0.0
#define FUZZEVIDENCE	 0.5
#define FUZZTRUE		 1.0

///////////////////////////////////////////////////////////////////////////////
// Useful macros to manage run time type information
// The DataType must match the type name !!!!!

#define GETVALRET(TYPE,VAR,STEPPABLE)	\
	{if (STEPPABLE==NULL)	return NULL;\
	Data *ret___val = STEPPABLE->Step(); \
	if (ret___val==NULL) return NULL; \
	if (ret___val->type != DT_ ## TYPE ) return NULL; \
	(VAR) = ( TYPE * ) ret___val;}
	
#define GETVAL(TYPE,VAR,STEPPABLE)	\
	{Data *ret___val;\
	if (STEPPABLE==NULL)	(VAR) = NULL;\
	else {ret___val = STEPPABLE->Step(); \
	if (ret___val==NULL) (VAR) = NULL; \
	else if (ret___val->type != DT_ ## TYPE ) (VAR) = NULL; \
	else {(VAR) = ( TYPE * ) ret___val;};};}
	
#define SAFECAST(TYPE,P)	\
	(((P)==NULL)?(NULL):(((P)->type==( DT_ ## TYPE ))?(( TYPE *)P):(NULL)))
	
#define TESTTYPE(TYPE,P)	\
	(((P)==NULL)?(FALSE):(((P)->type==( DT_ ## TYPE ))?(TRUE):(FALSE)))
	

// The names of these enums must be constructed according to the following
// rule:
//
//      DT_ plus name of class to which it corresponds, e.g. for class
//      "Number" the enum name is DT_Number	
	
enum DataType {
			DT_Fuzz,DT_Number,DT_Text,DT_DataList, DT_Variable, 
			
			// Waypoint navigation stuff (are these markers?)
			DT_Waypoint, DT_Route,DT_HiwayMap,DT_Wall,DT_WallEstimate,
			/////////////////////////////////////////////////////////
			// Misc. Markers
			DT_MARKER_MIN,
			
			// floor markers
			DT_FLOOR_MIN,
			DT_FloorLine,
			DT_FloorMarker,
			DT_FloorRay,
			DT_FloorSector,
			DT_FloorRect,
			DT_FloorCircle,
			DT_FLOOR_MAX,
			
			// visual markers
			DT_VISMIN,
			DT_VisualPoint,
			DT_VISMAX,
			
			DT_MARKER_MAX,
			// dummy for max index
			DT_MAXINDEX};

#define CHAIN (&chain_exit)

class Steppable : public ISobject {
public:
	Steppable():chain_dock(NULL){};
	virtual ~Steppable();
	virtual Data	*Step(void){return NULL;};
	virtual Fuzz	*IsDone(void){return NULL;}
	virtual	void	Abort(void){};
	virtual void	Reset(void){};
	
	virtual	void		SetChain(Steppable *chain=NULL);
	virtual Steppable	*GetChain(void);
	Steppable 			*Getport(Steppable *p);
	
	virtual unsigned int GetNoParameters(void) {printf("STEPPABLE\n");return 0;};
	boolean			Test(Steppable *tester, boolean &determining, double thres=FUZZTRUE);
protected:
	Fuzz	*fuzz_res;	// result of last Test operation
	Dock	*chain_dock;// dock to chain in more Steppables in UP
};


///////////////////////////////////////////////////////////////////////////////
//  Data: Markers and Fuzz

// The Data base class	
	
class Data : public Steppable {
public:
	Data(DataType type_=DT_Number):type(type_){};
	virtual	Data	*Step(void){return this;};	// data just returns itself
	virtual Fuzz	*IsDone(void);
	virtual Data	*Duplicate(void)=0;
	virtual boolean	SetData(Data *org_)=0;
	virtual Data	*Print(void)=0;
	DataType		type;
};


// wrapper class for generic List class
class DataList : public Data {
public:
	DataList(List *org_=NULL):Data(DT_DataList){if (org_) Set(org_);};
	DataList(DataList *org_):Data(DT_DataList){if (org_) Set(&(org_->content));};
	
	// manipulation
	void	Set(List *org_){
		if (!org_) return; 
		Clear();
		org_->Reset();
		while (content.Append(org_->Iterate()));
	};
	void	Set(DataList *org_){if (org_) Set(&(org_->content));};
	
	virtual boolean	SetData(Data *org_){
		if (!org_) return FALSE;
		if (org_->type!=type) return FALSE;
		Set((DataList*)org_);
		return TRUE;
	};

	void	Append(Data *newe_){(void)content.Append(newe_);};
	void	ResetList(void){content.Reset();};
	Data	*Iterate(void){return (Data*)content.Iterate();}; 
	void	Clear(void){content.Clear();};
	virtual Data	*Duplicate(void){return new DataList(this);};
	virtual Data	*Print(void){
		Data *c;
		printf("DataList(");
		content.Reset();
		c=(Data*)content.Iterate();
		while(c) {
			if (c) c->Print();
			c=(Data*)content.Iterate();
			if (c) printf(",");
		}
		printf(")");
		return this;
	};

private:
	List	content;
};




class Number : public Data {
public:
	Number(float initval=0.0):val(initval),Data(DT_Number){};
	
	// manipulation
	void	Set(float newval) 	{val=newval;};
	void	Set(Number *newnum)	{if (newnum) val=newnum->val;};
	float	Get(void)			{return val;};

	virtual boolean	SetData(Data *org_){
		if (!org_) return FALSE;
		if (org_->type!=type) return FALSE;
		Set((Number*)org_);
		return TRUE;
	};

	Number	&operator += (Number &x){val+=x.val;return *this;};
	Number	&operator *= (Number &x){val*=x.val;return *this;};
	Number	&operator /= (Number &x){val/=x.val;return *this;};
	Number	&operator -= (Number &x){val-=x.val;return *this;};
	Number	&operator  = (Number &x){val =x.val;return *this;};
	Number	&operator  = (double x) {val =x;return *this;};
	virtual Data	*Duplicate(void){return new Number(val);};
	virtual Data	*Print(void){printf("Number(%f)",val);return this;};
private:
	float	val;
};

#define MAX_TEXT_LEN 1000
class Text : public Data {
public:
	Text(const char *text_=NULL);
	~Text(){delete [] text;};
	
	// manipulation
	void	Set(const char *text_=NULL);
	void	Set(Text *org_){if (org_) Set(org_->text);};
	char	*Get(void) {return text;};

	virtual boolean	SetData(Data *org_){
		if (!org_) return FALSE;
		if (org_->type!=type) return FALSE;
		Set((Text*)org_);
		return TRUE;
	};

	void	Clear(void);
	virtual Data	*Duplicate(void){return new Text(text);};
	virtual Data	*Print(void){printf("Text(%s)",text?text:"");return this;};
private:
	char *text;
	word textlen;
};

class Variable : public Data {
public:
	Variable(const char *name_,Data &content_);
	Variable(const char *name_=NULL,Data *content_=NULL);
	~Variable(){delete content;};
	
	// manipulation
	Variable &operator =(Data &content_){Set(&content_);return *this;};
	Variable &operator =(Data *content_){Set(content_);return *this;};
	void	Set(Data *content_);
	virtual boolean	SetData(Data *org_){org_=org_;return FALSE;} // assign to silence warnings

	virtual	Data	*Step(void){return content;};	// data just returns itself
	virtual Data	*Duplicate(void);
	virtual Data	*Print(void)
		{printf("Variable(");if (content) content->Print();printf(")");return this;};
private:
	Data	*content;	// pointer to content
	Text	name;		
};

class Fuzz : public Data {
public:
	Fuzz(float initval=FUZZFALSE):Data(DT_Fuzz),val(initval){};
	
	// manipulation
	void	SetTrue(void)		{Set(FUZZTRUE);};
	void	SetFalse(void)		{Set(FUZZFALSE);}
	void	SetUnknown(void)	{Set(FUZZUNKNOWN);};
	void	SetDetermining(void){Set(FUZZDETERMINING);};
	void	SetEvidence(void)	{Set(FUZZEVIDENCE);};
	void	Set(Number &newval) {val=newval.Get();};
	void	Set(float newval) 	{val=newval;};
	void	Set(Fuzz *newval) 	{val=newval->Get();};
	float	Get(void)			{return val;};
	float	Not(void)			{return (val=Limit(FUZZTRUE-val,FUZZFALSE,FUZZTRUE));};
	float	And(float andval)	{return (val=Limit(Min(andval,val),FUZZFALSE,FUZZTRUE));};
	float	Or(float andval)	{return (val=Limit(Max(andval,val),FUZZFALSE,FUZZTRUE));};
	
	virtual boolean	SetData(Data *org_){
		if (!org_) return FALSE;
		if (org_->type!=type) return FALSE;
		Set((Fuzz*)org_);
		return TRUE;
	};

	// test for certain values
	boolean	FTrue(void) 		{return val>=0.5;};
	boolean	FFalse(void)		{return ((val<=0.5)&&(val>-0.5));};
	boolean	True(void) 			{return val>=1.0;};
	boolean	False(void)			{return ((val<=0.0)&&(val>-0.5));};
	boolean	Unknown(void)		{return ((val>-1.5)&&(val<-0.5));};
	boolean Determining(void)	{return ((val>-2.5)&&(val<-1.5));};
	boolean Evidence(void)		{return ((val> 0.0)&&(val< 1.0));};
	boolean	Gt(float tstval)	{return val>=tstval;};
	boolean Lt(float tstval)	{return val<=tstval;};

	virtual Data	*Duplicate(void){return new Fuzz(val);};
	virtual Data	*Print(void){printf("Fuzz(%f)",val);return this;};
private:
	float	val;
};

///////////////////////////////////////////////////////////////////////////////
//  TreeBase

class TreeBase : public Steppable {
public:
	TreeBase(
		Steppable *before_ = NULL,	// step these when committing the tree
		Steppable *while_  = NULL,	// step this on each update
		Steppable *after_  = NULL,	// step this upon removal
		Parameter *p1 = NULL,		// these are the parameters
		Parameter *p2 = NULL,
		Parameter *p3 = NULL,
		Parameter *p4 = NULL,
		Parameter *p5 = NULL,
		Parameter *p6 = NULL,
		Parameter *p7 = NULL,
		Parameter *p8 = NULL,
		Parameter *p9 = NULL);
	~TreeBase(){delete param_list;}
	
	// Pass all messages along to contained tree		
	virtual	Data	*Step(void)
						{return ((while_tree)?while_tree->Step():NULL);};	
	virtual Fuzz	*IsDone(void)
						{return ((while_tree)?while_tree->IsDone():&(fuzz_true));};	
	virtual	void	Abort(void)
						{if (while_tree) while_tree->Abort();};
	virtual void	Reset(void)
						{if (while_tree) while_tree->Reset();};
						
	// Handle committment and termination of this tree
	// These functions are called my the SupMgr
	void	Commit(void)   {if (before_tree) {before_tree->Reset();before_tree->Step();}};
	void	Terminate(void){if (after_tree) {after_tree->Reset();after_tree->Step();}};

	// Manage the parameter list and trees
	void 		SetBeforeTree(Steppable *t) {before_tree=t;}
	void 		SetWhileTree(Steppable *t) {while_tree=t;}
	void 		SetAfterTree(Steppable *t) {after_tree=t;}
	void		AppendParameters(
					Parameter *p1 = NULL,
					Parameter *p2 = NULL,
					Parameter *p3 = NULL,
					Parameter *p4 = NULL,
					Parameter *p5 = NULL,
					Parameter *p6 = NULL,
					Parameter *p7 = NULL,
					Parameter *p8 = NULL,
					Parameter *p9 = NULL);
					
	Parameter	*AppendParameter(Parameter* p); 	// returns the appended parameter
	Steppable	*GetParameter(unsigned int pos);	// returns parameter # no [1,2,...]
	Steppable	*SetParameter(unsigned int pos, Steppable *p);// sets parameter # no
	virtual unsigned int GetNoParameters(void);

private:
	Steppable	*before_tree;
	Steppable	*while_tree;
	Steppable	*after_tree;
	List		*param_list;	// list of registered parameters of the tree
};

///////////////////////////////////////////////////////////////////////////////
//  Parameter contains a SUP tree and passes all requests to it

#define PAR(P)    Parameter *P = new Parameter()	
#define PARD(P,D) Parameter *P = new Parameter(D)

class Parameter : public Steppable {
public:
	Parameter(Steppable *default_tree=NULL)
		:the_tree(default_tree){};
	
	// Manage the SUP tree
	void		Set(Steppable *new_tree=NULL) {the_tree=new_tree;};
	Steppable	*Get(void) {return the_tree;}
	
	// Pass all messages along to contained tree		
	virtual	Data	*Step(void)
						{return ((the_tree)?the_tree->Step():NULL);};	
	virtual Fuzz	*IsDone(void)
						{return ((the_tree)?the_tree->IsDone():&(fuzz_true));};	
	virtual	void	Abort(void)
						{if (the_tree) the_tree->Abort();};
	virtual void	Reset(void)
						{if (the_tree) the_tree->Reset();};
protected:
	Steppable	*the_tree;		// the decision tree, assembled SUPs
};

////////////////////////////////////////////////////////////////////////////////////////////
//
//	 A C T I O N
//
////////////////////////////////////////////////////////////////////////////////////////////

class Action : public Steppable {
public:
	virtual Fuzz	*IsDone(void){return &done;}
protected:
	Fuzz done;
};

class Ramp : public Action {
public:
	Ramp(time_t msecs_=0L);
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
private:
	RealTime	rt;
	time_t		msecs;
	time_t		start_time;
	Fuzz		retval;
};

class Pulsetrain : public Action {
public:
	Pulsetrain(time_t msecs_hi_,time_t msecs_lo_);
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
private:
	RealTime	rt;
	time_t		msecs_lo;
	time_t		msecs_hi;
	time_t		msecs_to;
	time_t		start_time;
	Fuzz		retval;
};

class LogOp : public Action {
public:
	LogOp(	Steppable *t1,
			Steppable *t2=NULL,
			Steppable *t3=NULL,
			Steppable *t4=NULL,
			Steppable *t5=NULL,
			Steppable *t6=NULL,
			Steppable *t7=NULL,
			Steppable *t8=NULL,
			Steppable *t9=NULL);
	
	void Append(Steppable *t);
	virtual	void	Abort(void);
	virtual void	Reset(void);
protected:
	List	tlist;
	Fuzz	retval;
};

class And : public LogOp {
public:
	And(	Steppable *t1,
			Steppable *t2,
			Steppable *t3=NULL,
			Steppable *t4=NULL,
			Steppable *t5=NULL,
			Steppable *t6=NULL,
			Steppable *t7=NULL,
			Steppable *t8=NULL,
			Steppable *t9=NULL);
	
	virtual	Data	*Step(void);
};

class Or : public LogOp {
public:
	Or(		Steppable *t1,
			Steppable *t2,
			Steppable *t3=NULL,
			Steppable *t4=NULL,
			Steppable *t5=NULL,
			Steppable *t6=NULL,
			Steppable *t7=NULL,
			Steppable *t8=NULL,
			Steppable *t9=NULL);
	
	virtual	Data	*Step(void);
};

class Not : public LogOp {
public:
	Not(Steppable *t=NULL);
	virtual	Data	*Step(void);
};

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
//  Scheduler

enum ProducerPriority {	SENSOR_PP,	WORLDMODEL_PP,	PROPENSITY_PP, 
						UP_PP,		
						ARBITER_PP,		MOTOR_PP,	ACTUATOR_PP,
						MAX_PP};

class Scheduler : public Action {
public:
	Scheduler(word interval_=0); 
	~Scheduler();
	 
	// functions to control execution
	void	SetInterval(word interval_=0){interval=interval_;};
	void	WaitSlice(void);	// wait for next tick
	void	Run(void);
	
	// functions to add steppables
	void	SetPriority(ProducerPriority pp); 
	void	Remove(Steppable *oldprod);
	void	Remove(ProducerPriority pp, Steppable *oldprod);
	void	Add(Steppable *newprod);
	void	Add(ProducerPriority pp, Steppable *newprod);
	
	virtual Data	*Step(void);

private:
	List				*producer_list;
	ProducerPriority	current_pp;
	Countdown			cnt;
	word				interval;
};

///////////////////////////////////////////////////////////////////////////////
//  Program Structures

class Noop : public Action {
public:
	Noop(void){done.Set(FUZZTRUE);};	
	virtual	Data	*Step(void){printf("NOOPing\n"); return NULL;};
};

class Exit : public Action {
public:
	Exit(void){done.Set(FUZZTRUE);};	
	virtual	Data	*Step(void){printf("Exiting..\n"); exit(0); };
};

class Debug : public Action {
public:
	Debug(char * text_, float nr_=0.0):nr(nr_),text(text_){done.Set(FUZZTRUE);};	
	virtual	Data	*Step(void)
		{printf("Debug (%f): %s\n",nr,text); return NULL;};
protected:
	void Init(void){};
	float nr;
	char *text;
};

class Every : public Action {
public:
	Every(unsigned long interval_,Steppable *action_);	
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
	virtual	Fuzz	*IsDone(void);
protected:
	Data			*last_retval;	// last returned value 
	unsigned long	interval;		// run it every n-th time
	unsigned long	counter;		// tick counter
	Steppable		*action;		// achieve this
};

class EveryTime : public Every {
public:
	EveryTime(unsigned long interval_,Steppable *action_);	
	virtual	Data	*Step(void);
private:
	time_t 	next_time;		// next event
	RealTime	time_base;	// real time source
};

class Setq : public Action {
public:
	Setq(	Variable *var_,
			Steppable 	*action_=NULL)
			:var(var_),par(NULL){action=Getport(action_);};
	Setq(	Parameter *par_,
			Steppable 	*action_=NULL)
			:var(NULL),par(par_){action=Getport(action_);};
	virtual	Data	*Step(void);
	virtual	void	Reset(void){if (action) action->Reset();};
	virtual	void	Abort(void){if (action) action->Abort();};
	virtual	Fuzz	*IsDone(void){if (action) return action->IsDone();else return &fuzz_true;};
private:
	Variable	*var;			// variable to set
	Parameter	*par;			// or a parameter
	Steppable	*action;		// achieve this
};

class Setparam : public Action {
public:
	Setparam(	Parameter *par_,
			Steppable 	*action_=NULL)
			:par(par_){action=Getport(action_);};
	virtual	Data	*Step(void);
	virtual	void	Reset(void){if (action) action->Reset();};
	virtual	void	Abort(void){if (action) action->Abort();};
	virtual	Fuzz	*IsDone(void){if (action) return action->IsDone();else return &fuzz_true;};
private:
	Parameter	*par;			// variable to set
	Steppable	*action;		// achieve this
};

class Withparam : public Action {
public:
	Withparam(	TreeBase *tree_, 
				Parameter *par1_,     Steppable *act1_,
				Parameter *par2_=NULL,Steppable *act2_=NULL,
				Parameter *par3_=NULL,Steppable *act3_=NULL,
				Parameter *par4_=NULL,Steppable *act4_=NULL,
				Parameter *par5_=NULL,Steppable *act5_=NULL
				)
			:tree(tree_),
			par1(par1_),act1(act1_),
			par2(par2_),act2(act2_),
			par3(par3_),act3(act3_),
			par4(par4_),act4(act4_),
			par5(par5_),act5(act5_)
			{
				assert(par1);
				assert(act1);
				act1=Getport(act1_);
				act2=Getport(act2_);
				act3=Getport(act3_);
				act4=Getport(act4_);
				act5=Getport(act5_);
			};
	virtual	Data	*Step(void);
	virtual	void	Reset(void){if (tree) tree->Reset();};
	virtual	void	Abort(void){if (tree) tree->Abort();};
	virtual	Fuzz	*IsDone(void){if (tree) return tree->IsDone();else return &fuzz_true;};
private:
	Parameter	*par1,*par2,*par3,*par4,*par5;	// variable to set
	Steppable	*act1,*act2,*act3,*act4,*act5;	// achieve this
	Steppable	*tree;			// the tree to use
};

class If : public Action {
public:
	If(		Steppable	*cond_, 
			Steppable 	*action_, 
			Steppable 	*else_action_=NULL,
			double	threshold_=FUZZEVIDENCE);
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
private:
	Steppable	*cond;			// termination condition
	Steppable	*action;		// achieve this
	Steppable	*else_action;	// called when cond is false
	double		threshold;		// use this to decide for TRUE
};


// return FALSE X times, then return TRUE eternally
class DoTimes : public Action {
public:
	DoTimes(unsigned long times_=0)
		:tim(times_),cnt(0L){done.Set(FUZZFALSE);};
	virtual	Data	*Step(void){if(++cnt>tim) done.Set(FUZZTRUE);return &done;};
	virtual	void	Reset(void){cnt=0L;done.Set(FUZZFALSE);};
	virtual	void	Abort(void){cnt=tim+1;done.Set(FUZZTRUE);};
private:
	unsigned long	tim;	// how many times should I do it?	
	unsigned long	cnt;	// how many times did I do it?
};

// return FALSE for INTERVAL msecs, then return TRUE eternally
class DoInterval : public Action {
public:
	DoInterval(time_t msecs_=0):len(msecs_){Reset();};
	virtual	Data	*Step(void); 
	virtual	void	Reset(void);
	virtual	void	Abort(void);
private:
	time_t		end;	// end point of interval
	time_t		len;	// length of interval
	RealTime	time_base;
};



class Timeout : public Action {
public:
	Timeout(Steppable *step_action_, time_t msecs, Steppable *t_action_=NULL);
	virtual	Data	*Step(void)=0;
	virtual	void	Reset(void){last_time=0;if (s_action) s_action->Reset();};
	virtual	void	Abort(void){if (s_action) s_action->Abort();};
	virtual	Fuzz	*IsDone(void){if (s_action) return s_action->IsDone();else return &fuzz_true;};
protected:
	Steppable	*s_action;		// step this on each step
	Steppable	*t_action;		// reset/abort... this on timeout or timein
	time_t		interval;		// Abort if not stepped for this many msecs
	time_t		last_time;		// last time I was stepped (yeah, step me, baby)
	RealTime	time_base;
};

class TimeoutAbort : public Timeout {
public:
	TimeoutAbort(Steppable *step_action_, time_t msecs, Steppable *t_action_=NULL)
		:Timeout(step_action_,msecs,t_action_){};
	virtual	Data	*Step(void);
};

class TimeoutReset : public Timeout {
public:
	TimeoutReset(Steppable *step_action_, time_t msecs, Steppable *t_action_=NULL)
		:Timeout(step_action_,msecs,t_action_){};
	virtual	Data	*Step(void);
};

class Timein : public Timeout {
public:
	Timein(Steppable *step_action_, time_t msecs_interval_, time_t msecs_recover_, Steppable *t_action_=NULL)
		:Timeout(step_action_,msecs_interval_,t_action_),recover(msecs_recover_),start_time(0){};
	virtual	Data	*Step(void)=0;
protected:
	time_t	recover;	// time to recover from stepping
	time_t	start_time;	// time when this whoosh of stepping started	
};

class TimeinReset : public Timein {
public:
	TimeinReset(Steppable *step_action_, time_t msecs_interval_, time_t msecs_recover_, Steppable *t_action_=NULL)
		:Timein(step_action_,msecs_interval_,msecs_recover_,t_action_){};
	virtual	Data	*Step(void);
};

class TimeinAbort : public Timein {
public:
	TimeinAbort(Steppable *step_action_, time_t msecs_interval_, time_t msecs_recover_, Steppable *t_action_=NULL)
		:Timein(step_action_,msecs_interval_,msecs_recover_,t_action_){};
	virtual	Data	*Step(void);
};

class TimedIf : public Action {
public:
	TimedIf(	Steppable	*cond_, 
				Steppable 	*action_, 
				time_t		msecs=0,
				Steppable 	*else_action_=NULL,
				boolean	retrigger_=FALSE,
				double	threshold_=FUZZEVIDENCE);
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
private:
	Steppable	*cond;			// termination condition
	Steppable	*action;		// achieve this
	Steppable	*else_action;	// called when cond is false
	double		threshold;		// use this to decide for TRUE
	boolean		last_test;		// last test result by cond
	time_t		unbounce;		// time when switch falls back
	time_t		interval;
	boolean		retrigger;		// is condition evaluated and triggering every time?
	RealTime	time_base;
};

class StickyIf : public Action {
public:
	StickyIf(	Steppable	*cond_, 
				Steppable 	*action_, 
				Steppable 	*else_action_=NULL,
				double	threshold_=FUZZEVIDENCE);
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
private:
	Steppable	*cond;			// termination condition
	Steppable	*action;		// achieve this
	Steppable	*else_action;	// called when cond is false
	double		threshold;		// use this to decide for TRUE
	boolean		sticking;		// is it currently sticking ?
};

class Debounce : public Action {
public:
	Debounce(	Steppable	*action_,
				time_t		msecs=0,
				Steppable	*alt_action_=NULL);	// time in milliseconds
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
	virtual	Fuzz	*IsDone(void);
private:
	Steppable	*action;		// achieve this
	Steppable	*alt_action;	// alternative action (when debouncing)
	time_t		unbounce;		// time when switch falls back
	time_t		interval;
	RealTime	time_base;
};

class ResetStep : public Action {
public:
	ResetStep(Steppable	*action_=NULL);
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
	virtual	Fuzz	*IsDone(void);
private:
	Steppable	*action;		// achieve this
};

class AbortStep : public Action {
public:
	AbortStep(Steppable	*action_=NULL);
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
	virtual	Fuzz	*IsDone(void);
private:
	Steppable	*action;		// achieve this
};


class IsDoneStep : public Action {
public:
	IsDoneStep(Steppable	*action_=NULL);
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
	virtual	Fuzz	*IsDone(void);
private:
	Steppable	*action;		// achieve this
};

class SupMgr;

class RemoveSup : public Action {
public:
	RemoveSup(SupMgr *mgr_,Steppable	**action_ptr_)
		:action_ptr(action_ptr_),supmgr(mgr_){
			assert(action_ptr && supmgr);
			done.Set(FUZZTRUE);
		};
	virtual	Data	*Step(void);
private:
	Steppable	**action_ptr;		// achieve this
	SupMgr		*supmgr;	
};

class Dock : public Action {
friend class SupMgr;
public:
	Dock(SupMgr *mgr, char *name=NULL, 
		 Steppable *new_exit=NULL, boolean is_default=FALSE);
	Dock();
	~Dock();
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
	virtual	Fuzz	*IsDone(void);
	
	// overwrite because Dock is itself a chain
	virtual	void	SetChain(Steppable *chain=NULL);
	virtual Steppable	*GetChain(void);

	// Dock-Specific functions
	Steppable		*GetExit(void);
	void			SetExit(Steppable *new_exit);
	void			ClearExit(void);
	void			SetAchievedCond(Steppable *cond=NULL);
	
	// Naming functions
	void	SetName(char *txt);
	boolean	IsName(char *txt);
	char	*GetName(void){return my_name;};
	
private:
	Steppable	*action;	
	Steppable 	*achieved_cond;
	SupMgr		*supmgr;	
	char		*my_name;
	unsigned int id;
	static unsigned int next_id;
};


//////////////////////////////////////////////////////////////////////////////
//
//   Multi - PrgStructs
//

class SubUP : public ISobject {
public:
	SubUP(Steppable *a,Steppable *t=NULL)
		{act=a;tst=t;}; 
	Steppable		*act;
	Steppable		*tst;
};

class Parallel : public Action  {
public:
	Parallel(	Steppable *a1=NULL,
				Steppable *a2=NULL,
				Steppable *a3=NULL,
				Steppable *a4=NULL,
				Steppable *a5=NULL,
				Steppable *a6=NULL,
				Steppable *a7=NULL,
				Steppable *a8=NULL,
				Steppable *a9=NULL);
	
	void			Append(Steppable *action_);
	
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
private:
	List	sup_list;		// sub-up list
}; 

class Sequence : public Action  {
public:
	Sequence(Steppable *done_action_=NULL,	
			 Steppable *restart_tst_=NULL,
			 Steppable *kill_tst_=NULL,
			 Steppable *action1=NULL, Steppable	*tc1=NULL,
			 Steppable *action2=NULL, Steppable	*tc2=NULL, 
			 Steppable *action3=NULL, Steppable	*tc3=NULL, 
			 Steppable *action4=NULL, Steppable	*tc4=NULL, 
			 Steppable *action5=NULL, Steppable	*tc5=NULL, 
			 Steppable *action6=NULL, Steppable	*tc6=NULL, 
			 Steppable *action7=NULL, Steppable	*tc7=NULL, 
			 Steppable *action8=NULL, Steppable	*tc8=NULL, 
			 Steppable *action9=NULL, Steppable	*tc9=NULL);
	
	void			Append(Steppable *act_, Steppable *tc_);
	
	virtual	Data	*Step(void);
	virtual	void	Abort(void);
	virtual	void	Reset(void);
private:
	Steppable		*done_action;	// called when all A done
	Steppable		*restart_tst;	// this test, if true, restarts sequence
	Steppable		*kill_tst;		// this test, if true, kills sequence
	List			sup_list;		// sup=up list
	SubUP			*current;		// currently active step
	word			steps;			// counter of executed steps
	boolean			first_done;		// was first test passed? first step done?
}; 

class Prog : public Action  {
public:
	Prog(Steppable *done_action_=NULL,	
			 Steppable *action1=NULL,
			 Steppable *action2=NULL,
			 Steppable *action3=NULL,
			 Steppable *action4=NULL,
			 Steppable *action5=NULL,
			 Steppable *action6=NULL,
			 Steppable *action7=NULL,
			 Steppable *action8=NULL,
			 Steppable *action9=NULL,
			 Steppable *action10=NULL,
			 Steppable *action11=NULL,
			 Steppable *action12=NULL,
			 Steppable *action13=NULL,
			 Steppable *action14=NULL,
			 Steppable *action15=NULL,
			 Steppable *action16=NULL,
			 Steppable *action17=NULL,
			 Steppable *action18=NULL,
			 Steppable *action19=NULL,
			 Steppable *action20=NULL
		); 
	
	void			Append(Steppable *act_);
	
	virtual	Data	*Step(void);
	virtual	void	Abort(void);
	virtual	void	Reset(void);
private:
	Steppable		*done_action;	// called when all A done
	List			sup_list;		// sup=up list
	Steppable		*current;		// currently active step
	word			steps;			// counter of executed steps
}; 

class Once : public Action  {
public:
	Once(Steppable *done_action_=NULL,	
			 Steppable *action1=NULL,
			 Steppable *action2=NULL,
			 Steppable *action3=NULL,
			 Steppable *action4=NULL,
			 Steppable *action5=NULL,
			 Steppable *action6=NULL,
			 Steppable *action7=NULL,
			 Steppable *action8=NULL,
			 Steppable *action9=NULL); 
	
	void			Append(Steppable *act_);
	
	virtual	Data	*Step(void);
	virtual	void	Abort(void);
	virtual	void	Reset(void);
	virtual	Fuzz	*IsDone(void);
private:
	Steppable		*done_action;	// called when all A done
	List			sup_list;		// sup=up list
}; 

typedef int	state_id;
#define STG_GLOBAL_STATEID	 0
#define STG_START_STATEID	-1
#define STG_STOP_STATEID	-2

class STGstate : public ISobject {
friend class StateTGraph;
private:
	STGstate(state_id id_=STG_GLOBAL_STATEID,Steppable *act_=NULL)
		{id=id_;act=act_;};
	state_id	id;		// identification of state
	Steppable	*act;	// action associated with that state
	List		exits;	// transitions exiting from that state
};

class STGtrans : public ISobject {
friend class StateTGraph;
private:
	STGtrans(state_id id_=STG_GLOBAL_STATEID,Steppable *tst_=NULL, STGstate *next_=NULL)
		{id=id_;tst=tst_;next=next_;};
	state_id	id;		// identification of transition
	Steppable	*tst;	// tester of that arc (test for a condition)
	STGstate	*next;	// next state of STG if condition is true
};


class StateTGraph : public Action {	// a state-transition graph / FSM
public:
	StateTGraph(Steppable *start_=NULL, 
				Steppable *stop_=NULL,
				Steppable *a1=NULL,
				Steppable *a2=NULL,
				Steppable *a3=NULL,
				Steppable *a4=NULL,
				Steppable *a5=NULL,
				Steppable *a6=NULL,
				Steppable *a7=NULL,
				Steppable *a8=NULL,
				Steppable *a9=NULL);
	
	void		AppendState(Steppable *news_);
	void		AppendTrans(Steppable *tst_, state_id from_, state_id to_);
		
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
	
	void		ListStates(void);
private:
	STGstate	*FindState(state_id id_);
	Data		*Transit(STGstate *next_s);

	List	states;		// list of the states of the STG
	List	loc_trans;	// list of the transitions (arcs) of the STG
	List	glob_trans;	// list of global transitions (leaving from all states)
	state_id	next_stateid;	// next id that is given to a new state
	state_id	next_transid;	// next id that is given to a new transition
	
	STGstate	*current_state;	// pointer to current state
};


class Loop : public Action  {	// repeat until t is met
public:
	Loop(	Steppable *action, 
			Steppable *cond=NULL, 
			Steppable *done_action=NULL,
			float  threshold_=FUZZTRUE);
		
	virtual	Data	*Step(void);
	virtual	void	Reset(void);
	virtual	void	Abort(void);
private:
	Steppable	*act;
	Steppable	*done_act;
	Steppable	*tst;
	float	threshold;
}; 

//////////////////////////////////////////////////////////////////////////////
//
//   S W I T C H
//

class Switch : public Action {
public:
	Switch(
		float	threshold_=0.1,				// winner must pass this
		Steppable  *default_action_=NULL,
		Steppable *p1=NULL, Steppable *a1=NULL,	// test/action pair candidates
		Steppable *p2=NULL, Steppable *a2=NULL,			
		Steppable *p3=NULL, Steppable *a3=NULL,			
		Steppable *p4=NULL, Steppable *a4=NULL,			
		Steppable *p5=NULL, Steppable *a5=NULL,			
		Steppable *p6=NULL, Steppable *a6=NULL,			
		Steppable *p7=NULL, Steppable *a7=NULL,			
		Steppable *p8=NULL, Steppable *a8=NULL,			
		Steppable *p9=NULL, Steppable *a9=NULL);			
	
	void			Append(Steppable *p, Steppable *a=NULL);
	
	virtual	Data	*Step(void);
	virtual	void	Reset(void);	// use AFTER Step (selection update)
	virtual	void	Abort(void);	// use AFTER Step (selection update)
	virtual	Fuzz	*IsDone(void);	// use AFTER Step (selection update)

private:
	Steppable	*Selector(void);	// do the action selection
	float		threshold;			// propensity must be > than this
	Steppable	*default_action;    // done if no winner > threshold
	List		sup_list;			// sub-UP list
	Steppable	*old_tco;			// previously selected action
};



#endif	/* _PrgStruct_h_ */





















