/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*//* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */


#include "SupMgr.h"

//---- SupMgr ---------------------------------------------------------


SupMgr::SupMgr(OperatorProxy *op_,Scheduler *sched_) 
	: op(op_),sched(sched_),default_dock(NULL)
{
	assert(sched);
	assert(op);
}

SupMgr::~SupMgr()
{
	docklist.Reset();
	Dock *dptr=NULL;
	while (NULL!=(dptr=(Dock*)docklist.Iterate())) delete (Dock*)dptr;
	blocklist.Reset();
	Steppable *bptr=NULL;
	while (NULL!=(bptr=(Steppable*)docklist.Iterate())) delete bptr;
}

Steppable	*SupMgr::AchieveWithBlock(Steppable *cond, Steppable *new_block, char *dock_name)
{
	if (new_block==NULL) return NULL;  // can't accept NULL block
	
	Dock *old_dock=NULL;
	
	if (dock_name)	old_dock = FindDock(dock_name);	// get the Dock
	else 			old_dock = default_dock;		// or use default
	
	if ( ! old_dock) return NULL;		// if no Dock applies return ERROR	
		
	SupBlock *new_sup_block = new SupBlock;
	blocklist.Append(new_sup_block);
	new_sup_block->successor	= old_dock->GetExit();
	new_sup_block->indock 		= old_dock;
	new_sup_block->the_block 	= new_block;
	new_block->SetChain(old_dock->GetExit());	// chain from new block
	old_dock->SetExit(new_block);				// re-route from old Dock
	old_dock->SetAchievedCond(cond);			// never achieved
	if (new_block->GetNoParameters()) {	// if it has parameters, must be TreeBase
		((TreeBase*)new_block)->Commit();
	} else {
		new_block->Reset();
	}
	return new_block;
}

Steppable *SupMgr::InsertBlock(Steppable *new_block, char *dock_name)
{
	return AchieveWithBlock(NULL,new_block,dock_name);
}

Steppable *SupMgr::RemoveBlock(Steppable *rem_block)
{
	assert(rem_block);
	if (! rem_block) return NULL;
	blocklist.Reset();
	SupBlock *cur=NULL;
	while (NULL!=(cur=(SupBlock*)blocklist.Iterate()))	{
		if (cur->the_block==rem_block) {
			if (rem_block->GetNoParameters()) // if it has parameters
				((TreeBase*)rem_block)->Terminate();
			blocklist.Remove(cur);
			assert(cur->indock);					// indock must exist
			cur->indock->SetExit(cur->successor);
			delete cur;
			return rem_block;
		}
	}
	return NULL;
}

Dock *SupMgr::AddDock(Dock *the_dock, char *dock_name, boolean is_default)
{
	assert(dock_name);
	Dock *ex_dock;
	if (NULL!=(ex_dock=FindDock(dock_name))) {
		op->ISERR3(iserr_dupdock, dock_name);
		return ex_dock;
	}
	
	docklist.Append(the_dock);
	if (is_default) default_dock = the_dock; // overwrites old default
	return the_dock;
}

Dock *SupMgr::FindDock(char *dock_name)
{
	if ( ! dock_name) return default_dock; // NULL argument to indicate default_dock
	docklist.Reset();
	Dock *ptr=NULL;
	while (NULL!=(ptr=(Dock*)docklist.Iterate()))
		if (ptr->IsName(dock_name)) return ptr;
	return ptr;
}

void SupMgr::PrintDocks(void)
{
	*op << "\nList of docks\n";
	*op << "--------------------------------\n";
	docklist.Reset();
	Dock *ptr=NULL;
	while (NULL!=(ptr=(Dock*)docklist.Iterate())) {
		*op << (int)ptr->id << " " << ptr->my_name;
		if (ptr==default_dock)	*op << " (DEFAULT)\n";
		else 					*op << "\n";
	}
	*op << "--------------------------------\n\n";
}

