/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _SupMgr_h_
#define _SupMgr_h_

/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include "PrgStruct.h"
#include "OperatorProxy.h"

class SupMgr;
class Scheduler;

class SupBlock : public ISobject {
friend class SupMgr;
private:	
	Steppable	*the_block;	// the block of sups that is insterted
	Dock		*indock;	// the dock that leads into the block
	Steppable	*successor;	// the dock that leads out of block (owner)
};


//---- SupMgr -----------------------------------------------------------

class SupMgr : public ISobject {
public:
	SupMgr(OperatorProxy *op_,Scheduler *sched_);
	~SupMgr();
	
	// Block management functions
	Steppable	*InsertBlock(Steppable *new_block, char *dock_name=NULL);
	Steppable	*RemoveBlock(Steppable *rem_block);
	Steppable	*AchieveWithBlock(Steppable *cond, Steppable *new_block, char *dock_name=NULL);
	
	// Dock management functions
	Dock	*AddDock(Dock *the_dock, char *dock_name, boolean is_default=FALSE);	// returns the new out dock
	Dock	*FindDock(char *dock_name=NULL); // NULL argument returns default_dock
	void	PrintDocks(void);
	
private:
	List	blocklist;	// this manager is OWNER of these sup-blocks
	List	docklist;	// this manager is OWNER of these docks
	Dock	*default_dock; // this is the default dock
	Scheduler *sched; 	// the default scheduler
	OperatorProxy *op;	// operator proxy IO channel
};

#endif	/* _SupMgr_h_ */
