/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _TBehavior_h_
#define _TBehavior_h_

/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include "IFsup.h"
#include <conio.h>
#include <stdio.h>


//---- Detect full circle -----------------------------------------------------------

#define NSECTORS 18
class FullCircleQ : public SensorQuery {
public:
	FullCircleQ(RobotProxy *robot_):SensorQuery(robot_){
		secwidth = 2*PI/(float)NSECTORS;
		Reset();
	};
	
	virtual Data	*Step(void) {
		if (done.True()) return &done;
		else {
			float x,y,a;
			robot->GetDRPosition(x,y,a);
			int secno = (int)(Limit(a+PI,0.0,2*PI)/secwidth);
			if (secno==NSECTORS) secno=0;
			assert((secno>=0)&&(secno<NSECTORS));
			if (!sector[secno]) printf("New sector deflorated: %d\n",secno);
			sector[secno] = TRUE;
			boolean itisdone=TRUE;
			for (int i=0;i<NSECTORS;i++) 
				itisdone = itisdone && sector[i];
			if (itisdone) done.Set(FUZZTRUE);
			return &done;
		}
	};
	virtual	void	Abort(void){
		done.Set(FUZZTRUE);
	}
	virtual void	Reset(void){
		for (int i=0;i<NSECTORS;i++) sector[i]=FALSE;
		done.Set(FUZZFALSE);
	};
private:
	float	secwidth;
	boolean sector[NSECTORS];	
};


//---- Condition -----------------------------------------------------------

class TestTester : public Action {
public:
	TestTester(int no_) {no=no_;};
	
	virtual Data	*Step(void) {
		float tmp;
		printf("\nEnter a test[%d] result [0..1]: ",no);
		fscanf(stdin, "%f",&tmp);
		printf("\nYou entered %f !\n",tmp);
		retval.Set(tmp);
		return &retval;
	};
	
	virtual	void	Abort(void){printf("ABORT tester %d\n",no);};
	virtual void	Reset(void){printf("RESET tester %d\n",no);};
private:
	Fuzz	retval;
	int		no;
};


class TestAction : public Action {
public:
	TestAction(int no_, boolean with_tester_=FALSE, boolean test_result_=TRUE) 
		:no(no_),with_tester(with_tester_),test_result(test_result_)
		{tt=new TestTester(20); done.Set(FUZZTRUE);};

	virtual Data *Step(void) {
		printf("STEP Action %d\n",no);
		return NULL;
	};
	
	virtual Fuzz *IsDone(void) {
		if (with_tester) return (Fuzz*)tt->Step();
		else {
			if (test_result) return &fuzz_true;
			else return &fuzz_false;
		}
	};
	
	virtual	void	Abort(void){printf("ABORT Action %d\n",no);};
	virtual void	Reset(void){printf("RESET Action %d\n",no);};
private:
	TestTester	*tt;
	int	no;	
	boolean with_tester;	// does isdone test?
	boolean test_result;	// of there is no tester, what result do you want
};

#endif	/* _TBehavior_h_ */
