/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*//* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include <sys/time.h>
#include <assert.h>

#ifdef __QNX__
  #include <sys/timeb.h>
  #include <i86.h>
#else 
extern "C" {
	extern int  gettimeofday  (struct timeval * __tp,struct timezone * __tz);
	extern void usleep(unsigned long);
}
#endif

#include "TimeArith.h"

time_t	RealTime::CurrentTime(void)
{
#ifdef __QNX__
        struct timeb timebuf;
        ftime(&timebuf);
        return (timebuf.time*1000+timebuf.millitm);
#else
        struct timeval tp;
        struct timezone tzp;
        
        gettimeofday( &tp, &tzp );

        return(time_t)((tp.tv_sec) * (time_t)1000) + (tp.tv_usec / (time_t)1000);
#endif        
}

void RealTime::Sleep(time_t msecs)
{
#ifdef __QNX__
	// this QNX function takes the number of milliseconds to sleep
	// as an argument
	delay(msecs);
#else
	// this function takes a long as an argument
	// the argument gives the number of microseconds to sleep
	usleep(msecs*1000L);
#endif	
}

StopWatch::StopWatch(boolean start_)
{
	running=FALSE;
	if (start_) Start();
}

void	StopWatch::Start(void)
{
	start_time = CurrentTime();
	running = TRUE;
}

time_t	StopWatch::Stop(void)
{
	running = FALSE;
	return CurrentTime()-start_time;
}

time_t  StopWatch::Lap(void)
{
	return CurrentTime()-start_time;
}

boolean StopWatch::Running(void)
{
	return running;
}

// countdown, very similar to StopWatch
// but different interface

Countdown::Countdown(time_t c_length)
{
	Count(c_length);
}

void	Countdown::Count(time_t c_length)
{
	assert(c_length>=0);
	end_time	= CurrentTime()+c_length;
}

time_t  Countdown::Remaining(void)
{
	time_t rem = end_time-CurrentTime();
	if (rem<=0) {
		return 0;
	} else {
		return rem;
	}
}

boolean Countdown::Expired(void)
{
	return (1>Remaining());
}

