/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#ifndef _Timing_h_
#define _Timing_h_

/* Rene Schaad                                                    */
/* Departement of Computer Science, Univ. of Zurich, Switzerland  */
/* schaad@ifi.unizh.ch                                            */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include "defines.h"

typedef void (*alarm_cb_t)(int);

//---- Timing -----------------------------------------------------------

class Timing {

public:

	Timing();
	~Timing();
	
	// create an alarm based watchdog (sending a signal)
	void	StartDog(unsigned int secs_);
	void	ResetDog(void);
	void 	StopDog(void);
	void	RegisterDogFunction(alarm_cb_t alarm_cb);
	
	// create a heartbeat (sending a proxy every so often)
	pid_t	CreateHeartbeat(void);
	void	SuspendHeartbeat(void);
	void	ResumeHeartbeat(void);
	void	StopHeartbeat(void);
	
	// create a timer based watchdog (sending a proxy)
	pid_t	CreateCommWatchdog(long comm_timeout_secs, long comm_timeout_nsecs);
	void	ResetCommWatchdog(void);
	void	StopCommWatchdog(void);
	
	static boolean  	do_tick;
	static pid_t		timer_proxy;
private:
	// stuff for software watchdog
	unsigned int watchdog_secs;
	
	// stuff for heartbeat
	int 				intid;
	
	// stuff for timer based watchdog (sending a proxy)
	pid_t 		c_watchdog_proxy;
	timer_t 	comm_watchdog;
	itimerspec 	c_timer_val_stop;
	itimerspec 	c_timer_val_start;
};

#endif	/* _Timing_h_ */
