/*

DISCLAIMER 

InSitu is provided as is, without any kind of guarantees or warranty. There is 
no documentation. The author does not provide any kind of support to users of 
this source code and is not responsible for any kind of problems or damages 
resulting from its use. 

The source code is the result of a long iterative design process for a research 
prototype and was never intended to be used in any kind of product. In general, 
the code cannot be expected to compile in any straightforward manner on your 
computer system, since the robot on which it was developed, has a custom design. 

*/

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <math.h>
#include <windows/Qwindows.h>
#include <graph.h>
#include <iostream.h>
#include <i86.h>
#include <sys/sched.h>
#include "TimeArith.h"
#include "CImage.h"
#include "FrameGrabber.h"
#include "RobotProxy.h"
#include "Timing.h"
#include "Packet.h"
#include "LocalMap.h"
#include "TBehavior.h"
#include "SupMgr.h"
#include "../include/Argscan.h"
#include "Interpreter.h"
#include "OperatorProxy.h"
#include "IFsup.h"
#include "ArithSup.h"
#include "Navsup.h"
#include "Graph.h"
#include "Monsup.h"
#include "APsup.h"

#define USAGE             " "
#define N_CMDARGS         0
#define PROCESS_PRIORITY     10				/* QNX process priority */ 

char *hiwaymap_name = "global_map";
pid_t sid;
int wi,pi;
boolean with_graphics=FALSE;

unsigned int old_colors[16][3]; /* 16 colors, with rgb */
unsigned int std_colors[16][3] = {
	0 		, 0 	, 0,
	6562 	, 6562 	, 6562,
	5625 	, 5625 	, 5625,
	3242 	, 3242 	, 3242,
	9843 	, 9843 	, 9843,
	4492 	, 7148 	, 6367,
	3242 	, 5000 	, 3749,
	0 		, 6562 	, 0,
	9843 	, 9843 	, 6562,
	9843 	, 9843 	, 0,
	8593 	, 8358 	, 7695,
	6562 	, 3242 	, 6562,
	9959 	, 6405 	, 6405,
	9959 	, 0 	, 0,
	0 		, 0 	, 7500,
	4179 	, 5272 	, 9843
};

////////////////////////////////////////////////////////////////////////
//   EXCEPTION HANDLERS


void handle_exit(void)
{
	cerr << "handle_exit called \n";
	if (with_graphics)
		for (int k=1;k<=16;k++) 
			ScreenColor(k,std_colors[k-1][0],std_colors[k-1][1],std_colors[k-1][2],"","");
}


void handle_alarm (int sig)
{
	cout << "ALARM!! (Signal ";
	cout << sig << ")\n";
	exit(0);
}

void handle_sigdefault (int sig)
{
	switch (sig) {
	  default:
			fprintf(stderr, "received signal %d\n", sig);
			break;
	}
	exit(0);
}

//////////////////////////////////////////////////////////////////////////////////////////
// Assign UNIX & QNX signal intercept functions

void InitSignals(void)
{
  // Attach signal and exit handlers

  atexit( handle_exit );              /* catch exit call */
  signal( SIGINT , handle_sigdefault);    /* catch SIGINT (control-c) */
  signal( SIGALRM , handle_sigdefault); /* catch SIGALARM */
  signal( SIGTERM , handle_sigdefault);  /* catch SIGTERM */ 

  // catch the rest of the signals to call exit
  signal( SIGABRT , handle_sigdefault); 
  signal( SIGFPE , handle_sigdefault); 
  signal( SIGHUP , handle_sigdefault); 
  signal( SIGILL , handle_sigdefault); 
  signal( SIGKILL , handle_sigdefault); 
  signal( SIGPIPE , handle_sigdefault); 
  signal( SIGQUIT , handle_sigdefault); 
  signal( SIGSEGV , handle_sigdefault); 
  signal( SIGUSR1 , handle_sigdefault); 
  signal( SIGUSR2 , handle_sigdefault); 
  signal( SIGCONT , handle_sigdefault); 
  signal( SIGSTOP , handle_sigdefault);
  // QNX - specific signals 
  signal( SIGBUS , handle_sigdefault); 
  signal( SIGDEV , handle_sigdefault); 
  signal( SIGPWR , handle_sigdefault); 
  // old unix signals (never actually generated)
  signal( SIGIOT , handle_sigdefault); 
  signal( SIGSYS , handle_sigdefault); 
  signal( SIGWINCH , handle_sigdefault); 
  signal( SIGURG , handle_sigdefault); 
  signal( SIGPOLL , handle_sigdefault); 
  signal( SIGEMT , handle_sigdefault); 
  signal( SIGTRAP , handle_sigdefault); 
}

void InitSystem(boolean graphics)
{
	// initialize signals
	InitSignals();
	
	// set process priority
	if (setprio(0,PROCESS_PRIORITY) == -1) {  // set priority
		cerr << "robotd: could not set preferred process priority: " << errno << "\n";
		exit(1);
	}	

	with_graphics = graphics;
	if (with_graphics) {
	// open graphics system
		if (NULL==(sid = GraphicsOpen(NULL)) )
		{
			printf("For use with QNX windows only.\n");
			exit(-1);
		} 

		// Initialize graphics windows
		DrawStart(NULL,64000);
		DialogAt(0,0,NULL,NULL);   /* position the window in the upper left corner */
		Dialog("sap","Frame Grabber Image","@/home/schaad/plascom/behd/grab.wnd",
			NULL,"@/home/schaad/plascom/behd/grab.pict","-b");
		DialogId(&wi,&pi);
		PictureCurrent(pi);
		WindowCurrent(wi);

		// save color table and generate new color table
		for (int k=1;k<=16;k++) {
			ColorToRGB(k,&old_colors[k-1][0],&old_colors[k-1][1],&old_colors[k-1][2]);
		}
		for (k=1;k<=16;k++) {
			ScreenColor( k,10000/16*k,10000/16*k,10000/16*k, "", "");
		}

		// set highest gray level to color red
		//ScreenColor( 16,10000,0,0, "", "");
	}
}


////////////////////////////////////////////////////////////////////////
//   Global System Structures
	
// this is for execution
StopWatch	sw;
CImage		tmp_image;
CImage		out_image;

// Allocate SUP services, frame grabber and robot hardware
RobotProxy	rufus;					// create robot connection
Scheduler	sched(100);				// create run-time manager
OPremote 	opr;					// create link to operator
SupMgr		mgr(&opr,&sched);		// create maneger of SUPs
Parser		pars(&rufus,&opr,&mgr);	// create a InSitu parser
LocalMap	lm(&rufus);				// create a local mapping

////////////////////////////////////////////////////////////////////////
//   SUP Library
	
HiwayMap 	the_map("global_map");

Waypoint 	start_cal1(10.5,-4.3,0.0,FALSE);// start of recalib
Waypoint	stop_cal1( 24.0,-4.3,0.0,FALSE); // stop of recalib
Waypoint 	start_sonar_cal1(10.0,-4.3,0.0,TRUE);// start of recalib
Waypoint	stop_sonar_cal1( 14.0,-4.3,0.0,FALSE); // stop of recalib
Number		wall_att1(PI2);					// attitude to face wall
Wall 		wall_cal1(9,-3,24,-3);			// wall to recalibrate to
WallEstimate west_cal1(ANYCOORD,-3,0.5);		// estimation of wall

Waypoint 	start_cal2(22.0,-2.5,0.0,FALSE);// start of recalib
Waypoint 	stop_cal2(22.0,-16.4,0.0,FALSE); // stop of recalib
Waypoint 	start_sonar_cal2(22.0,-12.4,0.0,TRUE);// start of recalib
Waypoint 	stop_sonar_cal2(22.0,-16.4,0.0,FALSE); // stop of recalib
Number		wall_att2(0.0);					// attitude to face wall
Wall 		wall_cal2(23.3,-2.5,23.3,-17.0);// wall to recalibrate to
WallEstimate west_cal2(23.3,ANYCOORD,0.5 );	// estimation of wall
	
AchieveVel	stop_it(&rufus,0.0,0.0);		// stop robot
Here 		here(&rufus);					// where am I
Waypoint 	home(7.71,-1.01);				// no place like..
RefuelQ 	mon_fuel(&rufus);				// propensity to refuel
RecalibQ 	mon_dr(&rufus,&start_cal1,75); // propensity to recal

////////////////////////////////////////////////////////////////////////
//   MAIN FUNCTION
	
int main(int argc, char ** argv) 
{
	// do argument checking
	SHOW_USAGE_TEST( exit(1), N_CMDARGS, USAGE ); 
	//	SCAN_STR_ARGV( exit(1), 1, cmd_file_name );   
	InitSystem(TRUE);
	rufus.UpdateSensors();
		
	
//Steppable *testable = 
//	new Register(
//		new Sequence(
//			new SetVel(&rufus,0.0,0.0),
//			NULL,NULL,
//			new GoAlongRay(&rufus,new ExtendRay(&rufus,new Number(PI4))),
//			new BlockedQ(&rufus,0.6),
//			new FollowWall(&rufus,0.15,0.6),
//			new EdgeDetected(&rufus),
//			new WaypointToFloorPoint(new Here(&rufus)),	// output this thing
//			new DoTimes(1)
//		),
//		new DoInterval(5000)
//	);
	

	/////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////
	//   
	//    FINAL SUPs
	//
		
	////////////////////////////////////////////////////////////////////////
	//   Unbump 
	
	Variable 	bumpback;  	// did I bump in front ? (Fuzz)
	Variable 	bumpleft;	// did I bump on left ? (Fuzz)
	
	TreeBase *unbump = 
		new TreeBase(
			NULL,
			new Once(
				new Prog(
					&stop_it,
					new If(&bumpback,						
							new Moveby(&rufus,0.1),		// back up 0.1 m
							new Moveby(&rufus,-0.1)	
						),							
					new If(&bumpleft,						
							new Turnby(&rufus,-PI2),	// turn away 90 deg.
							new Turnby(&rufus, PI2)
						)
				),
				new Setq(&bumpback, 						// bumpted front/back?
					new Or(
						new BumpedQ(&rufus,4),
						new BumpedQ(&rufus,5),
						new BumpedQ(&rufus,6)
					)
				),
				new Setq(&bumpleft, 						// bumped left/right?
					new Or(
						new BumpedQ(&rufus,5),
						new BumpedQ(&rufus,6),
						new BumpedQ(&rufus,7),
						new BumpedQ(&rufus,8)
					)
				)
			),
			NULL
		);
	pars.AppendTree("Unbump",unbump);
	assert(unbump);
		
	/////////////////////////////////////////////////////////////////
	//   Avoid obstacles  (using bumpers, virtual antlers and sonars)
	
	PAR(not_avoiding_tree);
		
	TreeBase *while_avoiding = 
		new TreeBase(
			NULL,
			new StickyIf(
				new BumpedQ(&rufus),			// if bumped
				new Parallel(
					new Debug("    unbumping"),
					unbump							// unbump!
				),
				new If(							// if in forbidden area go out
					new Not(new AvoidForbidden(&rufus,&the_map)),
					new If(						// if obstacles, avoid
						new Not(new SonarAvoid(&rufus,0.13)),
						not_avoiding_tree,
						new Debug("      avoiding SONAR")
					),
					new Debug("    avoiding FORBIDDEN ZONE")
				)
			),
			NULL,
			not_avoiding_tree
		);
	pars.AppendTree("WhileAvoiding",while_avoiding);
	assert(while_avoiding);

	/////////////////////////////////////////////////////////////////
	//   Stoping for obstacles  (using bumpers, virtual antlers and sonars)
	
	PAR(not_stopping_tree);
		
	TreeBase *while_stopping = 
		new TreeBase(
			NULL,
			new TimedIf(
				new BlockedQ(&rufus),
				new TimeinAbort(&stop_it,20000,2000,not_stopping_tree), 
				2000,
				not_stopping_tree,				
				TRUE							
			),
			NULL,
			not_stopping_tree
		);
	pars.AppendTree("WhileStopping",while_stopping);
	assert(while_stopping);

	/////////////////////////////////////////////////////////////////
	//   Go loco
		
	Variable route_loco;
	PAR(location);
	
	Steppable *go = new Goto(&rufus,new IterateRoute(&rufus,&route_loco));
		
	TreeBase *go_loco = 
		new TreeBase(
			NULL,
			new Prog(
				&stop_it,
				new Setq(
					&route_loco,
					new PlanARoute(
						&the_map,
						new Here(&rufus),	// gimme waypoint of THIS location
						location
					)
				),
				// move to home
				new Parallel(
					new Withparam(while_avoiding,not_avoiding_tree,go),
					new IsDoneStep(go)
				),
				new Say(&rufus,"Achieved goal location."),
				new Debug("Achieved goal location")
			),
			NULL,
			location
		);
	pars.AppendTree("Goloco",go_loco);
	assert(go_loco);	
	
	
	/////////////////////////////////////////////////////////////////
	//   Define home & go_home or ISS
	
	TreeBase *home_iss = new TreeBase(NULL,&home,NULL);
	pars.AppendTree("Home",home_iss);
	assert(home_iss);

	TreeBase *go_home = new TreeBase(NULL,new Withparam(go_loco,location,&home));
	pars.AppendTree("Gohome",go_home);
	assert(go_home);
	
	/////////////////////////////////////////////////////////////////
	//   Misc. useful trees
	
	PAR(turnby_arg);
	TreeBase *turnby = new TreeBase(NULL,new Turnby(&rufus,turnby_arg),NULL,turnby_arg);
	pars.AppendTree("Turnby",turnby);
	assert(turnby);
	
	
	/////////////////////////////////////////////////////////////////
	//   Recalibrate with walls
	
	PAR(start_cal);
	PAR(stop_cal);
	PAR(wall_cal);
	PAR(west_cal);
		
	Steppable *along_wall = 
		new Prog(
			&stop_it,
			new ResetWallEstimate(west_cal),
			new Parallel(
				//new FollowWall(&rufus,0.15,0.95,LEFT_WALL),
				//new EdgeDetected(&rufus)
				new Goto(&rufus,stop_cal),
				new EstimateWall(&rufus,3,west_cal) // use left sonar to estimate
			),
			&stop_it,
			new UpdateDRPWall(&rufus,west_cal,wall_cal),	// do the calibration,
			//new SetDist(&rufus,0,-1),
			new Say(&rufus,"Updating a wall.")
		);
			
	TreeBase *recalib = 
		new TreeBase(
			NULL,
			new Prog(
				&stop_it,									// stop when done
				new Withparam(go_loco,location,start_cal),	// goto starting point
				&stop_it,									// stop
				new Turnto(&rufus,stop_cal),				// orient along wall
				new Sequence(
					&stop_it,								// stop when done
					NULL,									// no restart test
					new IsDoneStep(along_wall),				// stop when walking is done
					new Withparam(while_stopping,not_stopping_tree,along_wall),
					&fuzz_false
				),
				&stop_it
			),
			NULL,
			start_cal,
			stop_cal,
			wall_cal,
			west_cal
		);
	pars.AppendTree("Recalib",recalib);
	assert(recalib);
	
	Steppable *recalib1 = new Withparam(recalib,
		start_cal,&start_sonar_cal1,
		stop_cal,&stop_sonar_cal1,
		wall_cal,&wall_cal1,
		west_cal,&west_cal1);
	Steppable *recalib2 = new Withparam(recalib,
		start_cal,&start_sonar_cal2,
		stop_cal,&stop_sonar_cal2,
		wall_cal,&wall_cal2,
		west_cal,&west_cal2);
		
	Steppable *do_recalib = 
		new TimeinReset(
			new Prog(
				&stop_it,
				recalib1,
				recalib2,
				new SetDist(&rufus,0,0)
			),
			800000,20000);
				

	/////////////////////////////////////////////////////////////////
	//   Recalibrate with bumped walls
	
//	PAR(bump_pt);
//	PAR(bump_att);
//	PAR(bump_west);
//		
//	TreeBase *bump_wall_pt = 
//		new TreeBase(
//			NULL,
//			new Prog(
//				&stop_it,
//				new Withparam(go_loco,location,bump_pt),
//				new TurntoAttitude(&rufus,bump_att),
//				new Sequence(NULL,NULL,NULL,
//					new GoAlongRay(&rufus,new ExtendRay(&rufus,new Number(0)),0.05),
//					new BumpedQ(&rufus)
//				),
//				new Parallel(
//					new EstimateBumpWall(&rufus,bump_west), // use left sonar to estimate
//					new SetVel(&rufus,0.0,0.0),
//					new Say(&rufus,"Oops.")
//				),
//				new Moveby(&rufus,-0.1),
//				new Turnby(&rufus,PI2),
//				new Say(&rufus,"Phase completed.")
//			),
//			NULL,
//			bump_pt,	// point from which I start bumpin'
//			bump_att,	// attitude to confront wall
//			bump_west	// estimate for wall position
//		);
//	pars.AppendTree("BumpWallPt",bump_wall_pt);
//	assert(bump_wall_pt);
//			
//	PAR(bump_pt_1);
//	PAR(bump_pt_2);	
//	PAR(bump_wall);	
//		
//	TreeBase *bump_wall_seq = 
//		new TreeBase(
//			NULL,
//			new TimeinReset(
//				new Prog(
//					&stop_it,
//					new ResetWallEstimate(bump_west),
//					new Withparam(bump_wall_pt,bump_pt,bump_pt_1),
//					new Withparam(bump_wall_pt,bump_pt,bump_pt_2),
//					new UpdateDRPWall(&rufus,bump_west,bump_wall),	// do the calibration,
//					new Say(&rufus,"Done calibrating.\n")
//				),
//				600000, // 10 minues time to do this, then abort and stop
//				20000	// need to be doing something else for 20 secs to reset
//			),
//			NULL,
//			bump_pt_1,
//			bump_pt_2,
//			bump_att,
//			bump_west,
//			bump_wall
//		);
//	pars.AppendTree("BumpWallSeq",bump_wall_seq);
//	assert(bump_wall_seq);
//	
//	Steppable *bump_wall1=
//		new Withparam(bump_wall_seq,
//			bump_pt_1,&start_cal1,
//			bump_pt_2,&stop_cal1,
//			bump_att,&wall_att1,
//			bump_west,&west_cal1,
//			bump_wall,&wall_cal1);
//	Steppable *bump_wall2=
//		new Withparam(bump_wall_seq,
//			bump_pt_1,&start_cal2,
//			bump_pt_2,&stop_cal2,
//			bump_att,&wall_att2,
//			bump_west,&west_cal2,
//			bump_wall,&wall_cal2);
//			
//	Steppable *do_recalib_bump = 
//		new TimeinReset(
//			new Prog(
//				&stop_it,
//				bump_wall1,
//				bump_wall2,
//				new SetDist(&rufus,0,0)
//			),
//			800000,20000);
				
			

	/////////////////////////////////////////////////////////////////
	// Monitor environment and track motion
	
	MonitorCam *moncam = new MonitorCam(&rufus);
	assert(moncam);
	
	Steppable *mon_and_track =
		new StickyIf(
			new Gt(new Abs(new PanQ(&rufus)),new Number(0.7)),
			new Prog(NULL,
				new Debug("---=== turning base to keep in range"),
				new Turnby(&rufus,new Mul(new PanQ(&rufus),new Number(0.5))),
				new SetCam(&rufus,0.0,0.0)
			),
			new Loop(
				new Prog(NULL,
					new Debug("---=== tracking target"),
					new DoInterval(620),
					moncam,
					new DoInterval(50),
					new If(
						new RegisteredQ(new ShowVisual(new SaccadeTo(&rufus,moncam),50,50)),
						new Report(&opr,new Text("Tracked intruder!\n")),
						new Report(&opr,new Text("No-one detected.\n"))
					)
				)
			)
		);
	assert(mon_and_track);
		
	Steppable *mt_a_while =
		new Sequence(
			&stop_it, NULL,NULL,
			mon_and_track,
			new DoInterval(45000),
			new SetCam(&rufus,0.0,0.0),
			new DoTimes(2)
		);
	assert(mt_a_while);
		
	
	/////////////////////////////////////////////////////////////////
	//  Move to random location
	
	Variable random_route;	// a randomly generated route
	
	Steppable *goto_random =
		new Goto(&rufus,new IterateRoute(&rufus,&random_route));
		
	Steppable *achieve_random =
		new Prog(
			&stop_it,		// stop robot at very end
			&stop_it,		// stop robot at start
			new Setq(		// plan a new route to random location
				&random_route,
				new PlanARandomRoute(&the_map,&here)
			),
			new Debug("  done planning a route, now going there"),
			new Sequence(	 		// do until location achieved
				&stop_it,			// this should not be reached
				NULL,NULL,			
				new StickyIf(
					new BumpedQ(&rufus),			// if bumped
					new Parallel(
						new Debug("    unbumping"),
						unbump							// unbump!
					),
					new If(							// if in forbidden area go out
						new Not(new AvoidForbidden(&rufus,&the_map)),
						new If(						// if obstacles, avoid
							new Not(new SonarAvoid(&rufus,0.20)),
							goto_random,
							new Debug("      avoiding SONAR")
						),
						new Debug("    avoiding FORBIDDEN ZONE")
					)
				),
				new IsDoneStep(goto_random)
			),
			new Debug("  location achieved")
		);
		

	/////////////////////////////////////////////////////////////////
	//  Walk / Monitor Loop
	
	Steppable *mon_loop =
		new Loop(
			new Prog(
				&stop_it,
				new Debug("starting to wander randomly"),
				achieve_random,
				new Debug("starting to monitor a while"),
				new Turnby(&rufus,PI),
				new Debug("turning around"),
				mt_a_while,
				new Debug("done monitoring")
			)
		);
		
	/////////////////////////////////////////////////////////////////
	//  Behavior Arbitration
	
	Steppable *top_propensity_select =
		new Switch(
			0.8,		mon_loop,
			&mon_dr,	do_recalib,
			&mon_fuel,	go_home
		);
	
	/////////////////////////////////////////////////////////////////
	//   Base dock
	
	Steppable *base_dock = new Dock(&mgr,"Base",mon_and_track,TRUE);
	assert(base_dock);
	
	///////////////////////////////////////////////
	// 
 	//  Install everything and run it
	// 
	sched.Add(SENSOR_PP,&pars);
	sched.Add(SENSOR_PP,&rufus);
	sched.Add(SENSOR_PP, new DrawAll());
	sched.Add(UP_PP, base_dock);
	
	sched.Run();
	
	return 0;
}

